package cn.superdesk.uniorder.common.mq.kafka.producer;


import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.dto.receive.consumer.superdeskmall.SuperMallSupplierReceiveParam;
import cn.superdesk.uniorder.dto.receive.consumer.superdeskmall.proxyclient.ProxyClientOrderReceiveParam;
import cn.superdesk.uniorder.dto.receive.consumer.superdeskmall.score.ScoreOrderReceiveParam;
import cn.superdesk.uniorder.dto.receive.consumer.superdeskmall.score.refund.ScoreOrderRefundReceiveParam;
import cn.superdesk.uniorder.dto.receive.consumer.superdeskmall.unified.UnifiedOrderReceiveParam;
import cn.superdesk.uniorder.dto.receive.consumer.superdeskmall.unified.refund.UnifiedOrderRefundReceiveParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Service;


/**
 * @Description: Java类作用描述
 * @Author: linjinyu
 * @CreateDate: 2021/3/8 上午11:40
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021/3/8 上午11:40
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Service
public class FakeSuperdeskMallProducerHandler {

	@Autowired
	private KafkaTemplate<String, String> kafkaTemplate;


	/****************************************** 超级前台 start ******************************************/

	@Value(value = "${superdesk.mall.kafka.topic.order.main}")
	private String SUPERDESK_MALL_KAFKA_TOPIC_ORDER_MAIN;
	@Value(value = "${superdesk.mall.kafka.topic.order.refund}")
	private String SUPERDESK_MALL_KAFKA_TOPIC_ORDER_REFUND;

	public void fakeProducerUnifiedOrder(UnifiedOrderReceiveParam unifiedOrderReceiveParam) {
		kafkaTemplate.send(SUPERDESK_MALL_KAFKA_TOPIC_ORDER_MAIN, JsonUtils.toJson(unifiedOrderReceiveParam));
	}

	public void fakeProducerUnifiedOrderRefund(UnifiedOrderRefundReceiveParam unifiedOrderRefundReceiveParam) {
		kafkaTemplate.send(SUPERDESK_MALL_KAFKA_TOPIC_ORDER_REFUND, JsonUtils.toJson(unifiedOrderRefundReceiveParam));
	}

	/****************************************** 超级前台   end ******************************************/


	/****************************************** 代客下单 新通道 start ******************************************/

//	proxyclient_order
//	proxyclient_order_refund

	//	ProxyClientOrder
//	proxyclient_order_refund

	@Value(value = "${superdesk.mall.kafka.topic.order.proxyclient}")
	private String SUPERDESK_MALL_KAFKA_TOPIC_ORDER_PROXY_CLIENT;
//	@Value(value = "${superdesk.mall.kafka.topic.order.proxyclient.refund}")
//	private String SUPERDESK_MALL_KAFKA_TOPIC_ORDER_PROXY_CLIENT_REFUND;

	public void fakeProducerProxyClientOrder(ProxyClientOrderReceiveParam scoreOrderReceiveParam) {
		kafkaTemplate.send(SUPERDESK_MALL_KAFKA_TOPIC_ORDER_PROXY_CLIENT, "order", JsonUtils.toJson(scoreOrderReceiveParam));
	}

//	public void fakeProducerProxyClientOrderRefund(ProxyClientOrderRefundReceiveParam scoreOrderRefundReceiveParam) {
//		kafkaTemplate.send(SUPERDESK_MALL_KAFKA_TOPIC_ORDER_PROXY_CLIENT_REFUND, "refund", JsonUtils.toJson(scoreOrderRefundReceiveParam));
//	}


	/****************************************** 侨城汇 start ******************************************/

	@Value(value = "${superdesk.mall.kafka.topic.order.score}")
	private String SUPERDESK_MALL_KAFKA_TOPIC_ORDER_SCORE;
	@Value(value = "${superdesk.mall.kafka.topic.order.score.refund}")
	private String SUPERDESK_MALL_KAFKA_TOPIC_ORDER_SCORE_REFUND;

	public void fakeProducerScoreOrder(ScoreOrderReceiveParam scoreOrderReceiveParam) {
		kafkaTemplate.send(SUPERDESK_MALL_KAFKA_TOPIC_ORDER_SCORE, "order", JsonUtils.toJson(scoreOrderReceiveParam));
	}

	public void fakeProducerScoreOrderRefund(ScoreOrderRefundReceiveParam scoreOrderRefundReceiveParam) {
		kafkaTemplate.send(SUPERDESK_MALL_KAFKA_TOPIC_ORDER_SCORE_REFUND, "refund", JsonUtils.toJson(scoreOrderRefundReceiveParam));
	}

	/****************************************** 侨城汇   end ******************************************/


	/****************************************** 企业采购-商户(供应商)数据 start ******************************************/

	public void fakeProducerSupplier(SuperMallSupplierReceiveParam superMallSupplierReceiveParam) {
		kafkaTemplate.send("supplierSync", "merchant", JsonUtils.toJson(superMallSupplierReceiveParam));
	}
	/****************************************** 企业采购-商户(供应商)数据 start ******************************************/


}
