package cn.superdesk.uniorder.common.numerical;

import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * @Description: 超级前台 业务处理 StringUtils
 * @Author: linjinyu
 * @CreateDate: 5/28/20 5:58 PM
 * @UpdateUser: linjinyu
 * @UpdateDate: 5/28/20 5:58 PM
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@NoArgsConstructor
@Slf4j
public class AnStringUtils {

	/**
	 * 获取UUID，去掉了-
	 *
	 * @return
	 * @author xuzhixiang
	 * @date 2017年9月24日16:16:11
	 */
	public static String getUUID() {
		String uuid = UUID.randomUUID().toString().replaceAll("-", "");
		log.debug("获取32位的UUID的调试日志-->> {}", uuid);
		return uuid;
	}


	/**
	 * usage : List<Long> searchVirtIds = AnStringUtils.strArrayConvertToLongList(event.getVirtIds(), ";");
	 *
	 * @param ids
	 * @param splitChat
	 * @return
	 */
	public static List<Long> stringArrayConvertToLongList(String ids, String splitChat) {


		if (StringUtils.isNotBlank(ids)) {
			String[] idArray = ids.split(splitChat);
			return Arrays.stream(idArray)
					.filter(s -> StringUtils.isNotBlank(s))
					.distinct()
					.map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
		} else {
			return new ArrayList<>();
		}
	}

	public static List<String> stringArrayConvertToStringList(String ids, String splitChat) {


		if (StringUtils.isNotBlank(ids)) {
			String[] idArray = ids.split(splitChat);
			return Arrays.stream(idArray)
					.map(s -> s.trim()).collect(Collectors.toList());
		} else {
			return new ArrayList<>();
		}
	}

	/**
	 * @param input
	 * @return
	 */
	public static String stringListConvertToString(List<String> input) {

		String output = "";

		if (AnCollectionUtils.isNotEmpty(input)) {
			output = input.stream().collect(Collectors.joining(","));
		}

		return output;
	}

	public static String longListConvertToString(List<Long> input) {

		String output = "";

		if (AnCollectionUtils.isNotEmpty(input)) {
			output = input.stream().map(s -> getString(s)).collect(Collectors.joining(","));
		}

		return output;
	}

	public static String getString(Object object) {

		if (null == object) {
			return "";
		}
		return object.toString().trim();

	}

	/****************************************** 数据脱敏 start ******************************************/
	/**
	 * 数据脱敏 手机号中间4位替换成*
	 * 13444444444 -> 134****4444
	 *
	 * @param telphoneNumber
	 * @return
	 */
	public static String vagueTelphoneNumber(String telphoneNumber) {

		String telphoneNumberVague = telphoneNumber.replaceAll("(\\d{3})\\d{4}(\\d{4})", "$1****$2");
		return telphoneNumberVague;
	}

	/**
	 * 数据脱敏 身份证号中间10位替换成*
	 *
	 * @param idCardNumber
	 * @return
	 */
	public static String vagueIdCardNumber(String idCardNumber) {
		String idCardNumberVague = idCardNumber.replaceAll("(\\d{4})\\d{10}(\\w{4})", "$1****$2");
		return idCardNumberVague;
	}

	/**
	 * 数据脱敏 去除真实姓名敏感信息
	 *
	 * @param realName
	 * @return
	 */
	public static String vagueRealName(String realName) {

		if (StringUtils.isNotBlank(realName)) {
			if (realName.length() > 1) {
				if (realName.length() == 2) {
					realName = realName.replaceAll("(\\w)(\\w)(\\w)", "$1*");
				}
				if (realName.length() > 2) {
					realName = realName.substring(0, 2) + "*";
				}
			}
		}
		return realName;


	}

	/**
	 * 数据脱敏 Email
	 *
	 * @param email
	 * @return
	 */
	public static String vagueEmail(String email) {
		String emailVague = email.replaceAll("(\\w+)\\w{5}@(\\w+)", "$1***@$2");
		return emailVague;
	}

	/****************************************** 数据脱敏   end ******************************************/
}




