package cn.superdesk.uniorder.dubbo.provider.accounting;

import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.enums.common.CommonSoftDeleteEnum;
import cn.superdesk.uniorder.common.enums.statement.AccountingOrderAccountingStatusEnum;
import cn.superdesk.uniorder.common.enums.statement.AccountingOrderBackAccountStatusEnum;
import cn.superdesk.uniorder.common.enums.statement.AccountingOrderInvoiceStatusEnum;
import cn.superdesk.uniorder.dao.entity.accounting.score.customer.AccountingScoreCustomer;
import cn.superdesk.uniorder.dao.entity.accounting.score.customer.AccountingScoreCustomerXCredit;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dao.entity.sys.notify.SysNotify;
import cn.superdesk.uniorder.dto.param.accounting.score.customer.AccountingScoreCustomerCreateParam;
import cn.superdesk.uniorder.dto.vo.accounting.score.customer.AccountingScoreCustomerDetailVo;
import cn.superdesk.uniorder.dto.vo.accounting.score.customer.AccountingScoreCustomerHeadVo;
import cn.superdesk.uniorder.dto.vo.accounting.score.customer.AccountingScoreCustomerVo;
import cn.superdesk.uniorder.dto.vo.score.ScoreOrderReconciliationNotYetVo;
import cn.superdesk.uniorder.dubbo.facade.accounting.DubboAccountingScoreCustomerService;
import cn.superdesk.uniorder.dubbo.facade.sys.notify.DubboSysNotifyService;
import cn.superdesk.uniorder.service.accounting.score.customer.AccountingScoreCustomerExportExcelService;
import cn.superdesk.uniorder.service.accounting.score.customer.AccountingScoreCustomerService;
import cn.superdesk.uniorder.service.accounting.score.customer.AccountingScoreCustomerXCreditService;
import cn.superdesk.uniorder.service.base.ScoreOrderService;
import cn.superdesk.uniorder.common.lamda.AnLamdaUtils;
import com.alibaba.dubbo.config.annotation.Reference;
import com.alibaba.dubbo.config.annotation.Service;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;


/**
 * @Description: Java类作用描述
 * @Author: linjinyu
 * @CreateDate: 6/28/20 11:43 AM
 * @UpdateUser: linjinyu
 * @UpdateDate: 6/28/20 11:43 AM
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Service(version = "1.0.0", retries = -1, timeout = 30000)
public class DubboAccountingScoreCustomerServiceImpl implements DubboAccountingScoreCustomerService {

	@Autowired
	private AccountingScoreCustomerService        accountingScoreCustomerService;
	@Autowired
	private AccountingScoreCustomerXCreditService accountingScoreCustomerXCreditService;

	@Autowired
	private ScoreOrderService scoreOrderService;

	/****************************************** export excel start ******************************************/
	@Autowired
	private AccountingScoreCustomerExportExcelService accountingScoreCustomerExportExcelService;

	/****************************************** 引用自身 Dubbo provider start ******************************************/
	@Reference(version = "1.0.0", timeout = 10000, retries = -1)
	private DubboSysNotifyService dubboSysNotifyService;


	/****************************************** 积分对账(结算) 对账单 start ******************************************/

	public List<Long> getAccountingXCreditIds(Long id) {

		Map<String, Object> setAccountingParams = new HashMap<>();

		return accountingScoreCustomerService.getAccountingXCreditIds(id);
	}

	/**
	 * 积分对账(结算) 对账单 列表数据 带分页
	 *
	 * @param params
	 * @param pageNum
	 * @param pageSize
	 * @return
	 */
	public R<List<AccountingScoreCustomerVo>> getAccountingList(Map<String, Object> params, int pageNum, int pageSize) {
		return accountingScoreCustomerService.getAccountingList(params, pageNum, pageSize);
	}

	/**
	 * 积分对账(结算) 对账单 列表数据
	 *
	 * @param params
	 * @param loginSysUser
	 * @return
	 */
	public List<AccountingScoreCustomerVo> getAccountingList(Map<String, Object> params, LoginSysUser loginSysUser) {
		return accountingScoreCustomerService.getAccountingList(params, loginSysUser);
	}

	/**
	 * 积分对账(结算) 对账单 页头 结算单数： 399 订单数： 105770 结算金额： 11859292.39
	 *
	 * @param params
	 * @return
	 */
	public AccountingScoreCustomerHeadVo getAccountingListHeadSum(Map<String, Object> params) {
		return accountingScoreCustomerService.getAccountingListHeadSum(params);
	}

	/**
	 * 积分 创建 对账单(结算单) ---- 客户结算
	 *
	 * @param userId
	 * @param scoreOrderReconciliationNotYetVoList
	 * @param scoreAccountingCustomerCreateParam
	 * @return
	 * @throws Exception
	 */
	@Override
	public AccountingScoreCustomer createScoreAccountingCustomer(
			Long userId,
			List<ScoreOrderReconciliationNotYetVo> scoreOrderReconciliationNotYetVoList,
			AccountingScoreCustomerCreateParam scoreAccountingCustomerCreateParam) throws Exception {

		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");


//		return scoreAccountingCustomerService.addObj();

//		0. 记录一个流水的第一个
		ScoreOrderReconciliationNotYetVo sample = scoreOrderReconciliationNotYetVoList.get(0);


//		1. 处理 ScoreAccountingCustomerXCredit
		List<AccountingScoreCustomerXCredit> scoreAccountingCustomerXCreditList = new ArrayList<>();
		for (ScoreOrderReconciliationNotYetVo item : scoreOrderReconciliationNotYetVoList) {

			AccountingScoreCustomerXCredit x = new AccountingScoreCustomerXCredit();

			// add 20200716
			x.setUnifiedOrderId(item.getUnifiedOrderId());
			x.setUnifiedOrderGoodsId(item.getUnifiedOrderGoodsId());
			x.setUnifiedOrderCreditId(item.getUnifiedOrderCreditId());

			x.setCreateTime(new Date());
			x.setCreateUserId(userId);
			x.setUpdateTime(new Date());
			x.setUpdateUserId(userId);

			scoreAccountingCustomerXCreditList.add(x);
		}

//		2. 处理 ScoreAccountingCustomer 结算单主体
		AccountingScoreCustomer accountingScoreCustomer = new AccountingScoreCustomer();

		accountingScoreCustomer.setAccountingOrderSn("");
		accountingScoreCustomer.setServiceNo(scoreAccountingCustomerCreateParam.getServiceNo());

		accountingScoreCustomer.setCreditType(scoreAccountingCustomerCreateParam.getCreditType());
		accountingScoreCustomer.setCreditName(scoreAccountingCustomerCreateParam.getCreditName());

		accountingScoreCustomer.setBatchs(scoreAccountingCustomerCreateParam.getBatchs());

		// 统计?
//		scoreAccountingCustomer.setQuantityOrder(scoreAccountingCustomerCreateParam.getQuantityOrder());
//		scoreAccountingCustomer.setQuantityOrderGoods(scoreAccountingCustomerCreateParam.getQuantityOrderGoods());
//		scoreAccountingCustomer.setQuantityOrderCredit(scoreAccountingCustomerCreateParam.getQuantityOrderCredit());

		Integer quantityOrder       = 0;
		Integer quantityOrderGoods  = 0;
		Integer quantityOrderCredit = 0;

		quantityOrder = scoreOrderReconciliationNotYetVoList.stream().filter(AnLamdaUtils.distinctByKey(o -> o.getOrderSn())).collect(Collectors.toList()).size();
		quantityOrderGoods = scoreOrderReconciliationNotYetVoList.stream().filter(AnLamdaUtils.distinctByKey(o -> o.getGoodsId())).collect(Collectors.toList()).size();
		quantityOrderCredit = scoreOrderReconciliationNotYetVoList.size();

		accountingScoreCustomer.setQuantityOrder(quantityOrder);
		accountingScoreCustomer.setQuantityOrderGoods(quantityOrderGoods);
		accountingScoreCustomer.setQuantityOrderCredit(quantityOrderCredit);


		accountingScoreCustomer.setAccountingAmount(scoreAccountingCustomerCreateParam.getAccountingAmount());
		accountingScoreCustomer.setAccountingStatus(AccountingOrderAccountingStatusEnum.未确认.getKey());


		accountingScoreCustomer.setInvoiceStatus(AccountingOrderInvoiceStatusEnum.未申请.getKey());
//		scoreAccountingCustomer.setInvoiceType();
//		scoreAccountingCustomer.setInvoiceCompanyName();
//		scoreAccountingCustomer.setInvoiceTaxpayersIDCode();

		accountingScoreCustomer.setEnterpriseId(sample.getPurchaseVirtId());

		accountingScoreCustomer.setPurchaseUserId(scoreAccountingCustomerCreateParam.getPurchaseUserId());
		accountingScoreCustomer.setPurchaseUserName(scoreAccountingCustomerCreateParam.getPurchaseUserName());
		accountingScoreCustomer.setPurchasePhone(scoreAccountingCustomerCreateParam.getPurchasePhone());
//		scoreAccountingCustomer.setPurchaseAddress();

		accountingScoreCustomer.setIsSendEmail(scoreAccountingCustomerCreateParam.getIsSendEmail());
//		scoreAccountingCustomer.setSendEmailTime();

		accountingScoreCustomer.setDateType(scoreAccountingCustomerCreateParam.getDateType());
		accountingScoreCustomer.setStartTime(sdf.parse(scoreAccountingCustomerCreateParam.getStartTime()));
		accountingScoreCustomer.setEndTime(sdf.parse(scoreAccountingCustomerCreateParam.getEndTime()));

		accountingScoreCustomer.setBackAccountStatus(AccountingOrderBackAccountStatusEnum.未回款.getKey());
		accountingScoreCustomer.setBankInfoId(scoreAccountingCustomerCreateParam.getBankInfoId());

		accountingScoreCustomer.setCreateUserId(userId);
		accountingScoreCustomer.setCreateTime(new Date());

		accountingScoreCustomer.setUpdateUserId(userId);
		accountingScoreCustomer.setUpdateTime(new Date());

		accountingScoreCustomer.setPayType(0);
		accountingScoreCustomer.setPayStatus(0);
//		scoreAccountingCustomer.setWxQrcodeImgUrl();

		accountingScoreCustomer.setDeleted(CommonSoftDeleteEnum.正常.getKey());

		return accountingScoreCustomerService.createScoreAccountingCustomer(accountingScoreCustomer, scoreAccountingCustomerXCreditList);

	}

	/****************************************** 积分对账(结算) 对账单   end ******************************************/


	/****************************************** 积分对账(结算) 对账单 详情 start ******************************************/

	/**
	 * 用于
	 * 1. 对账单-查看详情
	 * 2. 对账单-查看详情-导出报表
	 * 3. 对账单-查看详情-发送邮件 附件
	 *
	 * @param accountingOrderId
	 * @param loginSysUser
	 * @return
	 */
	@Override
	public AccountingScoreCustomerDetailVo commonStatementOfAccountDetailDimCredit(Long accountingOrderId, LoginSysUser loginSysUser) {

		return accountingScoreCustomerService.commonStatementOfAccountDetailDimCredit(accountingOrderId, loginSysUser);

	}

	@Override
	public String exportExcelStatementOfAccountDetailDimCredit(Long accountingOrderId, LoginSysUser loginSysUser) {

		log.info("\n异步操作 包裹 外层 exportExcelStatementOfAccountDetailDimCredit params: \n{}", JsonUtils.toJson(accountingOrderId));

		CompletableFuture.runAsync(() -> {

			try {

				// 三步走 mark by linjinyu at 20210305
				// 1. 数据
				// 2. 生成excel
				// 3. 通知

				AccountingScoreCustomerDetailVo exportData = commonStatementOfAccountDetailDimCredit(accountingOrderId, loginSysUser);

				SysNotify sysNotify = accountingScoreCustomerExportExcelService.exportExcelStatementOfAccountDetailDimCredit(exportData);

				// 通知
				dubboSysNotifyService.notifyExportExcelAccountingSupplierNotYetDimGoods(sysNotify, loginSysUser);


			} catch (Exception e2) {
				log.error("\n异步操作 包裹 内层 exportExcelStatementOfAccountDetailDimCredit {} -> 执行 异常:{}", JsonUtils.toJson(accountingOrderId), e2);
			}
		});

		return "导出任务已经接收，正在处理中...";

	}
	/****************************************** 积分对账(结算) 对账单 详情   end ******************************************/


	/****************************************** 积分对账(结算) 对账单 操作 start ******************************************/

	/**
	 * 重新对账
	 *
	 * @param id
	 * @param loginSysUser
	 */
	public void opCancelAccounting(Long id, LoginSysUser loginSysUser) {

		accountingScoreCustomerService.cancelAccounting(id, AccountingOrderAccountingStatusEnum.已取消.getKey());

	}

	/**
	 * 拒绝对账
	 *
	 * @param id
	 * @param loginSysUser
	 */
	public void opRefuseAccounting(Long id, LoginSysUser loginSysUser) {

		accountingScoreCustomerService.cancelAccounting(id, AccountingOrderAccountingStatusEnum.已拒绝.getKey());

	}

	/**
	 * 确认对账
	 *
	 * @param id
	 * @param loginSysUser
	 */
	public void opConfirmAccounting(Long id, LoginSysUser loginSysUser) {

		AccountingScoreCustomer accountingScoreCustomer = new AccountingScoreCustomer();

		accountingScoreCustomer.setId(id);
		accountingScoreCustomer.setAccountingStatus(AccountingOrderAccountingStatusEnum.已确认.getKey());
		accountingScoreCustomer.setUpdateTime(new Date());
		accountingScoreCustomer.setUpdateUserId(loginSysUser.getSysUserId());

		accountingScoreCustomerService.updateObj(accountingScoreCustomer);

	}

	public void opDeleteAccounting(Long id) {

		accountingScoreCustomerService.deleteAccounting(id);

	}

	/****************************************** 积分对账(结算) 对账单 操作   end ******************************************/


	@Override
	public void deleteObj(Long id) {
		accountingScoreCustomerService.deleteObj(id);
	}

	@Override
	public void updateObj(AccountingScoreCustomer scoreAccountingCustomer, LoginSysUser loginSysUser) {

		accountingScoreCustomerService.updateObj(scoreAccountingCustomer);
	}

	@Override
	public void updateFieldRemark(AccountingScoreCustomer statementOfAccount4Update, LoginSysUser loginSysUser){

		statementOfAccount4Update.setUpdateUserId(loginSysUser.getSysUserId());
		statementOfAccount4Update.setUpdateTime(new Date());

		accountingScoreCustomerService.updateObj(statementOfAccount4Update);

	}

	@Override
	public AccountingScoreCustomer findById(Long id) {
		return accountingScoreCustomerService.findById(id);
	}

	@Override
	public void addObj(AccountingScoreCustomer scoreAccountingCustomer) {
		accountingScoreCustomerService.addObj(scoreAccountingCustomer);
	}

	@Override
	public List<AccountingScoreCustomer> findAll(Map<String, Object> queryMap) {
		return null;
	}
}
