package cn.superdesk.uniorder.dubbo.provider.invoice.customer;

import cn.superdesk.uniorder.common.exception.UnifiedOrderRuntimeException;
import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dao.entity.invoice.customer.InvoiceApply;
import cn.superdesk.uniorder.dao.entity.order.ExpressCompany;
import cn.superdesk.uniorder.dao.entity.sys.notify.SysNotify;
import cn.superdesk.uniorder.dto.param.invoice.ExportExcelInvoiceApplyQueryCriteria;
import cn.superdesk.uniorder.dto.param.invoice.customer.InvoiceApplyCommonParams;
import cn.superdesk.uniorder.dto.vo.invoice.customer.*;
import cn.superdesk.uniorder.dubbo.facade.invoice.customer.DubboInvoiceApplyService;
import cn.superdesk.uniorder.dubbo.facade.sys.notify.DubboSysNotifyService;
import cn.superdesk.uniorder.event.InvoiceApplyTriggerEventPublisher;
import cn.superdesk.uniorder.event.invoiceApplyTrigger.TailUpdateAccountingOrderInvoiceStatusEvent;
import cn.superdesk.uniorder.service.invoice.customer.InvoiceApplyExportExcelService;
import cn.superdesk.uniorder.service.invoice.customer.InvoiceApplyService;
import com.alibaba.dubbo.config.annotation.Reference;
import com.alibaba.dubbo.config.annotation.Service;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.function.Function;


/**
 * @Description: Java类作用描述
 * @Author: linjinyu
 * @CreateDate: 2021/3/17 下午6:34
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021/3/17 下午6:34
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Service(version = "1.0.0", retries = -1, timeout = 10000)
public class DubboInvoiceApplyServiceImpl implements DubboInvoiceApplyService {

	@Autowired
	private InvoiceApplyService            invoiceApplyService;
	@Autowired
	private InvoiceApplyExportExcelService invoiceApplyExportExcelService;


	@Autowired
	private InvoiceApplyTriggerEventPublisher invoiceApplyTriggerEventPublisher;


	/****************************************** 引用自身 Dubbo provider start ******************************************/
	@Reference(version = "1.0.0", timeout = 10000, retries = -1)
	private DubboSysNotifyService dubboSysNotifyService;

	/****************************************** 引用自身 Dubbo provider   end ******************************************/

	public InvoiceApply findById(Long invoiceApplyId) {
		return invoiceApplyService.findById(invoiceApplyId);
	}

	public int updateInvoiceApplyByPrimaryKeySelective(InvoiceApply invoiceApply) {
		return invoiceApplyService.updateInvoiceApplyByPrimaryKeySelective(invoiceApply);
	}

	@Override
	public List<InvoiceApplySelectScoreVo> invoiceApplySelectFocusUnifiedDimOrder(Map<String, Object> params) throws UnifiedOrderRuntimeException {
		return invoiceApplyService.invoiceApplySelectFocusUnifiedDimOrder(params);
	}

	@Override
	public List<InvoiceApplySelectScoreVo> invoiceApplySelectFocusUnifiedDimGoods(Map<String, Object> params) throws UnifiedOrderRuntimeException {
		return invoiceApplyService.invoiceApplySelectFocusUnifiedDimGoods(params);
	}

	@Override
	public List<InvoiceApplySelectScoreVo> invoiceApplySelectFocusScoreDimCredit(ExportExcelInvoiceApplyQueryCriteria criteria) throws UnifiedOrderRuntimeException {
		return invoiceApplyService.invoiceApplySelectFocusScoreDimCredit(criteria);
	}


	@Override
	public void invoiceApply(String token, InvoiceApplyCommonParams params) throws UnifiedOrderRuntimeException {

		log.info("发票申请 {}", JsonUtils.toPrettyJson(params));

		// 这地方开启了事务
		TailUpdateAccountingOrderInvoiceStatusEvent event = invoiceApplyService.invoiceApply(token, params);

		// 更新对账单 开票申请状态
		invoiceApplyTriggerEventPublisher.triggerTailUpdateAccountingOrderInvoiceStatus(event);

	}


	/****************************************** 发票申请管理 start ******************************************/
	@Override
	public R<List<InvoiceApplyPage>> findViewByPageFocusUnified(Map<String, Object> params, int pageNum, int pageSize) {
		return invoiceApplyService.findViewByPageFocusUnified(params, pageNum, pageSize);
	}

	@Override
	public InvoiceApplyDataVo findViewHeadDataFocusUnified(Map<String, Object> params) throws UnifiedOrderRuntimeException {
		return invoiceApplyService.findViewHeadDataFocusUnified(params);
	}

	@Override
	public R<List<InvoiceApplyPage>> findViewByPageFocusScore(ExportExcelInvoiceApplyQueryCriteria criteria, int pageNum, int pageSize) {
		return invoiceApplyService.findViewByPageFocusScore(criteria, pageNum, pageSize);
	}

	@Override
	public InvoiceApplyDataVo findViewHeadDataFocusScore(ExportExcelInvoiceApplyQueryCriteria criteria) throws UnifiedOrderRuntimeException {
		return invoiceApplyService.findViewHeadDataFocusScore(criteria);
	}

	/****************************************** 发票申请管理   end ******************************************/


	/****************************************** 客户结算(对账) 开票申请 列表导出 Excel数据 start ******************************************/
	/**
	 * 客户结算-发票申请管理-opt 导出报表 获取数据
	 *
	 * @param params
	 * @return
	 * @throws UnifiedOrderRuntimeException
	 */
	@Override
	public List<ExportExcelInvoiceApplyForceUnifiedDimGoodsVo> findInvoiceApplyFocusUnifiedDimGoods(Map<String, Object> params) throws UnifiedOrderRuntimeException {
		return invoiceApplyService.findInvoiceApplyFocusUnifiedDimGoods(params);
	}

	/**
	 * 积分结算-发票申请管理-opt 导出报表 获取数据
	 *
	 * @param criteria
	 * @return
	 * @throws UnifiedOrderRuntimeException
	 */
	@Override
	public List<ExportExcelInvoiceApplyForceScoreDimCreditVo> findInvoiceApplyFocusScoreDimCredit(ExportExcelInvoiceApplyQueryCriteria criteria) throws UnifiedOrderRuntimeException {

		List<ExportExcelInvoiceApplyForceScoreDimCreditVo> result = invoiceApplyService.findInvoiceApplyFocusScoreDimCredit(criteria);

		return result;
	}


	/**
	 * TODO 未实现 这个获取数据都是有问题的 目前这个有个问题 Mapper 里边还是 订单维度的 表关联方式 mark by linjinyu at 20210317
	 *
	 * @param params
	 * @param loginSysUser
	 * @return
	 * @throws UnifiedOrderRuntimeException
	 */
	public String exportExcelInvoiceApplyFocusUnifiedDimGoods(Map<String, Object> params, LoginSysUser loginSysUser) throws UnifiedOrderRuntimeException {

		// 异步操作 包裹 mark by linjinyu at 20210317
		log.info("\n异步操作 包裹 外层 exportExcelInvoiceApplyFocusUnifiedDimGoods params: \n{}", JsonUtils.toJson(params));

		CompletableFuture<Void> voidCompletableFuture = CompletableFuture.runAsync(() -> {

			try {

				log.info("\n异步操作 包裹 内层 exportExcelInvoiceApplyFocusUnifiedDimGoods params: \n{}", JsonUtils.toJson(params));

				// 三步走 mark by linjinyu at 20210317
				// 1. 数据
				// 2. 生成excel
				// 3. 通知

				List<ExportExcelInvoiceApplyForceUnifiedDimGoodsVo> resultList = invoiceApplyService.findInvoiceApplyFocusUnifiedDimGoods(params);

				SysNotify sysNotify = invoiceApplyExportExcelService.exportExcelInvoiceApplyFocusUnifiedDimGoods(resultList);

				// 通知
				dubboSysNotifyService.notifyExportExcelInvoiceApplyFocusUnifiedDimGoods(sysNotify, loginSysUser);

			} catch (Exception e2) {
				log.error("\n异步操作 包裹 内层 exportExcelInvoiceApplyFocusUnifiedDimGoods{} -> 执行 异常:{}", JsonUtils.toJson(params), e2);
			}
		});

		return "导出任务已经接收，正在处理中...";

	}

	private static Map<Integer, Function<List<ExportExcelInvoiceApplyForceScoreDimCreditVo>, SysNotify>> modeOfExportExcelInvoiceApplyFocusScoreDimCredit = new HashMap<>();

	public String exportExcelInvoiceApplyFocusScoreDimCredit(ExportExcelInvoiceApplyQueryCriteria criteria, LoginSysUser loginSysUser) throws UnifiedOrderRuntimeException {

		// 初始化
		// 侨城汇
		//	|---- 1. 搜索框 条件 结算集
		//	|---- 2. 搜索框 条件 结算集 后 界面上 勾选 要以 开票申请编号 分 sheet 分合并前 合并后
		//	|---- 3. 搜索框 条件 结算集 后 界面上 勾选 要以 开票申请编号 分 sheet 财务专用
		modeOfExportExcelInvoiceApplyFocusScoreDimCredit.put(1, (resultList) -> invoiceApplyExportExcelService.exportExcelInvoiceApplyFocusScoreDimCreditMode1(resultList));
		modeOfExportExcelInvoiceApplyFocusScoreDimCredit.put(2, (resultList) -> invoiceApplyExportExcelService.exportExcelInvoiceApplyFocusScoreDimCreditMode2(resultList));
		modeOfExportExcelInvoiceApplyFocusScoreDimCredit.put(3, (resultList) -> invoiceApplyExportExcelService.exportExcelInvoiceApplyFocusScoreDimCreditMode3(resultList));

		// 异步操作 包裹 mark by linjinyu at 20210317
		log.info("\n异步操作 包裹 外层 exportExcelInvoiceApplyFocusScoreDimCredit params: \n{}", JsonUtils.toJson(criteria));

		CompletableFuture<Void> voidCompletableFuture = CompletableFuture.runAsync(() -> {

			try {

				log.info("\n异步操作 包裹 内层 exportExcelInvoiceApplyFocusScoreDimCredit params: \n{}", JsonUtils.toJson(criteria));

				// 三步走 mark by linjinyu at 20210317
				// 1. 数据
				// 2. 生成excel
				// 3. 通知
				List<ExportExcelInvoiceApplyForceScoreDimCreditVo> resultList = invoiceApplyService.findInvoiceApplyFocusScoreDimCredit(criteria);

//				log.info("\n\n\nexportExcelInvoiceApplyFocusScoreDimCredit 数据准备 : {}\n\n\n", JsonUtils.toJson(resultList));

//				SysNotify sysNotify = invoiceApplyExportExcelService.exportExcelInvoiceApplyFocusScoreDimCreditMode1(resultList);
				SysNotify sysNotify = modeOfExportExcelInvoiceApplyFocusScoreDimCredit.get(criteria.getMode()).apply(resultList);
				sysNotify.setRemark(criteria.getRemark());

				// 通知
				dubboSysNotifyService.notifyExportExcelInvoiceApplyFocusScoreDimCredit(sysNotify, loginSysUser);

			} catch (Exception e2) {
				log.error("\n异步操作 包裹 内层 exportExcelInvoiceApplyFocusScoreDimCredit{} -> 执行 异常:{}", JsonUtils.toJson(criteria), e2);
			}
		});

		return "导出任务已经接收，正在处理中...";

	}

	/****************************************** 客户结算(对账) 开票申请 列表导出 Excel数据 start ******************************************/


	/****************************************** 客户结算(对账) 开票申请 发票管理 详情 start ******************************************/
	@Override
	public InvoiceApplyDetailDimGoodsVo getUnifiedCustomerInvoice2GoodsDetail(Map<String, Object> params) throws UnifiedOrderRuntimeException {
		return invoiceApplyService.getUnifiedCustomerInvoice2GoodsDetail(params);
	}

	@Override
	public String exportExcelUnifiedCustomerInvoice2GoodsDetail(Map<String, Object> params, LoginSysUser loginSysUser) throws UnifiedOrderRuntimeException {

		// 异步操作 包裹 mark by linjinyu at 20210223
		log.info("\n异步操作 包裹 外层 exportUnifiedCustomerInvoice2GoodsDetail params: \n{}", JsonUtils.toJson(params));

		CompletableFuture<Void> voidCompletableFuture = CompletableFuture.runAsync(() -> {

			try {

				log.info("\n异步操作 包裹 内层 exportUnifiedCustomerInvoice2GoodsDetail params: \n{}", JsonUtils.toJson(params));

				// 三步走 mark by linjinyu at 20210305
				// 1. 数据
				// 2. 生成excel
				// 3. 通知

				InvoiceApplyDetailDimGoodsVo invoiceApplyDetailDimGoodsVo = invoiceApplyService.getUnifiedCustomerInvoice2GoodsDetail(params);

				SysNotify sysNotify = invoiceApplyExportExcelService.exportExcelUnifiedCustomerInvoice2GoodsDetail(invoiceApplyDetailDimGoodsVo);

				// 通知
				dubboSysNotifyService.notifyExportExcelUnifiedCustomerInvoice2GoodsDetail(sysNotify, loginSysUser);

			} catch (Exception e2) {
				log.error("\n异步操作 包裹 内层 exportUnifiedCustomerInvoice2GoodsDetail{} -> 执行 异常:{}", JsonUtils.toJson(params), e2);
			}
		});

		return "导出任务已经接收，正在处理中...";


	}

	@Override
	public InvoiceApplyDetailDimCreditVo getUnifiedCustomerInvoice2CreditDetail(ExportExcelInvoiceApplyQueryCriteria criteria, LoginSysUser loginSysUser) throws UnifiedOrderRuntimeException {
		return invoiceApplyService.getUnifiedCustomerInvoice2CreditDetail(criteria, loginSysUser);
	}

	/**
	 * 未完成 还没有排期做 先把架子做好了而已 mark by linjinyu at 20210305
	 *
	 * @param criteria
	 * @param loginSysUser
	 * @return
	 * @throws UnifiedOrderRuntimeException
	 */
	@Override
	public String exportExcelUnifiedCustomerInvoice2CreditDetail(ExportExcelInvoiceApplyQueryCriteria criteria, LoginSysUser loginSysUser) throws UnifiedOrderRuntimeException {


		// 异步操作 包裹 mark by linjinyu at 20210223
		log.info("\n异步操作 包裹 外层 exportExcelUnifiedCustomerInvoice2CreditDetail params: \n{}", JsonUtils.toJson(criteria));

		CompletableFuture.runAsync(() -> {

			try {
				// 三步走 mark by linjinyu at 20210305
				// 1. 数据
				// 2. 生成excel
				// 3. 通知
				InvoiceApplyDetailDimCreditVo invoiceApplyDetailDimCreditVo = invoiceApplyService.getUnifiedCustomerInvoice2CreditDetail(criteria, loginSysUser);

				SysNotify sysNotify = invoiceApplyExportExcelService.exportExcelUnifiedCustomerInvoice2CreditDetail(invoiceApplyDetailDimCreditVo);

				// 通知
				dubboSysNotifyService.notifyExportExcelUnifiedCustomerInvoice2CreditDetail(sysNotify, loginSysUser);


			} catch (Exception e2) {
				log.error("\n异步操作 包裹 内层 exportExcelUnifiedCustomerInvoice2CreditDetail {} -> 执行 异常:{}", JsonUtils.toJson(criteria), e2);
			}
		});

		return "导出任务已经接收，正在处理中...";
	}

	/****************************************** 客户结算(对账) 开票申请 发票管理 详情 start ******************************************/


	/****************************************** 第三方软件导出类 start ******************************************/
	/**
	 * exportExcelUnifiedCustomerInvoice2GoodsDetail
	 *
	 * @param invoiceApplyId
	 * @param loginSysUser
	 * @return
	 * @throws UnifiedOrderRuntimeException
	 */
	@Override
	public String exportExcelDimUnifiedInvoiceRegister4E6(Long invoiceApplyId, LoginSysUser loginSysUser) throws UnifiedOrderRuntimeException {

		log.info("\n异步操作 包裹 外层 exportExcelDimUnifiedInvoiceRegister4E6 params: \n{}", invoiceApplyId);

		Map<String, Object> params = new HashMap<>();

		// 发票申请Id invoiceApplyId
		params.put("id", invoiceApplyId);

		CompletableFuture.runAsync(() -> {

			try {
				// 三步走 mark by linjinyu at 20210305
				// 1. 数据
				// 2. 生成excel
				// 3. 通知
				InvoiceApplyDetailDimGoodsVo invoiceApplyDetailDimGoodsVo = invoiceApplyService.getUnifiedCustomerInvoice2GoodsDetail(params);

				SysNotify sysNotify = invoiceApplyExportExcelService.exportExcelDimUnifiedInvoiceRegister4E6(invoiceApplyDetailDimGoodsVo);

				// 通知
				dubboSysNotifyService.notifyExportExcelDimUnifiedInvoiceRegister4E6(sysNotify, loginSysUser);


			} catch (Exception e2) {
				log.error("\n异步操作 包裹 内层 exportExcelDimUnifiedInvoiceRegister4E6 {} -> 执行 异常:{}", invoiceApplyId, e2);
			}
		});

		return "导出任务已经接收，正在处理中...";

	}

	/****************************************** 第三方软件导出类   end ******************************************/


	@Override
	public String importInvoiceExcel(String token, List<Map<String, Object>> list) throws UnifiedOrderRuntimeException {
		return invoiceApplyService.importInvoiceExcel(token, list);
	}

	@Override
	public String importSendInvoiceExcel(String token, List<Map<String, Object>> list) throws UnifiedOrderRuntimeException {
		return invoiceApplyService.importSendInvoiceExcel(token, list);
	}


	@Override
	public List<ExpressCompany> findExpressCompany() throws UnifiedOrderRuntimeException {
		return invoiceApplyService.findExpressCompany();
	}

	@Override
	public R<List<InvoiceApply>> findInvoiceByPageNew(Map<String, Object> params, int pageNum, int pageSize) {
		return invoiceApplyService.findInvoiceByPageNew(params, pageNum, pageSize);
	}

	@Override
	public List<InvoiceApply> findInvoiceByOrderSn(Map<String, Object> invQuery) {
		return invoiceApplyService.findInvoiceByOrderSn(invQuery);
	}

	@Override
	public Map<String, InvoiceApply> findInvoiceByOrderSnList(List<String> orderSnList) {

		return invoiceApplyService.findInvoiceByOrderSnList(orderSnList);

	}


	@Override
	public void validatedInvoiceApplyRelationTable() {


		invoiceApplyService.validatedInvoiceApplyRelationTable();
	}


	@Override
	public void opAudit(Map<String, Object> params, LoginSysUser loginSysUser) throws UnifiedOrderRuntimeException {

		// 这地方开启了事务
		TailUpdateAccountingOrderInvoiceStatusEvent event = invoiceApplyService.opAudit(params, loginSysUser);

		invoiceApplyTriggerEventPublisher.triggerTailUpdateAccountingOrderInvoiceStatus(event);
	}

	/**
	 * (过 20200730 1711)
	 *
	 * @param params
	 * @param loginSysUser
	 * @throws UnifiedOrderRuntimeException
	 */
	@Override
	public void opAgain(Map<String, Object> params, LoginSysUser loginSysUser) throws UnifiedOrderRuntimeException {


		// 这地方开启了事务
		TailUpdateAccountingOrderInvoiceStatusEvent event = invoiceApplyService.opAgain(params, loginSysUser);

		invoiceApplyTriggerEventPublisher.triggerTailUpdateAccountingOrderInvoiceStatus(event);
	}


	/**
	 * 删除发票申请 关联删除 此申请 X 流水
	 *
	 * @param invoiceApplyId
	 * @param loginSysUser
	 */
	public void opDelete(Long invoiceApplyId, LoginSysUser loginSysUser) {
		invoiceApplyService.opDelete(invoiceApplyId, loginSysUser);
	}

	/**
	 * opt 开票登记
	 *
	 * @param params
	 * @param loginSysUser
	 * @throws UnifiedOrderRuntimeException
	 */
	@Override
	public void opRegister(Map<String, Object> params, LoginSysUser loginSysUser) throws UnifiedOrderRuntimeException {
		invoiceApplyService.opRegister(params, loginSysUser);
	}

	/**
	 * opt 寄票登记
	 *
	 * @param sendInvoiceRegisterList
	 * @param loginSysUser
	 * @throws UnifiedOrderRuntimeException
	 */
	@Override
	public void sendInvoiceRegister(List<Map> sendInvoiceRegisterList, LoginSysUser loginSysUser) throws UnifiedOrderRuntimeException {
		invoiceApplyService.invoiceRegisterSend(sendInvoiceRegisterList, loginSysUser);
	}

}
