package cn.superdesk.uniorder.dubbo.provider.invoice.supplier;

import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.common.numerical.AnCollectionUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.enums.invoice.*;
import cn.superdesk.uniorder.dao.entity.base.UnifiedOrder;
import cn.superdesk.uniorder.dao.entity.invoice.supplier.UnifiedSupplierInv;
import cn.superdesk.uniorder.dao.entity.invoice.supplier.UnifiedSupplierInvItem;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dto.vo.invoice.supplier.UnifiedSupplierInvDetail;
import cn.superdesk.uniorder.dto.vo.invoice.supplier.UnifiedSupplierInvGoodsDetail;
import cn.superdesk.uniorder.dto.vo.invoice.supplier.UnifiedSupplierInvoiceAddParams;
import cn.superdesk.uniorder.dto.vo.invoice.supplier.UnifiedSupplierInvoiceAddParamsPackage;
import cn.superdesk.uniorder.dto.vo.unified.InvoiceApplySupplierSelectGoodsVo;
import cn.superdesk.uniorder.dto.vo.unified.UnifiedInvHE;
import cn.superdesk.uniorder.service.base.UnifiedOrderService;
import cn.superdesk.uniorder.service.setting.UnifiedSnConfigService;
import cn.superdesk.uniorder.service.invoice.supplier.UnifiedSupplierInvItemService;
import cn.superdesk.uniorder.service.invoice.supplier.UnifiedSupplierInvService;
import cn.superdesk.uniorder.dubbo.facade.invoice.supplier.DubboUnifiedSupplierInvService;
import cn.superdesk.uniorder.common.numerical.AnConvertBeanUtil;
import cn.superdesk.uniorder.common.lamda.AnLamdaUtils;
import com.alibaba.dubbo.config.annotation.Service;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service(version = "1.0.0", retries = -1, timeout = 10000)
public class DubboUnifiedSupplierInvServiceImpl implements DubboUnifiedSupplierInvService {

	@Autowired
	private UnifiedSupplierInvService     unifiedSupplierInvService;
	@Autowired
	private UnifiedSupplierInvItemService unifiedSupplierInvItemService;
	@Autowired
	private UnifiedSnConfigService        unifiedSnConfigService;
	@Autowired
	private UnifiedOrderService           unifiedOrderService;

	@Override
	public R<List<UnifiedSupplierInv>> getInvoiceApplyList(Map<String, Object> params, int pageNum, int pageSize) {

		return unifiedSupplierInvService.getInvoiceApplyList(params, pageNum, pageSize);
	}

	@Override
	public UnifiedInvHE getInvoiceApplyListHeadSum(Map<String, Object> params) {

		return unifiedSupplierInvService.getInvoiceApplyListHeadSum(params);
	}

	/**
	 * @param id
	 * @return
	 */
	@Override
	public UnifiedSupplierInvDetail getInvoiceDetail(/* 发票申请Id */Long id) {

		UnifiedSupplierInv       invoiceApply       = unifiedSupplierInvService.findById(id);
		UnifiedSupplierInvDetail invoiceApplyDetail = AnConvertBeanUtil.copyBean(invoiceApply, UnifiedSupplierInvDetail.class);

		// 查询 goods 对账单 goods
		Map<String, Object> params = new HashMap<>();
		// |---- 发票申请号
		params.put("invoiceSn", invoiceApply.getInvoiceSn());
		List<UnifiedSupplierInvGoodsDetail> goods = unifiedSupplierInvItemService.getDetail(params);

		invoiceApplyDetail.setGoods(goods);

		return invoiceApplyDetail;
	}

	@Override
	public List<InvoiceApplySupplierSelectGoodsVo> getStatementOfAccountGoodsListByAccountingOrderSn(String accountingOrderSn) {
		return unifiedSupplierInvService.getStatementOfAccountGoodsListByAccountingOrderSn(accountingOrderSn);
	}

	@Override
	public List<InvoiceApplySupplierSelectGoodsVo> getStatementOfAccountGoodsListBySysUnifiedOrderGoodsIds(String accountingOrderSn, List<Long> processSysUnifiedOrderGoodsIds) {
		return unifiedSupplierInvService.getStatementOfAccountGoodsListBySysUnifiedOrderGoodsIds(accountingOrderSn, processSysUnifiedOrderGoodsIds);
	}

	/****************************************** 开票申请 start ******************************************/
	@Override
	public List<InvoiceApplySupplierSelectGoodsVo> invoiceApplySelectFocusUnified(String accountingOrderSn) {
		return unifiedSupplierInvService.invoiceApplySelectFocusUnified(accountingOrderSn);
	}

	@Override
	public UnifiedSupplierInv opInvoiceApply(UnifiedSupplierInvoiceAddParams invoiceApplyParams, LoginSysUser loginSysUser) {

		UnifiedSupplierInv invoiceApplyEntity = new UnifiedSupplierInv();

		String invoiceApplySn = unifiedSnConfigService.getSupplierInvoiceApplySnBySnowflake();
		invoiceApplyEntity.setInvoiceSn(invoiceApplySn);

		// 申请人部分
		invoiceApplyEntity.setApplyTime(new Date());
		invoiceApplyEntity.setApplyUserId(loginSysUser.getSysUserId());
		invoiceApplyEntity.setApplyUserName(loginSysUser.getSysUserName());

		// 审核
		invoiceApplyEntity.setAuditStatus(InvoiceApplyAuditStatusEnum.未审核.getKey());

		// 回款
		invoiceApplyEntity.setBackAccountStatus(InvoiceApplyBackAccountStatusEnum.未回款.getKey());

		// 开票登记
		invoiceApplyEntity.setInvoiceStatus(InvoiceApplyInvoiceProgressingStatusEnum.待开票.getKey());
		invoiceApplyEntity.setInvoiceTitle(invoiceApplyParams.getInvoiceTitle());
		invoiceApplyEntity.setInvoiceType(invoiceApplyParams.getInvType());

		invoiceApplyEntity.setMakeInvoiceStatus(InvoiceApplyMakeInvoiceStatusEnum.未开票.getKey());

		invoiceApplyEntity.setRemark(invoiceApplyParams.getRemark());

//        // 前端传过来 UnifiedOrderGoodsIds
//        List<Long> processSysUnifiedOrderGoodsIds = SuperdeskStringUtils.stringArrayConvertToLongList(invoiceApplyParams.getIds(), ",");
//
//        log.info("\nopInvoiceApply {}: \n\n\n\n{}\n\n\n\n", processSysUnifiedOrderGoodsIds.size(), JsonUtils.toJson(processSysUnifiedOrderGoodsIds));
//
//        List<InvoiceApplySupplierSelectGoodsVo> invoiceApplySupplierSelectGoodsVoList = unifiedSupplierInvService.getStatementOfAccountGoodsListBySysUnifiedOrderGoodsIds(processSysUnifiedOrderGoodsIds);
//
//        log.info("\nopInvoiceApply {} 转换后", invoiceApplySupplierSelectGoodsVoList.size());

		/****************************************** 重构 start ******************************************/

		List<InvoiceApplySupplierSelectGoodsVo> invoiceApplySupplierSelectGoodsVoList = new ArrayList<>();


//        List<UnifiedSupplierInvoiceAddParamsPackage> packageList = invoiceApplyParams.getPackageList();
		List<UnifiedSupplierInvoiceAddParamsPackage> packageList = JsonUtils.toList(invoiceApplyParams.getJsonStr(), UnifiedSupplierInvoiceAddParamsPackage.class);
		log.info("供应商 创建对账单 包体 {}", JsonUtils.toJson(packageList));


		Map<String, List<Long>> processAccountingOrderList = packageList.parallelStream()
				.collect(Collectors.groupingBy(UnifiedSupplierInvoiceAddParamsPackage::getAccountingOrderSn, Collectors.mapping(UnifiedSupplierInvoiceAddParamsPackage::getUnifiedOrderGoodsId, Collectors.toList())));

		for (Map.Entry<String, List<Long>> _entry : processAccountingOrderList.entrySet()) {

			List<InvoiceApplySupplierSelectGoodsVo> sampleData = unifiedSupplierInvService.getStatementOfAccountGoodsListBySysUnifiedOrderGoodsIds(_entry.getKey(), _entry.getValue());

			if (AnCollectionUtils.isNotEmpty(sampleData)) {
				invoiceApplySupplierSelectGoodsVoList.addAll(sampleData);
			}
		}

		/****************************************** 重构   end ******************************************/


		// 标记 他们是一伙的
//        // 订单数量
//        Map<String, String> orderQuantity = new HashMap<>();
//        // 订单金额
//        BigDecimal orderAmount = BigDecimal.ZERO;

		List<UnifiedSupplierInvItem> invoiceApplyXGoodsProcessList = new ArrayList<>();
		for (InvoiceApplySupplierSelectGoodsVo unifiedInvGoodsVo : invoiceApplySupplierSelectGoodsVoList) {

			UnifiedSupplierInvItem invoiceApplyXGoodsItem = new UnifiedSupplierInvItem();

			invoiceApplyXGoodsItem.setInvoiceSn(invoiceApplyEntity.getInvoiceSn());
			invoiceApplyXGoodsItem.setAccountingOrderSn(unifiedInvGoodsVo.getAccountingOrderSn());
			invoiceApplyXGoodsItem.setOrderSn(unifiedInvGoodsVo.getOrderSn());
			invoiceApplyXGoodsItem.setGoodsId(unifiedInvGoodsVo.getGoodsId());
			invoiceApplyXGoodsItem.setBackStatus(InvoiceOrderBackStatusEnum.未回款.getKey());

			// 标记 他们是一伙的
//            if (!orderQuantity.containsKey(unifiedInvGoodsVo.getOrderSn())) {
//                orderQuantity.put(unifiedInvGoodsVo.getOrderSn(), unifiedInvGoodsVo.getOrderSn());
//            }
//
//            orderAmount = orderAmount.add(new BigDecimal(unifiedInvGoodsVo.getCostPrice().multiply(new BigDecimal(unifiedInvGoodsVo.getQuantity())).toString()));

			invoiceApplyXGoodsProcessList.add(invoiceApplyXGoodsItem);
		}


		// 统计修正 mark by linjinyu 20201105
		/****************************************** 统计 start ******************************************/
		// 标记 他们是一伙的
//        invoiceApplyEntity.setOrderAmount(orderAmount);
//        invoiceApplyEntity.setOrderQuantity(orderQuantity.size());

		Long accountingOrderSnCount = invoiceApplySupplierSelectGoodsVoList.stream().filter(AnLamdaUtils.distinctByKey(o -> o.getAccountingOrderSn())).map(o -> o.getAccountingOrderSn()).collect(Collectors.counting());
		Long orderSnCount           = invoiceApplySupplierSelectGoodsVoList.stream().filter(AnLamdaUtils.distinctByKey(o -> o.getOrderSn())).map(o -> o.getOrderSn()).collect(Collectors.counting());
		Long goodsQuantity          = invoiceApplySupplierSelectGoodsVoList.stream().map(o -> o.getGoodsId()).collect(Collectors.counting());
		Long goodsItemQuantity      = invoiceApplySupplierSelectGoodsVoList.stream().map(o -> o.getQuantity()).collect(Collectors.counting());

		BigDecimal goodsAmount = invoiceApplySupplierSelectGoodsVoList
				.stream()
				.map(item -> item.getPrice() == null ? BigDecimal.ZERO : item.getPrice().multiply(new BigDecimal(item.getQuantity())))
				.reduce(BigDecimal.ZERO, BigDecimal::add);
		BigDecimal goodsCostAmount = invoiceApplySupplierSelectGoodsVoList
				.stream()
				.map(item -> item.getCostPrice() == null ? BigDecimal.ZERO : item.getCostPrice().multiply(new BigDecimal(item.getQuantity())))
				.reduce(BigDecimal.ZERO, BigDecimal::add);

		invoiceApplyEntity.setAccountingOrderQuantity(accountingOrderSnCount.intValue());
		invoiceApplyEntity.setOrderQuantity(orderSnCount.intValue());
		invoiceApplyEntity.setOrderAmount(goodsCostAmount);
		invoiceApplyEntity.setGoodsAmount(goodsAmount);
		invoiceApplyEntity.setGoodsCostAmount(goodsCostAmount);
		invoiceApplyEntity.setGoodsQuantity(goodsQuantity.intValue());
		invoiceApplyEntity.setGoodsItemQuantity(goodsItemQuantity.intValue());


		/****************************************** 统计   end ******************************************/

		UnifiedOrder order = unifiedOrderService.findByOrderSn(invoiceApplyXGoodsProcessList.get(0).getOrderSn());
		invoiceApplyEntity.setSaleId(order.getSaleId());
		invoiceApplyEntity.setSaleName(order.getSaleName());

		return unifiedSupplierInvService.opInvoiceApply(invoiceApplyEntity, invoiceApplyXGoodsProcessList);
	}

	/****************************************** 开票申请   end ******************************************/
	@Override
	public UnifiedSupplierInv updateObj(UnifiedSupplierInv update) {

		unifiedSupplierInvService.updateObj(update);

		return update;
	}

	@Override
	public void opAgain(Long id) {

		UnifiedSupplierInv invoiceApplyEntity = unifiedSupplierInvService.findById(id);
		invoiceApplyEntity.setMakeInvoiceStatus(InvoiceApplyMakeInvoiceStatusEnum.重新开票.getKey());

		unifiedSupplierInvService.opAgain(invoiceApplyEntity);
	}

	@Override
	public void opDelete(Long id) {

		UnifiedSupplierInv invoiceApplyEntity = unifiedSupplierInvService.findById(id);
		invoiceApplyEntity.setMakeInvoiceStatus(InvoiceApplyMakeInvoiceStatusEnum.重新开票.getKey());

		unifiedSupplierInvService.opDelete(invoiceApplyEntity);
	}


	@Override
	public void patchInvoiceApplyFieldStatistics() {
		unifiedSupplierInvService.patchInvoiceApplyFieldStatistics();
	}


}
