package cn.superdesk.uniorder.pattern.state.accounting.supplier.accountingStatus.context;

import cn.superdesk.uniorder.common.enums.statement.AccountingOrderAccountingStatusEnum;
import cn.superdesk.uniorder.dao.entity.accounting.unified.supplier.AccountingUnifiedSupplier;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.pattern.state.accounting.supplier.accountingStatus.state.StatementOfAccountState;
import cn.superdesk.uniorder.pattern.state.accounting.supplier.accountingStatus.state.concrete.*;
import lombok.Data;

/**
 * @Description: Java类作用描述
 * @Author: linjinyu
 * @CreateDate: 10/12/20 4:28 PM
 * @UpdateUser: linjinyu
 * @UpdateDate: 10/12/20 4:28 PM
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Data
//@ToString
//@Component
public class StatementOfAccountContext {


	//	已取消(-1, "已取消"),
	//	未确认(0, "未确认"),
	//	已确认(1, "已确认"),
	//	已拒绝(2, "已拒绝"),
	//	待提交(3, "待提交"),

	private StatementOfAccountState state_minus1_已取消;
	private StatementOfAccountState state_0_未确认;
	private StatementOfAccountState state_1_已确认;
	private StatementOfAccountState state_2_已拒绝;
	private StatementOfAccountState state_3_待提交;

	private StatementOfAccountState state;

	private AccountingUnifiedSupplier entity;

	private LoginSysUser loginSysUser;


	public StatementOfAccountContext(/*传入 对账单 实体*/AccountingUnifiedSupplier entity, LoginSysUser loginSysUser) {


		this.state_minus1_已取消 = new State_minus1_已取消(this);
		this.state_0_未确认 = new State_0_未确认(this);
		this.state_1_已确认 = new State_1_已确认(this);
		this.state_2_已拒绝 = new State_2_已拒绝(this);
		this.state_3_待提交 = new State_3_待提交(this);

		// 假定 传入 对账单 实体 为待提交
//		state = state待提交;

		this.entity = entity;
		this.loginSysUser = loginSysUser;

		switch (AccountingOrderAccountingStatusEnum.get(this.entity.getAccountingStatus())) {
			case 已取消:
				state = state_minus1_已取消;
				break;
			case 未确认:
				state = state_0_未确认;
				break;
			case 已确认:
				state = state_1_已确认;
				break;
			case 已拒绝:
				state = state_2_已拒绝;
				break;
			case 待提交:
				state = state_3_待提交;
				break;
			default:
				state = state_3_待提交;
				break;

		}
	}

	// 统一订单
	//	待提交(3, "待提交"),
	public void opCreate() {
		state.opCreate();
	}

	//	未确认(0, "未确认"),
	public void opSubmit() {
		state.opSubmit();
	}

	//	已取消(-1, "已取消"),
	public void opCancel() {
		state.opCancel();
	}

	// 供应商
	//	已确认(1, "已确认"),
	public void opConfirm() {
		state.opConfirm();
	}

	//	已拒绝(2, "已拒绝"),
	public void opAppeal() {
		state.opAppeal();
	}

	// 统一订单
	//	已取消(-1, "已取消"),
	public void opAgain() {
		state.opAgain();
	}

	@Override
	public String toString() {
		return "对账单 对账状态 :" + getState().getClass().getName();
	}

}