package cn.superdesk.uniorder.queue.delayed;

import cn.superdesk.uniorder.common.enums.common.CommonServiceEnum;
import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.common.queue.delayed.DelayQueueManager;
import cn.superdesk.uniorder.common.queue.delayed.DelayTaskAsync;
import cn.superdesk.uniorder.common.queue.delayed.DelayTaskBase;
import cn.superdesk.uniorder.dao.entity.sys.role.SysRoleExtData;
import cn.superdesk.uniorder.dao.entity.sys.role.SysRoleExtDataExample;
import cn.superdesk.uniorder.dao.entity.sys.role.SysRoleExtDataSample;
import cn.superdesk.uniorder.dao.entity.sys.role.SysRoleExtDataSampleExample;
import cn.superdesk.uniorder.dao.mapper.sys.role.SysRoleExtDataMapper;
import cn.superdesk.uniorder.dao.mapper.sys.role.SysRoleExtDataSampleMapper;
import cn.superdesk.uniorder.common.numerical.AnCollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Component
public class DelayQueueHandler extends DelayQueueManager {

	@Autowired
	private SysRoleExtDataMapper       sysRoleExtDataMapper;
	@Autowired
	private SysRoleExtDataSampleMapper sysRoleExtDataSampleMapper;

	/**
	 * 内部执行延时任务 关键字 运营商 渠道商 数据权限
	 *
	 * @param taskAsync
	 */

	public void processTask(DelayTaskAsync taskAsync) {

		log.info("执行延时任务:{}", taskAsync);

		// 根据task中的data自定义数据来处理相关逻辑，例 if (task.getData() instanceof XXX) {} 扩展

		if (taskAsync.getData() instanceof DelayTaskBase) {

			Long outChannelId = Long.valueOf(taskAsync.getData().getIdentifier());

			SysRoleExtDataExample sysRoleExtDataExample = new SysRoleExtDataExample();
			sysRoleExtDataExample
					.createCriteria()
					.andSourceFromQuoteIdEqualTo(outChannelId);

			List<SysRoleExtData> sysRoleExtDataList = sysRoleExtDataMapper.selectByExample(sysRoleExtDataExample);

			SysRoleExtDataSampleExample sysRoleExtDataSampleExample = new SysRoleExtDataSampleExample();
			sysRoleExtDataSampleExample.createCriteria()
					.andSourceFromQuoteParentIdEqualTo(outChannelId);

			List<SysRoleExtDataSample> sysRoleExtDataSampleList = sysRoleExtDataSampleMapper.selectByExample(sysRoleExtDataSampleExample);

			// 其中一个为空就没意义
			if (AnCollectionUtils.isNotEmpty(sysRoleExtDataList) && AnCollectionUtils.isNotEmpty(sysRoleExtDataSampleList)) {

				List<Long>   sysRoleExtSampleIds             = sysRoleExtDataSampleList.stream().map(SysRoleExtDataSample::getId).collect(Collectors.toList());
				List<String> sysRoleExtSampleServiceNoMult   = sysRoleExtDataSampleList.stream().map(SysRoleExtDataSample::getServiceNo).distinct().collect(Collectors.toList());
				List<String> sysRoleExtSampleServiceNameMult = new ArrayList<>();

				sysRoleExtSampleServiceNoMult.forEach(item -> {
					sysRoleExtSampleServiceNameMult.add(CommonServiceEnum.getByServiceNo(item).getServiceName());
				});

				SysRoleExtData forUpdate = sysRoleExtDataList.get(0);
				forUpdate.setSysRoleExtSampleIds(StringUtils.join(sysRoleExtSampleIds.toArray(), ","));
				// 这个是有问题 TODO 20200921 by linjinyu
				forUpdate.setServiceNoMult(StringUtils.join(sysRoleExtSampleServiceNoMult.toArray(), ","));
				forUpdate.setServiceNameMult(StringUtils.join(sysRoleExtSampleServiceNameMult.toArray(), ","));

				sysRoleExtDataMapper.updateByPrimaryKeySelective(forUpdate);

			} else {
				log.error("\n\n\n\n任务-{}\n数据权限角色 : {}\n数据权限许可 : {}\n\n\n\n", taskAsync, JsonUtils.toJson(sysRoleExtDataList), JsonUtils.toJson(sysRoleExtDataSampleList));
			}


		}

		// ...
	}
}
