package cn.superdesk.uniorder.service.accounting.jd.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.io.FileUtil;
import cn.superdesk.uniorder.common.enums.common.CommonProcessResultEnum;
import cn.superdesk.uniorder.common.enums.scheduler.SysQuartzScheduleTaskStatementOfAccountRunStateEnum;
import cn.superdesk.uniorder.common.excel.ExcelResponse;
import cn.superdesk.uniorder.common.numerical.AnBeanUtils;
import cn.superdesk.uniorder.dao.entity.accounting.jd.AccountingUnifiedSupplierOrderJdCheckUpReport;
import cn.superdesk.uniorder.dao.entity.sys.notify.SysNotify;
import cn.superdesk.uniorder.dto.env.provider.excel.ExcelExportConstant;
import cn.superdesk.uniorder.dto.param.accounting.jd.AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow;
import cn.superdesk.uniorder.notify.NotifyUtils;
import cn.superdesk.uniorder.service.accounting.jd.AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelService;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * @Description: Java类作用描述
 * @Author: linjinyu
 * @CreateDate: 2021/11/5 上午11:29
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021/11/5 上午11:29
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Service
public class AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelServiceImpl implements AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelService {

	@Autowired
	private ExcelExportConstant excelExportConstant;


	private static final String TEMPLATE_EXCEL_EXPORT_ACCOUNTING_UNIFIED_SUPPLIER_ORDER_JD_CHECKUP_REPORT = "exportExcelJdCheckUpReport_v20211108.xlsx";


	public SysNotify exportExcelJdCheckUpReport(List<AccountingUnifiedSupplierOrderJdCheckUpReport> resultList) {


		String excelExportPathRoot             = excelExportConstant.getPathRoot();
		String excelExportRelativePathTemplate = excelExportConstant.getPathTemplate();
		String excelExportRelativePathStorage  = excelExportConstant.getPathStorage();

		String relativePath = File.separator + "excel" + File.separator + "accounting" + File.separator + "score" + File.separator + "jd" + File.separator;

		String templateRelativePath = excelExportPathRoot + excelExportRelativePathTemplate + relativePath;
		String templateFullPath;

		String targetFullPath = excelExportPathRoot + excelExportRelativePathStorage;

		String targetFileName          = "京东对账" + "_" + "比对报告_" + "电子账单" + ExcelResponse.addTail() + ".xlsx";
		String targetWriteFileFullPath = targetFullPath + relativePath + targetFileName;

//		if (!FileUtil.exist(targetWriteFileFullPath)) {
//			FileUtil.mkdir(targetWriteFileFullPath);
//		}
		FileUtil.touch(targetWriteFileFullPath);


		templateFullPath = templateRelativePath + TEMPLATE_EXCEL_EXPORT_ACCOUNTING_UNIFIED_SUPPLIER_ORDER_JD_CHECKUP_REPORT;

		ExcelWriter excelWriter = EasyExcel
				.write(targetWriteFileFullPath)
//				.inMemory(Boolean.TRUE)
//				.withTemplate(templateFullPath)
				.head(AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow.class)
				.build();

		WriteSheet writeSheet0 = EasyExcel
				.writerSheet(0)
//				.registerWriteHandler(new AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelCellWriteHandler())
//				.registerWriteHandler(new AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRowWriteHandler())
				.build();

//		FillConfig fillConfig  = FillConfig.builder().forceNewRow(Boolean.FALSE).build();

		log.info("\n{}\n{}\n{}", templateFullPath, targetWriteFileFullPath, resultList.size());

		// 转换数据 AccountingUnifiedSupplierOrderJdCheckUpReport -> AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow

		List<AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow> convertResultList = convertAccountingUnifiedSupplierOrderJdCheckUpReport2AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow(resultList);

		// 数据填充
//		excelWriter.fill(new FillWrapper("detail", resultList), fillConfig, writeSheet0);
//		excelWriter.fill(new FillWrapper("detail", resultList), writeSheet0);
//		excelWriter.fill(convertResultList, writeSheet0);

		excelWriter.write(convertResultList, writeSheet0);

		// 设置强制计算公式：不然公式会以字符串的形式显示在excel中
//		Workbook workbook = excelWriter.writeContext().writeWorkbookHolder().getWorkbook();
//		workbook.setForceFormulaRecalculation(true);

		// 别忘记关闭流
		excelWriter.finish();

		return NotifyUtils.createSysNotify(targetFileName, relativePath + targetFileName, SysQuartzScheduleTaskStatementOfAccountRunStateEnum.成功);

	}

	private static List<AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow> convertAccountingUnifiedSupplierOrderJdCheckUpReport2AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow(List<AccountingUnifiedSupplierOrderJdCheckUpReport> resultList) {

		if (CollUtil.isEmpty(resultList)) {
			return null;
		}

		List<AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow> convertResultList = new ArrayList<>();

		resultList.forEach(item -> {

			AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow tmp = AnBeanUtils.copy(item, AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow.class);

			tmp.setFormulaOrderAmount(tmp.getOrderCostAmount().add(tmp.getRefundOrderCostAmount()).add(tmp.getTransportMerchantAssumeAmount()));
			tmp.setFormulaGoodsAmount(tmp.getGoodsCostAmount().add(tmp.getRefundGoodsCostAmount()).add(tmp.getTransportMerchantAssumeAmount()));

			tmp.setDifferOrderAmount(tmp.getJdOutStandingAmount().subtract(tmp.getFormulaOrderAmount()));
			tmp.setDifferGoodsAmount(tmp.getJdOutStandingAmount().subtract(tmp.getFormulaGoodsAmount()));

			tmp.setProcessResultStr(CommonProcessResultEnum.get(tmp.getProcessResult()).getDesc());

			convertResultList.add(tmp);

		});

		return convertResultList;

	}


}
