package cn.superdesk.uniorder.service.accounting.score.customer.impl;

import cn.superdesk.uniorder.common.enums.statement.AccountingOrderAccountingStatusEnum;
import cn.superdesk.uniorder.common.enums.statement.AccountingOrderInvoiceStatusEnum;
import cn.superdesk.uniorder.common.enums.statement.AccountingOrderIsSendEmailEnum;
import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dao.entity.sys.SysUser;
import cn.superdesk.uniorder.dto.vo.accounting.score.customer.AccountingScoreCustomerDetailVo;
import cn.superdesk.uniorder.dto.vo.score.ScoreOrderReconciliationNotYetVo;
import cn.superdesk.uniorder.service.base.ScoreOrderService;
import cn.superdesk.uniorder.service.setting.UnifiedSnConfigService;
import cn.superdesk.uniorder.service.sys.SysUserService;
import cn.superdesk.uniorder.common.numerical.AnCollectionUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.enums.common.CommonAccountingStateEnum;
import cn.superdesk.uniorder.common.enums.common.CommonSoftDeleteEnum;
import cn.superdesk.uniorder.dao.entity.accounting.score.customer.AccountingScoreCustomer;
import cn.superdesk.uniorder.dao.entity.accounting.score.customer.AccountingScoreCustomerExample;
import cn.superdesk.uniorder.dao.entity.accounting.score.customer.AccountingScoreCustomerXCredit;
import cn.superdesk.uniorder.dao.entity.accounting.score.customer.AccountingScoreCustomerXCreditExample;
import cn.superdesk.uniorder.dao.entity.base.UnifiedOrderCredit;
import cn.superdesk.uniorder.dao.entity.base.UnifiedOrderCreditExample;
import cn.superdesk.uniorder.dao.mapper.accounting.score.customer.AccountingScoreCustomerMapper;
import cn.superdesk.uniorder.dao.mapper.accounting.score.customer.AccountingScoreCustomerMapperEx;
import cn.superdesk.uniorder.dao.mapper.accounting.score.customer.AccountingScoreCustomerXCreditMapper;
import cn.superdesk.uniorder.dao.mapper.base.UnifiedOrderCreditMapper;
import cn.superdesk.uniorder.dto.vo.accounting.score.customer.AccountingScoreCustomerHeadVo;
import cn.superdesk.uniorder.dto.vo.accounting.score.customer.AccountingScoreCustomerVo;
import cn.superdesk.uniorder.service.accounting.score.customer.AccountingScoreCustomerService;
import cn.superdesk.uniorder.common.lamda.AnLamdaUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * @Description: Java类作用描述
 * @Author: linjinyu
 * @CreateDate: 6/28/20 1:03 PM
 * @UpdateUser: linjinyu
 * @UpdateDate: 6/28/20 1:03 PM
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Service
public class AccountingScoreCustomerServiceImpl implements AccountingScoreCustomerService {

	@Autowired
	private AccountingScoreCustomerMapper        accountingScoreCustomerMapper;
	@Autowired
	private AccountingScoreCustomerMapperEx      accountingScoreCustomerMapperEx;
	@Autowired
	private AccountingScoreCustomerXCreditMapper accountingScoreCustomerXCreditMapper;
	@Autowired
	private UnifiedOrderCreditMapper             unifiedOrderCreditMapper;

	@Autowired
	private ScoreOrderService scoreOrderService;

//	@Autowired
//	private SysConfigService sysConfigService;

	@Autowired
	private UnifiedSnConfigService unifiedSnConfigService;

	@Autowired
	private SysUserService sysUserService;


	public AccountingScoreCustomer findByAccountingOrderSn(String accountingOrderSn) {

		AccountingScoreCustomerExample example = new AccountingScoreCustomerExample();
		example.createCriteria()
				.andAccountingOrderSnEqualTo(accountingOrderSn);

		List<AccountingScoreCustomer> result = accountingScoreCustomerMapper.selectByExample(example);

		if (AnCollectionUtils.isNotEmpty(result)) {
			return result.get(0);
		} else {
			return null;
		}

	}

	/****************************************** 积分对账(结算) 对账单 start ******************************************/

	@Override
	public List<Long> getAccountingXCreditIds(Long id) {

		AccountingScoreCustomer search = accountingScoreCustomerMapper.selectByPrimaryKey(id);

		AccountingScoreCustomerXCreditExample example = new AccountingScoreCustomerXCreditExample();
		example.createCriteria()
				.andAccountingOrderSnEqualTo(search.getAccountingOrderSn());
		List<AccountingScoreCustomerXCredit> tempResult = accountingScoreCustomerXCreditMapper.selectByExample(example);

		log.info("getAccountingXCreditIds \n{}", JsonUtils.toJson(tempResult));
		log.info("getAccountingXCreditIds \n{}", JsonUtils.toJson(tempResult
				.stream()
				.map(AccountingScoreCustomerXCredit::getUnifiedOrderCreditId)
				.collect(Collectors.toList())));

		return tempResult
				.stream()
				.map(AccountingScoreCustomerXCredit::getUnifiedOrderCreditId)
				.collect(Collectors.toList());
	}

	/**
	 * @param params
	 * @param pageNum
	 * @param pageSize
	 * @return
	 */
	@Override
	public R<List<AccountingScoreCustomerVo>> getAccountingList(Map<String, Object> params, int pageNum, int pageSize) {

		log.info("奇怪的问题::{}:{}:{}", JsonUtils.toJson(params), pageNum, pageSize);

		PageHelper.startPage(pageNum, pageSize);
		List<AccountingScoreCustomerVo> result = accountingScoreCustomerMapperEx.getAccountingList(params);

		// 附加信息
		additionalInfoGetAccountingList(result);

		PageInfo<AccountingScoreCustomerVo> page = new PageInfo<AccountingScoreCustomerVo>(result);
		PageHelper.clearPage();
		return new R<List<AccountingScoreCustomerVo>>(page.getPageNum(), page.getPageSize(), (int) page.getTotal(), result);
	}

	@Override
	public List<AccountingScoreCustomerVo> getAccountingList(Map<String, Object> params, LoginSysUser loginSysUser) {

		List<AccountingScoreCustomerVo> result = new ArrayList<>();
		result = accountingScoreCustomerMapperEx.getAccountingList(params);

		// 附加信息
		additionalInfoGetAccountingList(result);
		return result;
	}

	// 附加信息
	public void additionalInfoGetAccountingList(List<AccountingScoreCustomerVo> result) {

		if (AnCollectionUtils.isEmpty(result)) {
			return;
		}

		List<Long> sysUserIds = new ArrayList<>();

		List<Long> createUserIds = result.stream().filter(o -> o.getCreateUserId() != null).filter(AnLamdaUtils.distinctByKey(o -> o.getCreateUserId())).map(o -> o.getCreateUserId()).collect(Collectors.toList());
		List<Long> updateUserIds = result.stream().filter(o -> o.getUpdateUserId() != null).filter(AnLamdaUtils.distinctByKey(o -> o.getUpdateUserId())).map(o -> o.getUpdateUserId()).collect(Collectors.toList());

		sysUserIds.addAll(createUserIds);
		sysUserIds.addAll(updateUserIds);

		sysUserIds = sysUserIds.stream().distinct().collect(Collectors.toList());

		log.info("\n{}", JsonUtils.toJson(sysUserIds));

		Map<Long, SysUser> sysUserMap = sysUserService.convertSysUserIds2MapSysUserId2Entity(sysUserIds);

		for (AccountingScoreCustomerVo vo : result) {
			vo.setAccountingStatusName(AccountingOrderAccountingStatusEnum.getDesc(vo.getAccountingStatus()));
			vo.setInvoiceStatusName(AccountingOrderInvoiceStatusEnum.getDesc(vo.getInvoiceStatus()));
			vo.setSendEmailStatusName(AccountingOrderIsSendEmailEnum.getDesc(vo.getIsSendEmail()));

			// 再补个

			if (sysUserMap.get(vo.getCreateUserId()) != null) {
				vo.setCreateUserName(sysUserMap.get(vo.getCreateUserId()).getFullname());
			} else if (vo.getCreateUserId().equals(1L)) {
				vo.setCreateUserName("系统");
			}

			if (sysUserMap.get(vo.getUpdateUserId()) != null) {
				vo.setUpdateUserName(sysUserMap.get(vo.getUpdateUserId()).getFullname());
			} else if (vo.getCreateUserId().equals(1L)) {
				vo.setUpdateUserName("系统");
			}
		}

		// 这个不知道是什么需求的，这样查，会超级慢
//			for (int i = 0; i < viewUnifiedOrderGoodsList.size(); i++) {
//
//				ViewUnifiedOrderGoods item = viewUnifiedOrderGoodsList.get(i);
//
//				item.setGoodsAmount(new BigDecimal(item.getPrice().doubleValue() * item.getQuantity()));
//
//				ViewUnifiedOrderGoodsUtils.setInvStateName(item);
//				ViewUnifiedOrderGoodsUtils.setPayTypeName(item);
//
//				// 临时 查找发票信息
//
//				Map<String, Object> invQuery = new HashMap<>();
//				invQuery.put("sn", item.getOrderSn());
//				List<InvoiceApply> applies = invoiceApplyDubboConsumerService.findInvoiceBySn(invQuery);
//
//				if (applies != null && applies.size() == 1) {
//					item.setInvNumber(applies.get(0).getInvoiceNumber());
//					item.setInvSn(applies.get(0).getInvoiceNo());
//				}
//			}
	}



	/**
	 * @param params
	 * @return
	 */
	@Override
	public AccountingScoreCustomerHeadVo getAccountingListHeadSum(Map<String, Object> params) {

		return accountingScoreCustomerMapperEx.getAccountingListHeadSum(params);

	}

	/****************************************** 积分对账(结算) 对账单   end ******************************************/


	/****************************************** 积分对账(结算) 对账单 详情 start ******************************************/

	@Override
	public AccountingScoreCustomerDetailVo commonStatementOfAccountDetailDimCredit(Long accountingOrderId, LoginSysUser loginSysUser) {

		AccountingScoreCustomerDetailVo accountingScoreCustomerDetailVo = new AccountingScoreCustomerDetailVo();

		// 得到 结算单 主体 ****************************************
		// |---- 筛选条件
		List<Long> idsMain = new ArrayList<>();
		idsMain.add(accountingOrderId);

		Map<String, Object> setAccountingParams = new HashMap<>();
		setAccountingParams.put("ids", idsMain.toArray());


		log.info("commonAccountingOrderDetail -> setAccountingParams {}", JsonUtils.toJson(setAccountingParams));

		List<AccountingScoreCustomerVo> setAccountingObj = getAccountingList(setAccountingParams, loginSysUser);


		log.info("commonAccountingOrderDetail -> 查询对账单 \n{}", JsonUtils.toJson(setAccountingObj));

		if (AnCollectionUtils.isNotEmpty(setAccountingObj)) {
			accountingScoreCustomerDetailVo.setAccounting(setAccountingObj.get(0));
		}


		// 得到 结算单 X 积分流水 ****************************************
		// |---- 筛选条件
		List<Long> idsCredit = getAccountingXCreditIds(accountingOrderId);

		Map<String, Object> setCreditListParams = new HashMap<>();
		setCreditListParams.put("ids", idsCredit.toArray());
		// 客户对账:1-已对账;2-未对账 这个方法默认的是 2
		// TODO 共用会有问题 引用的地方有 查看详情 导出详情 发邮件导出详情
//		setCreditListParams.put("accountingStateCustomer", AccountingOrderAccountingStatus.已确认.getKey());

		log.info("setCreditListParams {}", JsonUtils.toJson(setCreditListParams));

		List<ScoreOrderReconciliationNotYetVo> setCreditListObj = scoreOrderService.getScoreOrderReconciliationNotYet(setCreditListParams, loginSysUser);
		accountingScoreCustomerDetailVo.setCreditList(setCreditListObj);

		return accountingScoreCustomerDetailVo;
	}

	/****************************************** 积分对账(结算) 对账单 详情   end ******************************************/


	/****************************************** 积分对账(结算) 对账单 操作 start ******************************************/


	@Transactional
	@Override
	public AccountingScoreCustomer createScoreAccountingCustomer(
			AccountingScoreCustomer accountingScoreCustomer,
			List<AccountingScoreCustomerXCredit> scoreAccountingCustomerXCreditList) {

		accountingScoreCustomer.setAccountingOrderSn(unifiedSnConfigService.getCustomerStatementOfAccountSnBySnowflake(""));

		accountingScoreCustomerMapper.insert(accountingScoreCustomer);

		for (AccountingScoreCustomerXCredit x : scoreAccountingCustomerXCreditList) {

			x.setAccountingOrderId(accountingScoreCustomer.getId());
			x.setAccountingOrderSn(accountingScoreCustomer.getAccountingOrderSn());
			accountingScoreCustomerXCreditMapper.insert(x);

			UnifiedOrderCredit scoreOrderCredit = new UnifiedOrderCredit();
			scoreOrderCredit.setId(x.getUnifiedOrderCreditId());
			scoreOrderCredit.setAccountingStateCustomer(CommonAccountingStateEnum.已对账.getKey());
			unifiedOrderCreditMapper.updateByPrimaryKeySelective(scoreOrderCredit);

		}

		return accountingScoreCustomer;

	}


	/**
	 * 拒绝对账 重新对账 取消对账
	 *
	 * @param id
	 * @param accountingStatus
	 */
	@Transactional
	@Override
	public void cancelAccounting(Long id, /* 对账单(结算单)状态:0-未确认;1-已确认;-1-已取消; */Integer accountingStatus) {


		// 查出 结算单
		AccountingScoreCustomer search = accountingScoreCustomerMapper.selectByPrimaryKey(id);


		// 查出 结算单 与 积分流水 关联
		AccountingScoreCustomerXCreditExample xExample = new AccountingScoreCustomerXCreditExample();
		xExample.createCriteria()
				.andAccountingOrderSnEqualTo(search.getAccountingOrderSn());

		List<AccountingScoreCustomerXCredit> x = accountingScoreCustomerXCreditMapper.selectByExample(xExample);

		log.info("查出 结算单 与 积分流水 关联,\n {}", JsonUtils.toPrettyJson(x));

		List<Long> creditIds = x.stream()
				.map(AccountingScoreCustomerXCredit::getUnifiedOrderCreditId)
				.collect(Collectors.toList());

		List<Long> xIds = x.stream()
				.map(AccountingScoreCustomerXCredit::getId)
				.collect(Collectors.toList());

		if (AnCollectionUtils.isNotEmpty(creditIds)) {
			// 还原积分流水 对账状态
			UnifiedOrderCredit updateSample = new UnifiedOrderCredit();
			// |---- 客户对账:1-已对账;2-未对账
			updateSample.setAccountingStateCustomer(2);

			UnifiedOrderCreditExample updateExample = new UnifiedOrderCreditExample();
			updateExample.createCriteria()
					.andIdIn(creditIds);
			unifiedOrderCreditMapper.updateByExampleSelective(updateSample, updateExample);
		}


		// 屏蔽 为了已取消 结算单能 查看详情
		// 删除 x
		// AccountingScoreCustomerXCreditExample xDeleteExample = new AccountingScoreCustomerXCreditExample();
		// xDeleteExample.createCriteria()
		// 		.andIdIn(xIds);
		// accountingScoreCustomerXCreditMapper.deleteByExample(xDeleteExample);


		// 更新 结算单
		AccountingScoreCustomer accountingScoreCustomer = new AccountingScoreCustomer();
		accountingScoreCustomer.setId(id);
		accountingScoreCustomer.setAccountingStatus(accountingStatus);// 对账单(结算单)状态:0-未确认;1-已确认;-1-已取消;2-是什么鬼;

		accountingScoreCustomerMapper.updateByPrimaryKeySelective(accountingScoreCustomer);
	}

	/**
	 * 删除对账单
	 *
	 * @param id
	 */
	public void deleteAccounting(Long id) {

		// 查出 结算单
		AccountingScoreCustomer search = accountingScoreCustomerMapper.selectByPrimaryKey(id);

		if (search == null) {
			return;
		}

		// 查出 结算单 与 积分流水 关联
		AccountingScoreCustomerXCreditExample xExample = new AccountingScoreCustomerXCreditExample();
		xExample.createCriteria()
				.andAccountingOrderSnEqualTo(search.getAccountingOrderSn());

		List<AccountingScoreCustomerXCredit> x = accountingScoreCustomerXCreditMapper.selectByExample(xExample);

		log.info("查出 结算单 与 积分流水 关联,\n {}", JsonUtils.toPrettyJson(x));

		List<Long> xIds = x.stream()
				.map(AccountingScoreCustomerXCredit::getId)
				.collect(Collectors.toList());

		// 删除关联
		if (AnCollectionUtils.isNotEmpty(xIds)) {
			// 删除 x
			AccountingScoreCustomerXCreditExample xDeleteExample = new AccountingScoreCustomerXCreditExample();
			xDeleteExample.createCriteria()
					.andIdIn(xIds);
			accountingScoreCustomerXCreditMapper.deleteByExample(xDeleteExample);
		}

		// 删除主体
		// accountingScoreCustomerMapper.deleteByPrimaryKey(id);
		// TODO 改为软删除
		AccountingScoreCustomer forSoftDelete = new AccountingScoreCustomer();
		forSoftDelete.setId(id);
		forSoftDelete.setDeleted(CommonSoftDeleteEnum.删除.getKey());
		accountingScoreCustomerMapper.updateByPrimaryKeySelective(forSoftDelete);
	}

	/****************************************** 积分对账(结算) 对账单 操作   end ******************************************/


	@Override
	public PageInfo<AccountingScoreCustomer> findByPaging(Map<String, Object> params, Integer pageNo, Integer pageSize) {
		return null;
	}

	@Override
	public void deleteObj(Long id) {
		accountingScoreCustomerMapper.deleteByPrimaryKey(id);
	}

	@Override
	public void updateObj(AccountingScoreCustomer scoreAccountingCustomer) {

		accountingScoreCustomerMapper.updateByPrimaryKeySelective(scoreAccountingCustomer);
	}

	@Override
	public AccountingScoreCustomer findById(Long id) {
		return accountingScoreCustomerMapper.selectByPrimaryKey(id);
	}

	@Override
	public void addObj(AccountingScoreCustomer scoreAccountingCustomer) {
		accountingScoreCustomerMapper.insert(scoreAccountingCustomer);
	}

	@Override
	public List<AccountingScoreCustomer> findAll(Map<String, Object> queryMap) {
		return null;
	}


}
