package cn.superdesk.uniorder.service.base;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.superdesk.uniorder.common.enums.common.CommonAccountingLockingEnum;
import cn.superdesk.uniorder.common.enums.common.CommonOrderTypeEnum;
import cn.superdesk.uniorder.dao.entity.base.*;
import cn.superdesk.uniorder.dao.mapper.base.UnifiedOrderGoodsMapper;
import cn.superdesk.uniorder.dao.mapper.base.UnifiedOrderMapper;
import cn.superdesk.uniorder.dao.mapper.base.UnifiedOrderMapperEx;
import cn.superdesk.uniorder.dao.mapper.base.UnifiedOrderMapperExCustomerDimCredit;
import cn.superdesk.uniorder.dto.param.operation.UnifiedOrderIssuesFlowingParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Description: 退单为主休逻辑
 * @Author: linjinyu
 * @CreateDate: 2021/7/1 下午5:22
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021/7/1 下午5:22
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Service
public class CommonMergeRefundOrderProcessService {

	@Autowired
	private UnifiedOrderMapper                    unifiedOrderMapper;
	@Autowired
	private UnifiedOrderMapperEx                  unifiedOrderMapperEx;
	@Autowired
	private UnifiedOrderMapperExCustomerDimCredit unifiedOrderMapperExScore;

	@Autowired
	private UnifiedOrderGoodsMapper unifiedOrderGoodsMapper;


	/**
	 * 双主体情况 - 退单作为新主体模式 对标 processScoreOrderRefundUnifiedOrder 差别就是不核算了
	 *
	 * @param unifiedOrderReturn
	 * @return
	 */
	public UnifiedOrder processScoreOrderRefundUnifiedOrder(UnifiedOrderReturn unifiedOrderReturn) {

		UnifiedOrder unifiedOrder = unifiedOrderMapperEx.findByOrderSn(unifiedOrderReturn.getOrderSn());
		UnifiedOrder refundOrder  = unifiedOrderMapperEx.findByOrderSn(unifiedOrderReturn.getReturnSn());

		// 如果 正单没有 就不处理这个退单
		if (null == unifiedOrder) {
			// TODO 记录错误
			return null;
		}

		// 退单不存在
		if (null == refundOrder) {

			refundOrder = new UnifiedOrder();

			copyFieldToRefundEntity(refundOrder, unifiedOrder);
			writeFiledToRefundEntity(refundOrder, unifiedOrderReturn);
			unifiedOrderMapper.insertSelective(refundOrder);
		}
		// 退单存在
		else {

			copyFieldToRefundEntity(refundOrder, unifiedOrder);
			writeFiledToRefundEntity(refundOrder, unifiedOrderReturn);
			unifiedOrderMapper.updateByPrimaryKeySelective(refundOrder);
		}

		reverseUpdateOrderFieldRefundOrderSn(unifiedOrder, unifiedOrderReturn);

		return refundOrder;

	}

	/**
	 * 更新正单的 refundOrderSn 字段
	 *
	 * @param unifiedOrder
	 * @param unifiedOrderReturn
	 */
	private void reverseUpdateOrderFieldRefundOrderSn(UnifiedOrder unifiedOrder, UnifiedOrderReturn unifiedOrderReturn) {

		UnifiedOrder unifiedOrder4UpdateRefundOrderSn = new UnifiedOrder();
		unifiedOrder4UpdateRefundOrderSn.setId(unifiedOrder.getId());
		unifiedOrder4UpdateRefundOrderSn.setRefundOrderSn(unifiedOrderReturn.getReturnSn());

		unifiedOrderMapper.updateByPrimaryKeySelective(unifiedOrder4UpdateRefundOrderSn);

	}

	/**
	 * 拷贝字段 TODO 因为过程可能有变，但也不是全部属性都会覆写更新的
	 *
	 * @param unifiedOrder
	 * @param refundOrder
	 */
	private void copyFieldToRefundEntity(UnifiedOrder refundOrder, UnifiedOrder unifiedOrder) {

		refundOrder.setServiceNo(unifiedOrder.getServiceNo());
		refundOrder.setServiceName(unifiedOrder.getServiceName());

		refundOrder.setPurchasePhone(unifiedOrder.getPurchasePhone());
		refundOrder.setPurchaseUserId(unifiedOrder.getPurchaseUserId());
		refundOrder.setPurchaseUserName(unifiedOrder.getPurchaseUserName());

		refundOrder.setPurchaseVirtId(unifiedOrder.getPurchaseVirtId());
		refundOrder.setPurchaseVirtName(unifiedOrder.getPurchaseVirtName());

		refundOrder.setProjectId(unifiedOrder.getProjectId());
		refundOrder.setProjectName(unifiedOrder.getProjectName());

		refundOrder.setSaleId(unifiedOrder.getSaleId());
		refundOrder.setSaleName(unifiedOrder.getSaleName());

		// TODO 写死
		refundOrder.setSettlementType(2);
		// TODO 写死
		refundOrder.setSourceChannel(1);
		// TODO 写死
		refundOrder.setAppraiseState(2);

	}

	/**
	 * 覆写字段
	 *
	 * @param refundOrder
	 * @param unifiedOrderReturn
	 */
	private void writeFiledToRefundEntity(UnifiedOrder refundOrder, UnifiedOrderReturn unifiedOrderReturn) {

		refundOrder.setOrderType(CommonOrderTypeEnum.退单.getKey());

		// 这个逻辑就是这样，不要问，自己想想
		// 退单主体
		// 订单单号字段 存 退单单号
		// 退单单号字段 存 正单单号
		refundOrder.setOrderSn(unifiedOrderReturn.getReturnSn());
		refundOrder.setRefundOrderSn(unifiedOrderReturn.getOrderSn());

		refundOrder.setCompletState(unifiedOrderReturn.getCompletState());

		// 以下还是要验证一下
		refundOrder.setPayState(unifiedOrderReturn.getPayState());

		// 订单状态
		refundOrder.setOrderState(unifiedOrderReturn.getOrderState());
		refundOrder.setOrderStateName(unifiedOrderReturn.getOrderStateName());

		refundOrder.setCreateTime(unifiedOrderReturn.getCreateTime());
		refundOrder.setPayTime(unifiedOrderReturn.getPayTime());
		refundOrder.setSendTime(unifiedOrderReturn.getSendTime());
		refundOrder.setFinishTime(unifiedOrderReturn.getFinishTime());

	}

	/**
	 * 锁定
	 *
	 * @param orderSn
	 */
	public void lockByOrderSn(String orderSn) {
		processBizGoodsLocking(orderSn, CommonAccountingLockingEnum.锁定);
	}

	/**
	 * 解锁
	 *
	 * @param orderSn
	 */
	public void unlockByOrderSn(String orderSn) {
		processBizGoodsLocking(orderSn, CommonAccountingLockingEnum.正常);
	}

	/**
	 * 处理 结算时间 填充
	 *
	 * @param orderSn
	 * @param refundOrderSn
	 * @param refundOrderfinishTime
	 */
	public void processBizOrderRecordAccountingTime(String orderSn, String refundOrderSn, Date refundOrderfinishTime, UnifiedOrderIssuesFlowingParam callback) {

		List<String> updateOrderSnList = new ArrayList<>();

		if (StrUtil.isNotBlank(orderSn)) {
			updateOrderSnList.add(orderSn);
		}

		if (StrUtil.isNotBlank(refundOrderSn)) {
			updateOrderSnList.add(refundOrderSn);
		}

		if (CollUtil.isNotEmpty(updateOrderSnList) && ObjectUtil.isNotEmpty(refundOrderfinishTime)) {

			UnifiedOrderExample unifiedOrderExample = new UnifiedOrderExample();
			unifiedOrderExample
					.createCriteria()
					.andOrderSnIn(updateOrderSnList);

			UnifiedOrder unifiedOrder4Update = new UnifiedOrder();
			unifiedOrder4Update.setAccountingTime(refundOrderfinishTime);
			unifiedOrderMapper.updateByExampleSelective(unifiedOrder4Update, unifiedOrderExample);
		} else {
			StringBuffer processReport = new StringBuffer(callback.getProcessReport());
			processReport.append("订单号或退单号或退单完成时间缺失，结算时间填充失败;");

			callback.setProcessReport(processReport.toString());
		}

	}

	private void processBizGoodsLocking(String orderSn, CommonAccountingLockingEnum lockingEnum) {

		UnifiedOrderGoodsExample example = new UnifiedOrderGoodsExample();
		example.createCriteria()
				.andOrderSnEqualTo(orderSn);

		UnifiedOrderGoods unifiedOrderGoods4Lock = new UnifiedOrderGoods();
		unifiedOrderGoods4Lock.setGoodsLockingSupplier(lockingEnum.getKey());

		unifiedOrderGoodsMapper.updateByExampleSelective(unifiedOrderGoods4Lock, example);
	}
}
