package cn.superdesk.uniorder.service.fund.impl;

import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.exception.constants.ExceptionMsg;
import cn.superdesk.uniorder.common.exception.UnifiedOrderRuntimeException;
import cn.superdesk.uniorder.dao.entity.fund.SysFundActivated;
import cn.superdesk.uniorder.dao.entity.fund.SysFundGrant;
import cn.superdesk.uniorder.dao.entity.fund.SysFundUser;
import cn.superdesk.uniorder.dao.mapper.fund.SysFundActivatedMapper;
import cn.superdesk.uniorder.dao.mapper.fund.SysFundUserMapper;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dto.vo.fund.SysFundActivatedVoExt;
import cn.superdesk.uniorder.dto.vo.fund.SysFundUserVoExt;
import cn.superdesk.uniorder.service.fund.SysFundActivatedService;
import cn.superdesk.uniorder.service.fund.SysFundGrantService;
import cn.superdesk.uniorder.service.fund.SysFundUserService;
import cn.superdesk.uniorder.common.numerical.AnDateHelper;
import cn.superdesk.uniorder.common.security.JwtUtil;
import cn.superdesk.uniorder.common.security.PushUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
@Transactional
public class SysFundActivatedServiceImpl implements SysFundActivatedService {


	@Value("${h5.url}")
	private String h5url;
	@Value("${pushMessage.orgId}")
	private String orgId;
	@Value("${pushMessage.url}")
	private String pushMessageUrl;


	@Autowired
	private SysFundUserMapper      fundUserMapper;
	@Autowired
	private SysFundActivatedMapper fundActivatedMapper;
	@Autowired
	private SysFundGrantService    sysFundGrantService;
	@Autowired
	private SysFundUserService     sysFundUserService;


	@Override
	public PageInfo<SysFundActivated> findByPaging(Map<String, Object> params, Integer pageNo, Integer pageSize) {
		return null;
	}

	@Override
	public void deleteObj(Long id) {
		fundActivatedMapper.deleteByPrimaryKey(id);
	}

	@Override
	public void updateObj(SysFundActivated sysFundActivated) {
		sysFundActivated.setCheckTime(AnDateHelper.getNow());
		fundActivatedMapper.updateByPrimaryKeySelective(sysFundActivated);
	}

	@Override
	public SysFundActivated findById(Long id) {
		return fundActivatedMapper.selectByPrimaryKey(id);
	}

	@Override
	public void addObj(SysFundActivated sysFundActivated) {
		sysFundActivated.setCreateTime(AnDateHelper.getNow());
		fundActivatedMapper.insertSelective(sysFundActivated);
	}

	@Override
	public List<SysFundActivated> findAll(Map<String, Object> queryMap) {
		return null;
	}

	@Override
	public void create(String token, SysFundActivated sysFundActivated) throws UnifiedOrderRuntimeException {

		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);

		if (StringUtils.isBlank(sysFundActivated.getInvoiceNo())) {
			throw new UnifiedOrderRuntimeException(ExceptionMsg.SYS_ERROR.getCode(), "发票号不能为空");
		}
		if (sysFundActivated.getCheckCount() == null) {
			throw new UnifiedOrderRuntimeException(ExceptionMsg.SYS_ERROR.getCode(), "核销金额不能为空");
		}
		if (sysFundActivated.getFundCount() == null) {
			throw new UnifiedOrderRuntimeException(ExceptionMsg.SYS_ERROR.getCode(), "激活基金金额不能为空");
		}

		SysFundGrant sysFundGrant = sysFundGrantService.findByInvoiceNo(sysFundActivated.getInvoiceNo());
		if (sysFundGrant == null) {
			throw new UnifiedOrderRuntimeException(ExceptionMsg.SYS_ERROR.getCode(), "发票号不存在");
		}

		if (sysFundActivated.getFundUserId() == null) {
			throw new UnifiedOrderRuntimeException(ExceptionMsg.SYS_ERROR.getCode(), "基金用户Id不能为空");
		}
		SysFundUser sysFundUser = sysFundUserService.findById(sysFundActivated.getFundUserId());

		// 检测核销金额是否大于发票金额
//		sysFundGrant.getInvoiceCount().compareTo(sysFundActivated.getCheckCount()) == -1
		if (sysFundGrant.getInvoiceCount().compareTo(sysFundActivated.getCheckCount()) < 0) {
			throw new UnifiedOrderRuntimeException(ExceptionMsg.SYS_ERROR.getCode(), "核销金额过大");
		}
		// 检测激活基金金额是否大于待激活基金金额
		int flag = sysFundGrant.getNoActivatedFund().compareTo(sysFundActivated.getFundCount());
		if (flag == -1) {
			throw new UnifiedOrderRuntimeException(ExceptionMsg.SYS_ERROR.getCode(), "激活基金金额过大");
		} else if (flag == 0) {
			sysFundGrant.setActivatedStatus(2);
		} else {
			sysFundGrant.setActivatedStatus(1);
		}
		//未核销金额-核销金额
		sysFundGrant.setNoCheckCount(sysFundGrant.getNoCheckCount().subtract(sysFundActivated.getCheckCount()));
		//待激活基金-激活基金
		sysFundGrant.setNoActivatedFund(sysFundGrant.getNoActivatedFund().subtract(sysFundActivated.getFundCount()));
		sysFundGrantService.updateObj(sysFundGrant);

		//待激活基金-激活基金
		sysFundUser.setNoActivatedFund(sysFundUser.getNoActivatedFund().subtract(sysFundActivated.getFundCount()));
		//发放基金余额+激活基金
		sysFundUser.setActivatedFundCount(sysFundUser.getActivatedFundCount().add(sysFundActivated.getFundCount()));
		sysFundUser.setUpdateUserId(loginSysUser.getSysUserId());
		sysFundUserService.updateObj(sysFundUser);

		//添加记录
		sysFundActivated.setCreateUserId(loginSysUser.getSysUserId());
		sysFundActivated.setCreateTime(AnDateHelper.getNow());
		fundActivatedMapper.insertSelective(sysFundActivated);

		Map<String, Object> params = new HashMap<>();
		params.put("userId", sysFundUser.getUserId());
		List<SysFundUserVoExt> sysFundUserVOList = fundUserMapper.selectByCondition(params);
		SysFundUserVoExt       sysFundUserVO     = sysFundUserVOList.get(0);
		String res = PushUtil.pushActivatedMessage(
				orgId,
				pushMessageUrl,
				h5url,
				sysFundUserVO.getUserId().toString(),
				sysFundUserVO.getRealname(),
				sysFundUserVO.getMobile(),
				sysFundUserVO.getOrganizationName(),
				sysFundActivated.getFundCount().toString(),
				sysFundUserVO.getGrantFundBalance().toString());
		log.info("电话：" + sysFundUserVO.getMobile() + "到账短信推送========>" + res);
	}

	@Override
	public void update(String token, SysFundActivated sysFundActivated) throws UnifiedOrderRuntimeException {
		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);

		if (sysFundActivated.getId() == null) {
			throw new UnifiedOrderRuntimeException(ExceptionMsg.SYS_ERROR.getCode(), "Id不能为空");
		}

		sysFundActivated.setActivatedStatus(1);
		sysFundActivated.setCreateUserId(loginSysUser.getSysUserId());
		sysFundActivated.setCheckTime(AnDateHelper.getNow());
		fundActivatedMapper.updateByPrimaryKeySelective(sysFundActivated);
	}

	@Override
	public R<List<SysFundActivatedVoExt>> findAllViewPage(Map<String, Object> params, int pageNum, int pageSize) {

		PageHelper.startPage(pageNum, pageSize);
		List<SysFundActivatedVoExt>     ls   = fundActivatedMapper.selectByCondition(params);
		PageInfo<SysFundActivatedVoExt> page = new PageInfo<SysFundActivatedVoExt>(ls);
		PageHelper.clearPage();
		return new R<List<SysFundActivatedVoExt>>(page.getPageNum(), page.getPageSize(), (int) page.getTotal(), ls);

	}

	@Override
	public void importExcel(String token, Map<String, List<Map<String, Object>>> map) throws UnifiedOrderRuntimeException {

		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);

		// 保存推送信息，避免回滚依旧发送短信
		List<Map<String, Object>> pushMessageList = new ArrayList<>();

		// 循环mobile分类
		for (Map.Entry<String, List<Map<String, Object>>> entry : map.entrySet()) {
			Map<String, Object> pushMessageParams = new HashMap<>();
			String              mobile            = entry.getKey();

			Map<String, Object> params = new HashMap<>();
			params.put("mobile", mobile);

			List<SysFundUserVoExt> sysFundUserVOList = fundUserMapper.selectByCondition(params);

			if (sysFundUserVOList == null || sysFundUserVOList.size() == 0) {
				throw new UnifiedOrderRuntimeException(ExceptionMsg.SYS_ERROR.getCode(), "电话：" + mobile + "得基金用户不存在");
			}

			SysFundUserVoExt sysFundUserVO = sysFundUserVOList.get(0);

			List<Map<String, Object>> list = map.get(mobile);

			BigDecimal allActivatedFundCount = new BigDecimal("0");
			BigDecimal allFundCount          = new BigDecimal("0");

			for (Map<String, Object> map2 : list) {
				SysFundActivated sysFundActivated = new SysFundActivated();

				String invoiceNo = map2.get("invoiceNo").toString();
				if (StringUtils.isBlank(invoiceNo)) {
					throw new UnifiedOrderRuntimeException(ExceptionMsg.SYS_ERROR.getCode(), "存在发票号为空");
				}
				BigDecimal checkCount = new BigDecimal(map2.get("checkCount").toString());
				if (checkCount == null) {
					throw new UnifiedOrderRuntimeException(ExceptionMsg.SYS_ERROR.getCode(), "存在核销金额为空");
				}
				BigDecimal fundCount = new BigDecimal(map2.get("fundCount").toString());
				if (fundCount == null) {
					throw new UnifiedOrderRuntimeException(ExceptionMsg.SYS_ERROR.getCode(), "存在基金金额为空");
				}
				sysFundActivated.setInvoiceNo(invoiceNo);
				sysFundActivated.setCheckCount(checkCount);
				sysFundActivated.setFundCount(fundCount);
				sysFundActivated.setFundUserId(sysFundUserVO.getId());

				SysFundGrant sysFundGrant = sysFundGrantService.findByInvoiceNo(sysFundActivated.getInvoiceNo());
				if (sysFundGrant == null) {
					throw new UnifiedOrderRuntimeException(ExceptionMsg.SYS_ERROR.getCode(), invoiceNo + "该发票号不存在");
				}
				// 检测核销金额是否大于发票金额
//				sysFundGrant.getInvoiceCount().compareTo(sysFundActivated.getCheckCount()) == -1
				if (sysFundGrant.getInvoiceCount().compareTo(sysFundActivated.getCheckCount()) < 0) {
					throw new UnifiedOrderRuntimeException(ExceptionMsg.SYS_ERROR.getCode(), "核销金额过大");
				}
				// 检测激活基金金额是否大于待激活基金金额
				int flag = sysFundGrant.getNoActivatedFund().compareTo(sysFundActivated.getFundCount());
//				flag == -1
				if (flag < 0) {
					throw new UnifiedOrderRuntimeException(ExceptionMsg.SYS_ERROR.getCode(), "激活基金过大");
				} else if (flag == 0) {
					sysFundGrant.setActivatedStatus(2);
				} else {
					sysFundGrant.setActivatedStatus(1);
				}
				// 未核销金额-核销金额
				sysFundGrant.setNoCheckCount(sysFundGrant.getNoCheckCount().subtract(sysFundActivated.getCheckCount()));
				// 待激活基金-激活基金
				sysFundGrant.setNoActivatedFund(sysFundGrant.getNoActivatedFund().subtract(sysFundActivated.getFundCount()));
				sysFundGrantService.updateObj(sysFundGrant);

				SysFundUser sysFundUser = sysFundUserService.findById(sysFundActivated.getFundUserId());

				// 待激活基金-激活基金
				sysFundUser.setNoActivatedFund(sysFundUser.getNoActivatedFund().subtract(sysFundActivated.getFundCount()));
				// 发放基金余额+激活基金
				sysFundUser.setActivatedFundCount(sysFundUser.getActivatedFundCount().add(sysFundActivated.getFundCount()));
				sysFundUser.setUpdateUserId(loginSysUser.getSysUserId());
				sysFundUserService.updateObj(sysFundUser);

				// 添加记录
				sysFundActivated.setCreateUserId(loginSysUser.getSysUserId());
				sysFundActivated.setCreateTime(AnDateHelper.getNow());
				fundActivatedMapper.insertSelective(sysFundActivated);

				// 把统一基金用户的开票金额和基金金额加起来
				allFundCount = allFundCount.add(sysFundGrant.getFundCount());
				allActivatedFundCount = sysFundUser.getActivatedFundCount();
			}

			pushMessageParams.put("id", sysFundUserVO.getUserId());
			pushMessageParams.put("mobile", mobile);
			pushMessageParams.put("realname", sysFundUserVO.getRealname());
			pushMessageParams.put("organizationName", sysFundUserVO.getOrganizationName());
			pushMessageParams.put("fundCount", allFundCount);
			pushMessageParams.put("grantFundBalance", allActivatedFundCount.subtract(sysFundUserVO.getConsumerFund()));

			pushMessageList.add(pushMessageParams);

		}
		for (Map<String, Object> pushMessageParams : pushMessageList) {

			String res = PushUtil.pushActivatedMessage(
					orgId,
					pushMessageUrl,
					h5url,
					pushMessageParams.get("id").toString(),
					pushMessageParams.get("realname").toString(),
					pushMessageParams.get("mobile").toString(),
					pushMessageParams.get("organizationName").toString(),
					pushMessageParams.get("fundCount").toString(),
					pushMessageParams.get("grantFundBalance").toString()
			);

			log.info("电话：" + pushMessageParams.get("mobile").toString() + "到账短信推送========>" + res + "==内容======>" + pushMessageParams);
		}
	}
}
