package cn.superdesk.uniorder.service.invoice.customer;

import cn.superdesk.uniorder.common.enums.statement.AccountingOrderTypeDimEnum;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.enums.invoice.InvoiceApplyBackAccountStatusEnum;
import cn.superdesk.uniorder.common.exception.UnifiedOrderRuntimeException;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dao.entity.invoice.customer.InvoiceApply;
import cn.superdesk.uniorder.dao.entity.order.ExpressCompany;
import cn.superdesk.uniorder.dto.param.invoice.ExportExcelInvoiceApplyQueryCriteria;
import cn.superdesk.uniorder.dto.param.invoice.customer.InvoiceApplyCommonParams;
import cn.superdesk.uniorder.dto.vo.invoice.customer.*;
import cn.superdesk.uniorder.event.invoiceApplyTrigger.TailUpdateAccountingOrderInvoiceStatusEvent;

import java.util.List;
import java.util.Map;

/**
 * @Description: 开票申请 发票管理
 * @Author: linjinyu 重构
 * @CreateDate: 7/28/20 7:52 PM
 * @UpdateUser: linjinyu
 * @UpdateDate: 7/28/20 7:52 PM
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
public interface InvoiceApplyService {

	int updateInvoiceApplyByPrimaryKeySelective(InvoiceApply invoiceApply);

	int updateFieldBackAccountStatusByCheckRatio(Integer fullCount, Integer seizeCount, Long invoiceApplyId);

	int updateFieldBackAccountStatusByInvoiceApplyId(Long invoiceApplyId, InvoiceApplyBackAccountStatusEnum invoiceApplyBackAccountStatusEnum);

	List<InvoiceApplySelectScoreVo> invoiceApplySelectFocusUnifiedDimOrder(Map<String, Object> params) throws UnifiedOrderRuntimeException;

	List<InvoiceApplySelectScoreVo> invoiceApplySelectFocusUnifiedDimGoods(Map<String, Object> params) throws UnifiedOrderRuntimeException;

	List<InvoiceApplySelectScoreVo> invoiceApplySelectFocusScoreDimCredit(ExportExcelInvoiceApplyQueryCriteria criteria) throws UnifiedOrderRuntimeException;


	/**
	 * 共用 程序 里边有分支
	 *
	 * @param token
	 * @param params
	 * @throws UnifiedOrderRuntimeException
	 */
	TailUpdateAccountingOrderInvoiceStatusEvent invoiceApply(String token, InvoiceApplyCommonParams params) throws UnifiedOrderRuntimeException;


	/****************************************** 开票申请 列表的导出Excel数据 start ******************************************/
	/**
	 * 客户结算-发票申请管理 列表-opt 导出报表
	 *
	 * @param params
	 * @return
	 * @throws UnifiedOrderRuntimeException
	 */
	List<ExportExcelInvoiceApplyForceUnifiedDimGoodsVo> findInvoiceApplyFocusUnifiedDimGoods(Map<String, Object> params) throws UnifiedOrderRuntimeException;

	/**
	 * 积分结算-发票申请管理 列表-opt 导出报表
	 *
	 * @param criteria
	 * @return
	 * @throws UnifiedOrderRuntimeException
	 */
	List<ExportExcelInvoiceApplyForceScoreDimCreditVo> findInvoiceApplyFocusScoreDimCredit(ExportExcelInvoiceApplyQueryCriteria criteria) throws UnifiedOrderRuntimeException;
	/****************************************** 开票申请 列表的导出Excel数据   end ******************************************/

	/****************************************** 发票申请管理 start ******************************************/
	R<List<InvoiceApplyPage>> findViewByPageFocusUnified(Map<String, Object> params, int pageNum, int pageSize) throws UnifiedOrderRuntimeException;

	InvoiceApplyDataVo findViewHeadDataFocusUnified(Map<String, Object> params) throws UnifiedOrderRuntimeException;

	R<List<InvoiceApplyPage>> findViewByPageFocusScore(ExportExcelInvoiceApplyQueryCriteria criteria, int pageNum, int pageSize) throws UnifiedOrderRuntimeException;

	InvoiceApplyDataVo findViewHeadDataFocusScore(ExportExcelInvoiceApplyQueryCriteria criteria) throws UnifiedOrderRuntimeException;

	/**
	 * 开票申请后 反向更新对账单 开票状态
	 * xml
	 * 0 未开票
	 * 1 全部开票
	 * 2 部分开票
	 * bean
	 * 0-未开票;1-已开票;2-部分开票;
	 *
	 * @param affectedAccountingOrderIds
	 * @param dim
	 */
	void processTailUpdateAccountingOrderInvoiceStatus(List<Long> affectedAccountingOrderIds, AccountingOrderTypeDimEnum dim);

	/****************************************** 发票申请管理   end ******************************************/






	InvoiceApplyDetailDimGoodsVo getUnifiedCustomerInvoice2GoodsDetail(Map<String, Object> params) throws UnifiedOrderRuntimeException;

	InvoiceApplyDetailDimCreditVo getUnifiedCustomerInvoice2CreditDetail(ExportExcelInvoiceApplyQueryCriteria criteria, LoginSysUser loginSysUser) throws UnifiedOrderRuntimeException;



	/**
	 * 积分结算-发票管理-导入寄票信息(批量操作)
	 *
	 * @param token
	 * @param list
	 * @throws UnifiedOrderRuntimeException
	 */
	String importInvoiceExcel(String token, List<Map<String, Object>> list) throws UnifiedOrderRuntimeException;

	/**
	 * 积分结算-发票管理-导入发票信息(批量操作)
	 *
	 * @param token
	 * @param list
	 * @throws UnifiedOrderRuntimeException
	 */
	String importSendInvoiceExcel(String token, List<Map<String, Object>> list) throws UnifiedOrderRuntimeException;


	List<ExpressCompany> findExpressCompany() throws UnifiedOrderRuntimeException;

	List<InvoiceApply> findAll(Map<String, Object> params);

	InvoiceApply findById(Long invoiceApplyId);

	R<List<InvoiceApply>> findInvoiceByPageNew(Map<String, Object> params, int pageNum, int pageSize);


	List<InvoiceApply> findInvoiceByOrderSn(Map<String, Object> invQuery);

	List<InvoiceApply> query4CheckIsOverdue();

	Map<String, InvoiceApply> findInvoiceByOrderSnList(List<String> orderSnList);


	void validatedInvoiceApplyRelationTable();


	TailUpdateAccountingOrderInvoiceStatusEvent opAudit(Map<String, Object> params, LoginSysUser loginSysUser) throws UnifiedOrderRuntimeException;

	/**
	 * 重新开票
	 *
	 * @param params
	 * @param loginSysUser
	 * @return
	 * @throws UnifiedOrderRuntimeException
	 */
	TailUpdateAccountingOrderInvoiceStatusEvent opAgain(Map<String, Object> params, LoginSysUser loginSysUser) throws UnifiedOrderRuntimeException;

	void opDelete(Long invoiceApplyId, LoginSysUser loginSysUser);

	void opRegister(Map<String, Object> params, LoginSysUser loginSysUser) throws UnifiedOrderRuntimeException;

	/**
	 * @param sendInvoiceRegisterList
	 * @param loginSysUser
	 * @throws UnifiedOrderRuntimeException
	 */
	void invoiceRegisterSend(List<Map> sendInvoiceRegisterList, LoginSysUser loginSysUser) throws UnifiedOrderRuntimeException;

}
