package cn.superdesk.uniorder.service.invoice.customer.impl;

import cn.superdesk.uniorder.common.enums.invoice.InvoiceApplyInvoiceProgressingStatusEnum;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dao.entity.invoice.customer.InvoiceApply;
import cn.superdesk.uniorder.pattern.state.invoice.customer.invoiceApply.invoiceProgressingStatus.context.InvoiceApplyFieldInvoiceProgressingStatusContext;
import cn.superdesk.uniorder.service.invoice.customer.InvoiceApplyFieldInvoiceProgressingStatusService;
import cn.superdesk.uniorder.service.invoice.customer.InvoiceApplyService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * @Description: 简易状态机 星型结构
 * @Author: linjinyu
 * @CreateDate: 10/12/20 4:28 PM
 * @UpdateUser: linjinyu
 * @UpdateDate: 10/12/20 4:28 PM
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Service
public class InvoiceApplyFieldInvoiceProgressingStatusServiceImpl implements InvoiceApplyFieldInvoiceProgressingStatusService {

	@Autowired
	private InvoiceApplyService invoiceApplyService;

	/****************************************** 状态机 公共方法 机内使用 实际流转 重用 start ******************************************/

	// 状态流转为 待开票(0, "待开票")
	public void opInitByStateMachine(InvoiceApply entity, LoginSysUser loginSysUser) {

		InvoiceApply forUpdate = new InvoiceApply();
		forUpdate.setId(entity.getId());

		// TODO 更新状态逻辑...

//		forUpdate.setInvoiceProgressingStatus(InvoiceApplyInvoiceProgressingStatusEnum.待开票.getKey());
		forUpdate.setInvoiceStatus(InvoiceApplyInvoiceProgressingStatusEnum.待开票.getKey());

		forUpdate.setInvoiceProgressingUserId(loginSysUser.getSysUserId());
		forUpdate.setInvoiceProgressingTime(new Date());

		invoiceApplyService.updateInvoiceApplyByPrimaryKeySelective(forUpdate);
	}

	// 状态流转为 已作废(2, "已作废")
	public void opBecomeInvalidByStateMachine(InvoiceApply entity, LoginSysUser loginSysUser) {

		InvoiceApply forUpdate = new InvoiceApply();
		forUpdate.setId(entity.getId());

		// TODO 更新状态逻辑...

//		forUpdate.setInvoiceProgressingStatus(InvoiceApplyInvoiceProgressingStatusEnum.已作废.getKey());
		forUpdate.setInvoiceStatus(InvoiceApplyInvoiceProgressingStatusEnum.已作废.getKey());

		forUpdate.setInvoiceProgressingUserId(loginSysUser.getSysUserId());
		forUpdate.setInvoiceProgressingTime(new Date());


		invoiceApplyService.updateInvoiceApplyByPrimaryKeySelective(forUpdate);
	}

	// 状态流转为 开票中(10, "开票中")
	public void opOngoingByStateMachine(InvoiceApply entity, LoginSysUser loginSysUser) {

		InvoiceApply forUpdate = new InvoiceApply();
		forUpdate.setId(entity.getId());

		// TODO 更新状态逻辑...

//		forUpdate.setInvoiceProgressingStatus(InvoiceApplyInvoiceProgressingStatusEnum.开票中.getKey());
		forUpdate.setInvoiceStatus(InvoiceApplyInvoiceProgressingStatusEnum.开票中.getKey());

		forUpdate.setInvoiceProgressingUserId(loginSysUser.getSysUserId());
		forUpdate.setInvoiceProgressingTime(new Date());

		invoiceApplyService.updateInvoiceApplyByPrimaryKeySelective(forUpdate);
	}

	// 状态流转为 已开票(11, "已开票")
	public void opCompleteByStateMachine(InvoiceApply entity, LoginSysUser loginSysUser) {

		InvoiceApply forUpdate = new InvoiceApply();
		forUpdate.setId(entity.getId());

		// TODO 更新状态逻辑...

//		forUpdate.setInvoiceProgressingStatus(InvoiceApplyInvoiceProgressingStatusEnum.已开票.getKey());
		forUpdate.setInvoiceStatus(InvoiceApplyInvoiceProgressingStatusEnum.已开票.getKey());

		forUpdate.setInvoiceProgressingUserId(loginSysUser.getSysUserId());
		forUpdate.setInvoiceProgressingTime(new Date());

		invoiceApplyService.updateInvoiceApplyByPrimaryKeySelective(forUpdate);
	}

	/****************************************** 状态机 公共方法 机内使用 实际流转 重用   end ******************************************/


	/****************************************** 状态机 外部使用 类门面 start ******************************************/

	// 状态流转为 待开票(0, "待开票")
	public void opInit(InvoiceApply entity, LoginSysUser loginSysUser) {
		InvoiceApplyFieldInvoiceProgressingStatusContext context = new InvoiceApplyFieldInvoiceProgressingStatusContext(entity, loginSysUser);
		context.opInit();
	}

	// 状态流转为 已作废(2, "已作废")
	public void opBecomeInvalid(InvoiceApply entity, LoginSysUser loginSysUser) {
		InvoiceApplyFieldInvoiceProgressingStatusContext context = new InvoiceApplyFieldInvoiceProgressingStatusContext(entity, loginSysUser);
		context.opBecomeInvalid();
	}

	// 状态流转为 开票中(10, "开票中")
	public void opOngoing(InvoiceApply entity, LoginSysUser loginSysUser) {
		InvoiceApplyFieldInvoiceProgressingStatusContext context = new InvoiceApplyFieldInvoiceProgressingStatusContext(entity, loginSysUser);
		context.opOngoing();
	}

	// 状态流转为 已开票(11, "已开票")
	public void opComplete(InvoiceApply entity, LoginSysUser loginSysUser) {
		InvoiceApplyFieldInvoiceProgressingStatusContext context = new InvoiceApplyFieldInvoiceProgressingStatusContext(entity, loginSysUser);
		context.opComplete();
	}

	/****************************************** 状态机 外部使用 类门面   end ******************************************/
}
