package cn.superdesk.uniorder.service.invoice.customer.impl;

import cn.superdesk.uniorder.common.enums.invoice.InvoiceApplyReceiveInvoiceStatusEnum;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dao.entity.invoice.customer.InvoiceApply;
import cn.superdesk.uniorder.pattern.state.invoice.customer.invoiceApply.receiveInvoiceStatus.context.InvoiceApplyFieldReceiveInvoiceStatusContext;
import cn.superdesk.uniorder.service.invoice.customer.InvoiceApplyFieldReceiveInvoiceStatusService;
import cn.superdesk.uniorder.service.invoice.customer.InvoiceApplyService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * @Description: 简易状态机 星型结构
 * @Author: linjinyu
 * @CreateDate: 10/12/20 4:28 PM
 * @UpdateUser: linjinyu
 * @UpdateDate: 10/12/20 4:28 PM
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Service
public class InvoiceApplyFieldReceiveInvoiceStatusServiceImpl implements InvoiceApplyFieldReceiveInvoiceStatusService {

	@Autowired
	private InvoiceApplyService invoiceApplyService;

	/****************************************** 状态机 公共方法 机内使用 实际流转 重用 start ******************************************/

	// 状态流转为 未领票(0, "未领票")
	public void opInitByStateMachine(InvoiceApply entity, LoginSysUser loginSysUser) {

		InvoiceApply forUpdate = new InvoiceApply();
		forUpdate.setId(entity.getId());

		// TODO 更新状态逻辑...

		forUpdate.setReceiveInvoiceStatus(InvoiceApplyReceiveInvoiceStatusEnum.未领票.getKey());

		forUpdate.setReceiveInvoiceUserId(loginSysUser.getSysUserId());
		forUpdate.setReceiveInvoiceTime(new Date());

		invoiceApplyService.updateInvoiceApplyByPrimaryKeySelective(forUpdate);
	}

	// 状态流转为 已领票(1, "已领票")
	public void opReceiveByStateMachine(InvoiceApply entity, LoginSysUser loginSysUser) {

		InvoiceApply forUpdate = new InvoiceApply();
		forUpdate.setId(entity.getId());

		// TODO 更新状态逻辑...

		forUpdate.setReceiveInvoiceStatus(InvoiceApplyReceiveInvoiceStatusEnum.已领票.getKey());

		forUpdate.setReceiveInvoiceUserId(loginSysUser.getSysUserId());
		forUpdate.setReceiveInvoiceTime(new Date());

		invoiceApplyService.updateInvoiceApplyByPrimaryKeySelective(forUpdate);
	}

	/****************************************** 状态机 公共方法 机内使用 实际流转 重用   end ******************************************/


	/****************************************** 状态机 外部使用 类门面 start ******************************************/

	// 状态流转为 未领票(0, "未领票")
	public void opInit(InvoiceApply entity, LoginSysUser loginSysUser) {
		InvoiceApplyFieldReceiveInvoiceStatusContext context = new InvoiceApplyFieldReceiveInvoiceStatusContext(entity, loginSysUser);
		context.opInit();
	}

	// 状态流转为 已领票(1, "已领票")
	public void opReceive(InvoiceApply entity, LoginSysUser loginSysUser) {
		InvoiceApplyFieldReceiveInvoiceStatusContext context = new InvoiceApplyFieldReceiveInvoiceStatusContext(entity, loginSysUser);
		context.opReceive();
	}

	/****************************************** 状态机 外部使用 类门面   end ******************************************/
}
