package cn.superdesk.uniorder.service.invoice.impl;

import cn.hutool.core.util.StrUtil;
import cn.superdesk.uniorder.common.mq.redis.service.RedisService;
import cn.superdesk.uniorder.common.mq.redis.constants.RedisKeyPrefix;
import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.common.numerical.AnBeanUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.dao.entity.invoice.InvoiceType;
import cn.superdesk.uniorder.dao.entity.invoice.InvoiceTypeExample;
import cn.superdesk.uniorder.dao.mapper.invoice.InvoiceTypeMapper;
import cn.superdesk.uniorder.dao.mapper.invoice.InvoiceTypeMapperEx;
import cn.superdesk.uniorder.dto.param.invoice.InvoiceTypeCacheCriteria;
import cn.superdesk.uniorder.service.invoice.InvoiceTypeService;
import cn.superdesk.uniorder.common.numerical.AnStringUtils;
import cn.superdesk.uniorder.common.numerical.AnCollectionUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * @Description: 重构
 * @Author: linjinyu
 * @CreateDate: 2021/7/23 下午4:03
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021/7/23 下午4:03
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Service
public class InvoiceTypeServiceImpl implements InvoiceTypeService {

	@Autowired
	private InvoiceTypeMapper   invoiceTypeMapper;
	@Autowired
	private InvoiceTypeMapperEx invoiceTypeMapperEx;

	@Autowired
	private RedisService redisService;


	/****************************************** extends BaseService 部分 start ******************************************/
	@Override
	public void addObj(InvoiceType entity) {
		invoiceTypeMapper.insert(entity);
	}

	@Override
	public void updateObj(InvoiceType entity) {
		invoiceTypeMapper.updateByPrimaryKeySelective(entity);
	}

	@Override
	public void deleteObj(Long id) {
		invoiceTypeMapper.deleteByPrimaryKey(id);
	}

	@Override
	public InvoiceType findById(Long id) {
		return invoiceTypeMapper.selectByPrimaryKey(id);
	}

	@Override
	public PageInfo<InvoiceType> findByPaging(Map<String, Object> conditions, Integer pageNun, Integer pageSize) {
		return null;
	}

	@Override
	public List<InvoiceType> findAll(Map<String, Object> queryMap) {
		return invoiceTypeMapperEx.findAll(queryMap);
	}

	/****************************************** extends BaseService 部分   end ******************************************/


	public void addInvoiceType(InvoiceType entity) {
		invoiceTypeMapper.insertSelective(entity);
	}

	public void updateInvoiceType(InvoiceType entity) {
		InvoiceType originEntity = invoiceTypeMapper.selectByPrimaryKey(entity.getId());
//		AssertUtil.notNull(originEntity);
		AnBeanUtils.copy(entity, originEntity);
		invoiceTypeMapper.updateByPrimaryKey(originEntity);
	}

	public void deleteInvoiceType(Long id) {
		invoiceTypeMapper.deleteByPrimaryKey(id);
	}

	public InvoiceType findInvoiceTypeById(Long id) {
		InvoiceType entity = invoiceTypeMapper.selectByPrimaryKey(id);
//		AssertUtil.notNull(entity);
		return entity;
	}

	public R<List<InvoiceType>> pageQuery(Map<String, Object> conditions, int pageNun, int pageSize) {

		// 样例 保留
//		InvoiceType invoiceTypeConditions = BeanUtils.mapToBean(conditions,InvoiceType.class);

		InvoiceTypeExample          example  = new InvoiceTypeExample();
		InvoiceTypeExample.Criteria criteria = example.createCriteria();

		PageHelper.startPage(pageNun, pageSize);
		List<InvoiceType>     resultList = invoiceTypeMapper.selectByExample(example);
		PageInfo<InvoiceType> page       = new PageInfo<InvoiceType>(resultList);
		PageHelper.clearPage();

		return new R<List<InvoiceType>>(page.getPageNum(), page.getPageSize(), (int) page.getTotal(), resultList);

	}


	/****************************************** 工具方法 start ******************************************/

	public Map<String, InvoiceType> convertToMapFieldTypeName2Entity() {

		Map<String, InvoiceType> result = new HashMap<>();

		InvoiceTypeExample          example  = new InvoiceTypeExample();
		InvoiceTypeExample.Criteria criteria = example.createCriteria();

		List<InvoiceType> resultList = invoiceTypeMapper.selectByExample(example);

		if (AnCollectionUtils.isNotEmpty(resultList)) {
			result = resultList.stream().collect(Collectors.toMap(InvoiceType::getTypeName, v -> v, (k, v) -> v));
		}

		return result;

	}

	/**
	 * InvoiceTypeCriteria conditions
	 *
	 * @return
	 */
	public Map<String, String> convertToMapFieldTypeName2String() {

		Map<String, String> result = new HashMap<>();

		Map<String, InvoiceType> tmp = convertToMapFieldTypeName2Entity();

		tmp.forEach((companyName, entity) -> {
			result.put(companyName, JsonUtils.toJson(entity));
//			result.put(URLUtil.encode((companyName)), JsonUtils.toJson(entity));
		});

		return result;

	}

	public Map<Long, InvoiceType> convertToMapFieldId2Entity() {

		Map<Long, InvoiceType> result = new HashMap<>();

		InvoiceTypeExample          example  = new InvoiceTypeExample();
		InvoiceTypeExample.Criteria criteria = example.createCriteria();

		List<InvoiceType> resultList = invoiceTypeMapper.selectByExample(example);

		if (AnCollectionUtils.isNotEmpty(resultList)) {
			result = resultList.stream().collect(Collectors.toMap(InvoiceType::getId, v -> v, (k, v) -> v));
		}

		return result;

	}

	/**
	 * InvoiceTypeCriteria conditions
	 *
	 * @return
	 */
	public Map<String, String> convertToMapFieldId2String() {

		Map<String, String> result = new HashMap<>();

		Map<Long, InvoiceType> tmp = convertToMapFieldId2Entity();

		tmp.forEach((id, entity) -> {
			result.put(AnStringUtils.getString(id), JsonUtils.toJson(entity));
		});

		return result;

	}


	/****************************************** 工具方法   end ******************************************/

	/****************************************** 缓存 start ******************************************/

	public void cacheInit(InvoiceTypeCacheCriteria criteria) {

//		InvoiceTypeCriteria conditions = new InvoiceTypeCriteria();

		List<String> conditions = Arrays.asList("id", "type_name");

		if (StrUtil.isNotBlank(criteria.getHashTableKeyField()) && conditions.contains(criteria.getHashTableKeyField())) {

			switch (RedisKeyPrefix.get(RedisKeyPrefix.关联性校验_发票类型.getKey() + ":" + criteria.getHashTableKeyField())) {
				case 关联性校验_发票类型_种类ID:

					Map<String, String> mapValueFieldId2String = convertToMapFieldId2String();

					// 验证 SystemConsts.RedisExpireTime.REFRESH_TOKEN_EXPIRE_TIME
					redisService.hashSave(mapValueFieldId2String, RedisKeyPrefix.关联性校验_前缀, RedisKeyPrefix.关联性校验_发票类型_种类ID.getKey(), 0);

					break;
				case 关联性校验_发票类型_种类名称:

					Map<String, String> mapValueFieldTypeName2String = convertToMapFieldTypeName2String();

					// 验证 SystemConsts.RedisExpireTime.REFRESH_TOKEN_EXPIRE_TIME
					redisService.hashSave(mapValueFieldTypeName2String, RedisKeyPrefix.关联性校验_前缀, RedisKeyPrefix.关联性校验_发票类型_种类名称.getKey(), 0);

					// 测试过 结果与上边一个样
					//		mapValue.forEach((companyName, entityJsonString) -> {
					//			cacheService.hashSet(RedisKeyPrefix.关联性校验_前缀.getKey() + ":" + RedisKeyPrefix.关联性校验_发票类型.getKey(), companyName, entityJsonString);
					//		});

					break;
			}
		}


	}

	public void cacheRefresh(InvoiceTypeCacheCriteria criteria) {

		cacheRemove(criteria);
		cacheInit(criteria);
	}

	public void cacheRemove(InvoiceTypeCacheCriteria criteria) {

		List<String> conditions = Arrays.asList("id", "type_name");

		if (StrUtil.isNotBlank(criteria.getHashTableKeyField()) && conditions.contains(criteria.getHashTableKeyField())) {

			switch (RedisKeyPrefix.get(RedisKeyPrefix.关联性校验_发票类型.getKey() + ":" + criteria.getHashTableKeyField())) {
				case 关联性校验_发票类型_种类ID:
					redisService.deleteKey(RedisKeyPrefix.关联性校验_前缀, RedisKeyPrefix.关联性校验_发票类型_种类ID.getKey());
					break;
				case 关联性校验_发票类型_种类名称:
					redisService.deleteKey(RedisKeyPrefix.关联性校验_前缀, RedisKeyPrefix.关联性校验_发票类型_种类名称.getKey());
					break;
			}
		}
	}

	public String cacheGet(InvoiceTypeCacheCriteria criteria) {

		log.info("hashTable@{}@field@{}@cacheService@{}", criteria.getHashTable() + ":" + criteria.getHashTableKeyField(), criteria.getKeyField(), redisService);

		return (String) redisService.hashGetByKey(criteria.getHashTable() + ":" + criteria.getHashTableKeyField(), criteria.getKeyField());
	}

	/**
	 * 套娃呢
	 *
	 * @param criteria
	 * @return
	 */
	public InvoiceType cacheGetEntity(InvoiceTypeCacheCriteria criteria) {

		String cacheResult = cacheGet(criteria);

		if (StrUtil.isNotBlank(cacheResult)) {
			return JsonUtils.toObject(cacheResult, InvoiceType.class);
		} else {
			return null;
		}
	}

	/****************************************** 缓存   end ******************************************/


}
