package cn.superdesk.uniorder.service.invoice.supplier.impl;

import cn.superdesk.uniorder.common.numerical.AnCollectionUtils;
import cn.superdesk.uniorder.dao.entity.accounting.unified.supplier.AccountingUnifiedSupplier;
import cn.superdesk.uniorder.dao.entity.accounting.unified.supplier.AccountingUnifiedSupplierExample;
import cn.superdesk.uniorder.dao.entity.invoice.supplier.UnifiedSupplierInv;
import cn.superdesk.uniorder.dao.entity.invoice.supplier.UnifiedSupplierInvItem;
import cn.superdesk.uniorder.dao.entity.invoice.supplier.UnifiedSupplierInvItemExample;
import cn.superdesk.uniorder.dao.mapper.accounting.unified.supplier.AccountingUnifiedSupplierMapper;
import cn.superdesk.uniorder.dao.mapper.invoice.supplier.UnifiedSupplierInvItemMapper;
import cn.superdesk.uniorder.dao.mapper.invoice.supplier.UnifiedSupplierInvItemMapperEx;
import cn.superdesk.uniorder.dao.mapper.invoice.supplier.UnifiedSupplierInvMapper;
import cn.superdesk.uniorder.dao.mapper.invoice.supplier.UnifiedSupplierInvMapperEx;
import cn.superdesk.uniorder.dto.vo.invoice.supplier.InvoiceApplyXUnifiedOrderGoodsEdVo;
import cn.superdesk.uniorder.dto.vo.invoice.supplier.UnifiedSupplierInvGoodsDetail;
import cn.superdesk.uniorder.service.invoice.supplier.UnifiedSupplierInvItemService;
import cn.superdesk.uniorder.common.lamda.AnLamdaUtils;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 供应商 开票申请 X 订单 X 商品
 * @Author: linjinyu
 * @CreateDate: 2020/11/4 下午2:39
 * @UpdateUser: linjinyu
 * @UpdateDate: 2020/11/4 下午2:39
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Service
public class UnifiedSupplierInvItemServiceImpl implements UnifiedSupplierInvItemService {

    @Autowired
    private UnifiedSupplierInvMapper   unifiedSupplierInvMapper;
    @Autowired
    private UnifiedSupplierInvMapperEx unifiedSupplierInvMapperEx;

    @Autowired
    private UnifiedSupplierInvItemMapper   unifiedSupplierInvItemMapper;
    @Autowired
    private UnifiedSupplierInvItemMapperEx unifiedSupplierInvItemMapperEx;

    @Autowired
    private AccountingUnifiedSupplierMapper accountingUnifiedSupplierMapper;

    @Override
    public PageInfo<UnifiedSupplierInvItem> findByPaging(Map<String, Object> params, Integer pageNo, Integer pageSize) {

        return null;
    }

    @Override
    public void deleteObj(Long id) {

        unifiedSupplierInvItemMapper.deleteByPrimaryKey(id);
    }

    @Override
    public void updateObj(UnifiedSupplierInvItem t) {

        unifiedSupplierInvItemMapper.updateByPrimaryKeySelective(t);
    }

    @Override
    public UnifiedSupplierInvItem findById(Long id) {

        return unifiedSupplierInvItemMapper.selectByPrimaryKey(id);
    }

    @Override
    public void addObj(UnifiedSupplierInvItem t) {

        unifiedSupplierInvItemMapper.insert(t);
    }

    @Override
    public List<UnifiedSupplierInvItem> findAll(Map<String, Object> queryMap) {

        return null;
    }

    @Override
    public List<UnifiedSupplierInvGoodsDetail> getDetail(Map<String, Object> params) {

        return unifiedSupplierInvItemMapperEx.getUnifiedSupplierInvoice2GoodsDetail(params);
    }

    /****************************************** 工具方法 start ******************************************/

    /**
     * 用于 开票申请 排除已开票的商品
     * * 装的是 基于 某对账单Sn 查出这个对账单 商品 已申请开票
     *
     * @param accountingOrderSn
     * @return
     */
    @Override
    public List<InvoiceApplyXUnifiedOrderGoodsEdVo> getInvoiceApplyXUnifiedOrderGoodsEd(String accountingOrderSn) {
        return unifiedSupplierInvItemMapperEx.getInvoiceApplyXUnifiedOrderGoodsEd(accountingOrderSn);
    }

    /**
     * 根据 开票申请Id 获取 关联 积分流水 ids 集合
     *
     * @param invoiceApplyId
     * @return
     */
    @Override
    public List<Long> findGoodsSetByInvoicApplyId(Long invoiceApplyId) {

        List<Long> result = new ArrayList<>();

        UnifiedSupplierInv invoiceApply = unifiedSupplierInvMapper.selectByPrimaryKey(invoiceApplyId);

        if (null != invoiceApply) {
            result = findGoodsSetByInvoicApplySn(invoiceApply.getInvoiceSn());
        }

        return result;

    }

    @Override
    public List<Long> findGoodsSetByInvoicApplySn(String invoiceApplySn) {

        List<Long> result = new ArrayList<>();

        UnifiedSupplierInvItemExample example = new UnifiedSupplierInvItemExample();
        example.createCriteria()
                .andInvoiceSnEqualTo(invoiceApplySn);

        List<UnifiedSupplierInvItem> invoiceApplyXGoodsList = unifiedSupplierInvItemMapper.selectByExample(example);

        if (AnCollectionUtils.isNotEmpty(invoiceApplyXGoodsList)) {
            result = invoiceApplyXGoodsList.stream().filter(AnLamdaUtils.distinctByKey(o -> o.getGoodsId())).map(o -> o.getGoodsId()).collect(Collectors.toList());
        }

        return result;
    }

    /**
     * 根据开票申请Id 获取 关联 对账单 ids 集合
     * <p>
     * 由于 这个 开票申请 X 对账单 X 订单 X 商品 表是没有记录 invoiceApplyId and accountingOrderid
     * <p>
     * 所以不实际
     *
     * @param invoiceApplyId
     * @return
     */
    @Override
    public List<Long> getAffectedAccountingOrderIdsByInvoiceApplyId(Long invoiceApplyId) {

        List<Long> result = new ArrayList<>();
        return result;

    }

    @Override
    public List<String> getAffectedAccountingOrderSnsByInvoiceApplySn(String invoiceApplySn) {

        List<String> result = new ArrayList<>();

        UnifiedSupplierInvItemExample example = new UnifiedSupplierInvItemExample();
        example.createCriteria()
                .andInvoiceSnEqualTo(invoiceApplySn);

        List<UnifiedSupplierInvItem> invoiceApplyXGoodsList = unifiedSupplierInvItemMapper.selectByExample(example);

        if (AnCollectionUtils.isNotEmpty(invoiceApplyXGoodsList)) {
            result = invoiceApplyXGoodsList.stream().filter(AnLamdaUtils.distinctByKey(o -> o.getAccountingOrderSn())).map(o -> o.getAccountingOrderSn()).collect(Collectors.toList());
        }

        return result;
    }


    /****************************************** 工具方法   end ******************************************/

    /****************************************** 基于对账单Id 检测 开票状态 start ******************************************/
    /**
     * 是否全部开票的问题 0 未开票 1 全部开票 2 部分开票
     *
     * @param accountingOrderId
     * @return
     */
    @Override
    public Integer checkInvoiceStatusByAccountingOrderIdFocusSupplier(Long accountingOrderId) {

        // 对账单表   基于对账单Id 来查出 关联的 商品
        // 开票申请表  基于对账单Id 来查出 关联的 商品
        return 0;
    }

    /**
     * 是否全部开票的问题 0 未开票 1 全部开票 2 部分开票
     *
     * @param accountingOrderSn
     * @return
     */
    @Override
    public Integer checkInvoiceStatusByAccountingOrderSnFocusSupplier(String accountingOrderSn) {

        Integer invoiceStatus = unifiedSupplierInvItemMapperEx.checkInvoiceStatusByAccountingOrderSn(accountingOrderSn);

        return invoiceStatus;
    }

    /****************************************** 基于对账单Id 检测 开票状态   end ******************************************/

    /****************************************** 基于对账单Sn检测开票状态 并 更新 对账单 开票状态 start ******************************************/
    @Override
    public void checkInvoiceStatusByAccountingOrderIdAndUpdateAccountingEntityFocusSupplier(Long accountingOrderId) {

    }

    @Override
    public void checkInvoiceStatusByAccountingOrderSnAndUpdateAccountingEntityFocusSupplier(String accountingOrderSn) {

        // 用于查找是 全部开票 还是 部分开票的问题
        Integer invoiceStatus = checkInvoiceStatusByAccountingOrderSnFocusSupplier(accountingOrderSn);

        // 对账单 结算单
        AccountingUnifiedSupplierExample example4Update = new AccountingUnifiedSupplierExample();
        example4Update.createCriteria()
                .andAccountingOrderSnEqualTo(accountingOrderSn);

        AccountingUnifiedSupplier accountingUnifiedSupplier = new AccountingUnifiedSupplier();
        accountingUnifiedSupplier.setInvoiceStatus(invoiceStatus);

        accountingUnifiedSupplierMapper.updateByExampleSelective(accountingUnifiedSupplier, example4Update);
    }

    /****************************************** 基于对账单Sn检测开票状态 并 更新 对账单 开票状态   end ******************************************/

}
