package cn.superdesk.uniorder.service.organisation.impl;

import cn.superdesk.uniorder.common.enums.common.CommonEnabledEnum;
import cn.superdesk.uniorder.common.enums.common.CommonSoftDeleteEnum;
import cn.superdesk.uniorder.common.enums.common.CommonServiceEnum;
import cn.superdesk.uniorder.common.exception.utils.ExceptionUtils;
import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.common.numerical.AnBeanUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.dao.entity.organisation.UnifiedSupplier;
import cn.superdesk.uniorder.dao.entity.organisation.UnifiedSupplierExample;
import cn.superdesk.uniorder.dao.mapper.unified.UnifiedSupplierMapper;
import cn.superdesk.uniorder.dao.mapper.unified.UnifiedSupplierMapperEx;
import cn.superdesk.uniorder.dto.param.organisation.UnifiedSupplierCriteria;
import cn.superdesk.uniorder.dto.receive.consumer.superdeskmall.SlaveSupplierReceiveParam;
import cn.superdesk.uniorder.dto.receive.consumer.superdeskmall.SuperMallSupplierReceiveParam;
import cn.superdesk.uniorder.dto.vo.organisation.UnifiedSupplierVo;
import cn.superdesk.uniorder.service.organisation.UnifiedSupplierService;
import cn.superdesk.uniorder.common.numerical.AnCollectionUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;


/**
 * @Description: 商户/供应商 对账主体
 * @Author: linjinyu
 * @CreateDate: 8/5/20 5:06 PM
 * @UpdateUser: linjinyu
 * @UpdateDate: 8/5/20 5:06 PM
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Service
public class UnifiedSupplierServiceImpl implements UnifiedSupplierService {


	@Autowired
	private UnifiedSupplierMapper   unifiedSupplierMapper;
	@Autowired
	private UnifiedSupplierMapperEx unifiedSupplierMapperEx;


	@Override
	public PageInfo<UnifiedSupplier> findByPaging(Map<String, Object> params, Integer pageNo, Integer pageSize) {
		return null;
	}

	@Override
	public void deleteObj(Long id) {

	}

	@Override
	public void updateObj(UnifiedSupplier unifiedSupplier) {
		unifiedSupplierMapper.updateByPrimaryKeySelective(unifiedSupplier);
	}

	@Override
	public UnifiedSupplier findById(Long id) {
		return unifiedSupplierMapper.selectByPrimaryKey(id);
	}

	@Override
	public void addObj(UnifiedSupplier unifiedSupplier) {
		unifiedSupplierMapper.insert(unifiedSupplier);
	}

	@Override
	public List<UnifiedSupplier> findAll(Map<String, Object> queryMap) {

		log.info("{}", JsonUtils.toJson(queryMap));

		UnifiedSupplierCriteria unifiedSupplierCriteria = AnBeanUtils.mapToBean(queryMap, UnifiedSupplierCriteria.class);

		log.info("入参: {}\n转换: {}", JsonUtils.toJson(queryMap), JsonUtils.toJson(unifiedSupplierCriteria));

		return unifiedSupplierMapperEx.findAll(unifiedSupplierCriteria);
	}

	public List<UnifiedSupplier> findByCriteria(UnifiedSupplierCriteria unifiedSupplierCriteria) {

		log.info("{}", JsonUtils.toJson(unifiedSupplierCriteria));

		log.info("入参: {}", JsonUtils.toJson(unifiedSupplierCriteria));

		return unifiedSupplierMapperEx.findAll(unifiedSupplierCriteria);
	}


	/****************************************** extends BaseService 部分   end ******************************************/


	public void addUnifiedSupplier(UnifiedSupplier entity) {
		unifiedSupplierMapper.insertSelective(entity);
	}

	public void updateUnifiedSupplier(UnifiedSupplier entity) {
		UnifiedSupplier originEntity = unifiedSupplierMapper.selectByPrimaryKey(entity.getId());
//		AssertUtil.notNull(originEntity);
		AnBeanUtils.copy(entity, originEntity);
		unifiedSupplierMapper.updateByPrimaryKey(originEntity);
	}

	public void deleteUnifiedSupplier(Long id) {
		unifiedSupplierMapper.deleteByPrimaryKey(id);
	}

	public UnifiedSupplier findUnifiedSupplierById(Long id) {
		UnifiedSupplier entity = unifiedSupplierMapper.selectByPrimaryKey(id);
//		AssertUtil.notNull(entity);
		return entity;
	}

	public R<List<UnifiedSupplier>> pageQuery(Map<String, Object> conditions, int pageNun, int pageSize) {

		// 样例 保留
//		UnifiedSupplier unifiedSupplierConditions = BeanUtils.mapToBean(conditions,UnifiedSupplier.class);


		PageHelper.startPage(pageNun, pageSize);
		UnifiedSupplierExample          example    = new UnifiedSupplierExample();
		UnifiedSupplierExample.Criteria criteria   = example.createCriteria();
		List<UnifiedSupplier>           resultList = unifiedSupplierMapper.selectByExample(example);
		PageInfo<UnifiedSupplier>       page       = new PageInfo<UnifiedSupplier>(resultList);
		PageHelper.clearPage();

		return new R<List<UnifiedSupplier>>(page.getPageNum(), page.getPageSize(), (int) page.getTotal(), resultList);

	}

	@Override
	public R<List<UnifiedSupplierVo>> getDataByPage(UnifiedSupplierCriteria unifiedSupplierCriteria, int pageNo, int pageSize) {

		PageHelper.startPage(pageNo, pageSize);
		List<UnifiedSupplier>     result = unifiedSupplierMapperEx.findAll(unifiedSupplierCriteria);
		PageInfo<UnifiedSupplier> page   = new PageInfo<>(result);
		PageHelper.clearPage();

		// 补充信息
		List<UnifiedSupplierVo> resultAdditionalInfo = new ArrayList<>();
		result.forEach(item -> {
			UnifiedSupplierVo itemVo = AnBeanUtils.copy(item, UnifiedSupplierVo.class);
			itemVo.setServiceName(CommonServiceEnum.getByServiceNo(itemVo.getServiceNo()).getServiceName());
			resultAdditionalInfo.add(itemVo);

		});

		return new R<List<UnifiedSupplierVo>>(page.getPageNum(), page.getPageSize(), (int) page.getTotal(), resultAdditionalInfo);
	}

	private void convertSlaveSupplierReceiveParam2Entity4SuperdeskMall(SuperMallSupplierReceiveParam superMallSupplierReceiveParam, UnifiedSupplier unifiedSupplier) {

		unifiedSupplier.setSaleId(superMallSupplierReceiveParam.getSaleId());
		unifiedSupplier.setSaleName(superMallSupplierReceiveParam.getSaleName());

		unifiedSupplier.setServiceNo(superMallSupplierReceiveParam.getServiceNo());

		unifiedSupplier.setContactsName(superMallSupplierReceiveParam.getContactsName());
		unifiedSupplier.setContactsPhone(superMallSupplierReceiveParam.getContactsPhone());
		unifiedSupplier.setContactsAdd(superMallSupplierReceiveParam.getContactsAdd());


		if (null == unifiedSupplier.getId()) {
			unifiedSupplier.setCreateTime(new Date());
			unifiedSupplier.setCreateUserId(1L);
		} else {
			unifiedSupplier.setUpdateTime(new Date());
			unifiedSupplier.setUpdateUserId(1L);
		}

		if (null == superMallSupplierReceiveParam.getEnabled()) {
			unifiedSupplier.setEnabled(CommonEnabledEnum.启用.getKey());
		} else {
			unifiedSupplier.setEnabled(CommonEnabledEnum.get(superMallSupplierReceiveParam.getEnabled()).getKey());
		}

		if (null == superMallSupplierReceiveParam.getDeleted()) {
			unifiedSupplier.setDeleted(CommonSoftDeleteEnum.正常.getKey());
		} else {
			unifiedSupplier.setDeleted(CommonSoftDeleteEnum.get(superMallSupplierReceiveParam.getDeleted()).getKey());
		}

		log.info("\n转换??:\n{}\n{}", JsonUtils.toJson(superMallSupplierReceiveParam), JsonUtils.toJson(unifiedSupplier));

	}

	private void convertSlaveSupplierReceiveParam2Entity4SuperdeskMallFLSC_QCH(SlaveSupplierReceiveParam slaveSupplierReceiveParam, UnifiedSupplier unifiedSupplier) {

		unifiedSupplier.setSaleId(slaveSupplierReceiveParam.getId());
		unifiedSupplier.setSaleName(slaveSupplierReceiveParam.getMerchName());

		// 侨城汇 福利商城
		unifiedSupplier.setServiceNo(CommonServiceEnum.FLSC_QCH.getServiceNo());

		unifiedSupplier.setContactsName(slaveSupplierReceiveParam.getContactsName());
		unifiedSupplier.setContactsPhone(slaveSupplierReceiveParam.getContactsPhone());
		unifiedSupplier.setContactsAdd(slaveSupplierReceiveParam.getContactsAdd());

		if (null == unifiedSupplier.getId()) {
			unifiedSupplier.setCreateTime(new Date());
			unifiedSupplier.setCreateUserId(1L);
		} else {
			unifiedSupplier.setUpdateTime(new Date());
			unifiedSupplier.setUpdateUserId(1L);
		}

		unifiedSupplier.setEnabled(CommonEnabledEnum.启用.getKey());

	}

	/**
	 * 代客专用 合并 供应商(商户) 信息
	 *
	 * @param saleId
	 * @param saleName
	 * @param serviceNo
	 */
	public void mergeUnifiedSupplier(Long saleId, String saleName, String serviceNo) {

		UnifiedSupplier processUnifiedSupplier = null;

		UnifiedSupplierExample example = new UnifiedSupplierExample();
		example.createCriteria()
				.andSaleIdEqualTo(saleId)
				// 区分serviceNo
				.andServiceNoEqualTo(serviceNo);

		List<UnifiedSupplier> unifiedSupplierList = unifiedSupplierMapper.selectByExample(example);

		if (AnCollectionUtils.isEmpty(unifiedSupplierList)) {

			processUnifiedSupplier = new UnifiedSupplier();
			processUnifiedSupplier.setSaleId(saleId);
			processUnifiedSupplier.setSaleName(saleName);
			processUnifiedSupplier.setServiceNo(serviceNo);

			processUnifiedSupplier.setCreateTime(new Date());
			processUnifiedSupplier.setCreateUserId(1L);

			unifiedSupplierMapper.insertSelective(processUnifiedSupplier);

		} else {

			processUnifiedSupplier = unifiedSupplierList.get(0);
			processUnifiedSupplier.setSaleId(saleId);
			processUnifiedSupplier.setSaleName(saleName);
			processUnifiedSupplier.setServiceNo(serviceNo);

			processUnifiedSupplier.setUpdateTime(new Date());
			processUnifiedSupplier.setUpdateUserId(1L);


			unifiedSupplierMapper.updateByPrimaryKeySelective(processUnifiedSupplier);
		}

	}

	/****************************************** kafka 超级前台 企业采购 企业福利 start ******************************************/

	@Override
	public void kafkaMergeReceiveMerchant4SuperdeskMall(SuperMallSupplierReceiveParam superMallSupplierReceiveParam) {

		try {

			UnifiedSupplier unifiedSupplier = null;

			UnifiedSupplierExample example = new UnifiedSupplierExample();
			example.createCriteria()
					.andSaleIdEqualTo(superMallSupplierReceiveParam.getSaleId())
					.andServiceNoEqualTo(superMallSupplierReceiveParam.getServiceNo());

			List<UnifiedSupplier> unifiedSupplierList = unifiedSupplierMapper.selectByExample(example);

			if (AnCollectionUtils.isEmpty(unifiedSupplierList)) {

				unifiedSupplier = new UnifiedSupplier();

				convertSlaveSupplierReceiveParam2Entity4SuperdeskMall(superMallSupplierReceiveParam, unifiedSupplier);

				unifiedSupplierMapper.insertSelective(unifiedSupplier);

			} else {

				unifiedSupplier = unifiedSupplierList.get(0);

				convertSlaveSupplierReceiveParam2Entity4SuperdeskMall(superMallSupplierReceiveParam, unifiedSupplier);

				unifiedSupplierMapper.updateByPrimaryKeySelective(unifiedSupplier);
			}

		} catch (Exception e) {

			e.printStackTrace();
			log.error("kafkaMergeReceiveMerchant4SuperdeskMall 失败:{}", ExceptionUtils.getMessage(e));

		}
	}

	@Override
	public void kafkaDeleteReceiveMerchant4SuperdeskMall(SuperMallSupplierReceiveParam superMallSupplierReceiveParam) {


		UnifiedSupplier unifiedSupplier = null;

		UnifiedSupplierExample example = new UnifiedSupplierExample();
		example.createCriteria()
				.andSaleIdEqualTo(superMallSupplierReceiveParam.getSaleId())
				.andServiceNoEqualTo(superMallSupplierReceiveParam.getServiceNo());

		List<UnifiedSupplier> unifiedSupplierList = unifiedSupplierMapper.selectByExample(example);

		if (AnCollectionUtils.isNotEmpty(unifiedSupplierList)) {

			unifiedSupplier = unifiedSupplierList.get(0);
			unifiedSupplier.setDeleted(CommonSoftDeleteEnum.删除.getKey());
			unifiedSupplierMapper.updateByPrimaryKeySelective(unifiedSupplier);
		}

	}
	/****************************************** kafka 超级前台 企业采购 企业福利   end ******************************************/

	/****************************************** kafka 侨城汇专用 start ******************************************/

	/**
	 * 侨城汇 专用 商户信息同步
	 *
	 * @param slaveSupplierReceiveParam
	 */
	@Override
	public void kafkaMergeReceiveMerchant4SuperdeskMallFLSC_QCH(SlaveSupplierReceiveParam slaveSupplierReceiveParam) {

		try {


			UnifiedSupplier unifiedSupplier = null;

			UnifiedSupplierExample example = new UnifiedSupplierExample();
			example.createCriteria()
					.andSaleIdEqualTo(slaveSupplierReceiveParam.getId());

			List<UnifiedSupplier> unifiedSupplierList = unifiedSupplierMapper.selectByExample(example);

			if (AnCollectionUtils.isEmpty(unifiedSupplierList)) {

				unifiedSupplier = new UnifiedSupplier();

				convertSlaveSupplierReceiveParam2Entity4SuperdeskMallFLSC_QCH(slaveSupplierReceiveParam, unifiedSupplier);

				unifiedSupplierMapper.insertSelective(unifiedSupplier);

			} else {

				unifiedSupplier = unifiedSupplierList.get(0);

				convertSlaveSupplierReceiveParam2Entity4SuperdeskMallFLSC_QCH(slaveSupplierReceiveParam, unifiedSupplier);

				unifiedSupplierMapper.updateByPrimaryKeySelective(unifiedSupplier);
			}

		} catch (Exception e) {

			e.printStackTrace();
			log.error("kafkaMergeReceiveMerchant4SuperdeskMallFLSC_QCH 失败:{}", ExceptionUtils.getMessage(e));

		}

	}

	/**
	 * 20200707 林志鹏说是没删除
	 *
	 * @param message
	 */
	@Override
	@Deprecated
	public void kafkaDeleteReceiveMerchant4SuperdeskMallFLSC_QCH(String message) {

	}

	/****************************************** kafka 侨城汇专用   end ******************************************/
}
