package cn.superdesk.uniorder.service.scheduler.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.superdesk.uniorder.common.enums.common.CommonAuditStatusEnum;
import cn.superdesk.uniorder.common.enums.common.CommonProcessResultEnum;
import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dao.entity.accounting.unified.supplier.AccountingUnifiedSupplier;
import cn.superdesk.uniorder.dao.entity.scheduler.SysQuartzScheduleTaskStatementOfAccountJd;
import cn.superdesk.uniorder.dto.param.accounting.jd.AccountingUnifiedSupplierOrderJdCheckUpReportTaskQueryCriteria;
import cn.superdesk.uniorder.dto.param.accounting.unified.supplier.UnifiedSupplierCreateDimGoodsParam;
import cn.superdesk.uniorder.dto.param.accounting.unified.supplier.UnifiedSupplierCreateDimGoodsParamJsonStruct;
import cn.superdesk.uniorder.pattern.state.accounting.supplier.accountingStatusJd.context.SysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusContext;
import cn.superdesk.uniorder.service.accounting.jd.AccountingUnifiedSupplierOrderJdCheckUpReportService;
import cn.superdesk.uniorder.service.accounting.unified.supplier.AccountingUnifiedSupplierServiceOpCreateDimGoods;
import cn.superdesk.uniorder.service.accounting.unified.supplier.AccountingUnifiedSupplierServiceOpCreateDimOrder;
import cn.superdesk.uniorder.service.scheduler.SysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusService;
import cn.superdesk.uniorder.service.scheduler.SysQuartzScheduleTaskStatementOfAccountJdService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description: 简易状态机 星型结构
 * @Author: linjinyu
 * @CreateDate: 10/12/20 4:28 PM
 * @UpdateUser: linjinyu
 * @UpdateDate: 10/12/20 4:28 PM
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Service
public class SysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusServiceImpl implements SysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusService {

	@Autowired
	private SysQuartzScheduleTaskStatementOfAccountJdService     sysQuartzScheduleTaskStatementOfAccountJdService;
	@Autowired
	private AccountingUnifiedSupplierOrderJdCheckUpReportService accountingUnifiedSupplierOrderJdCheckUpReportService;

	@Autowired
	private AccountingUnifiedSupplierServiceOpCreateDimOrder accountingUnifiedSupplierServiceOpCreateDimOrder;
	@Autowired
	private AccountingUnifiedSupplierServiceOpCreateDimGoods accountingUnifiedSupplierServiceOpCreateDimGoods;


	/****************************************** 状态机 公共方法 机内使用 实际流转 重用 start ******************************************/

	// 状态流转为 已取消(-1, "已取消")
	public void opCancelByStateMachine(SysQuartzScheduleTaskStatementOfAccountJd entity, LoginSysUser loginSysUser) {

		SysQuartzScheduleTaskStatementOfAccountJd forUpdate = new SysQuartzScheduleTaskStatementOfAccountJd();
		forUpdate.setId(entity.getId());

		// TODO 更新状态逻辑...

		forUpdate.setAccountingStatus(CommonAuditStatusEnum.已取消.getKey());

		sysQuartzScheduleTaskStatementOfAccountJdService.updateSysQuartzScheduleTaskStatementOfAccountJd(forUpdate);
	}

	// 状态流转为 待确认(0, "待确认")
	public void opInitOrResetByStateMachine(SysQuartzScheduleTaskStatementOfAccountJd entity, LoginSysUser loginSysUser) {

		SysQuartzScheduleTaskStatementOfAccountJd forUpdate = new SysQuartzScheduleTaskStatementOfAccountJd();
		forUpdate.setId(entity.getId());

		// TODO 更新状态逻辑...

		forUpdate.setAccountingStatus(CommonAuditStatusEnum.待确认.getKey());

		sysQuartzScheduleTaskStatementOfAccountJdService.updateSysQuartzScheduleTaskStatementOfAccountJd(forUpdate);
	}

	// 状态流转为 已确认(1, "已确认")
	public void opConfirmByStateMachine(SysQuartzScheduleTaskStatementOfAccountJd entity, LoginSysUser loginSysUser) {

		// TODO 生成统一订单的 供应商对账单

		log.info("{}", JsonUtils.toJson(entity));
		/****************************************** 接入 start ******************************************/

		UnifiedSupplierCreateDimGoodsParam unifiedSupplierCreateDimGoodsParam = new UnifiedSupplierCreateDimGoodsParam();

		// 京东的特殊处理
		unifiedSupplierCreateDimGoodsParam.setSpecialHandle(886);

		// yyyy-MM-dd HH:mm:ss
		unifiedSupplierCreateDimGoodsParam.setStartTime(DateUtil.formatDateTime(entity.getCycleBeginTime()));
		unifiedSupplierCreateDimGoodsParam.setEndTime(DateUtil.formatDateTime(entity.getCycleEndTime()));

		// 构造
		List<UnifiedSupplierCreateDimGoodsParamJsonStruct> requestParams = new ArrayList<>();

		requestParams = accountingUnifiedSupplierOrderJdCheckUpReportService.findUnifiedSupplierCreateDimGoodsParam(
				AccountingUnifiedSupplierOrderJdCheckUpReportTaskQueryCriteria
						.builder()
						.batchId(entity.getBatchId())
						.processResult(CommonProcessResultEnum.正常.getKey())
						.build()
		);

		unifiedSupplierCreateDimGoodsParam.setJsonStr(JsonUtils.toJson(requestParams));

		// TODO 有空再改任务方式 mark by linjinyu at 20211026
		R<AccountingUnifiedSupplier> response = accountingUnifiedSupplierServiceOpCreateDimGoods.opCreateAccountingDimGoodsPreProcess(unifiedSupplierCreateDimGoodsParam, loginSysUser);

		/****************************************** 接入   end ******************************************/

		SysQuartzScheduleTaskStatementOfAccountJd forUpdate = new SysQuartzScheduleTaskStatementOfAccountJd();
		forUpdate.setId(entity.getId());

		if (ObjectUtil.isNotNull(response.getData())) {

			// TODO 更新状态逻辑...
			forUpdate.setAccountingOrderSn(response.getData().getAccountingOrderSn());
			forUpdate.setAccountingStatus(CommonAuditStatusEnum.已确认.getKey());

		} else {
			// 记录 异常
			forUpdate.setProcessState(CommonProcessResultEnum.异常_数据处理问题.getKey());
			forUpdate.setAccountingOrderSn("生成对账单异常");
			forUpdate.setAccountingStatus(CommonAuditStatusEnum.已取消.getKey());
		}

		sysQuartzScheduleTaskStatementOfAccountJdService.updateSysQuartzScheduleTaskStatementOfAccountJd(forUpdate);


	}

	/****************************************** 状态机 公共方法 机内使用 实际流转 重用   end ******************************************/


	/****************************************** 状态机 外部使用 类门面 start ******************************************/

	// 状态流转为 已取消(-1, "已取消")
	public void opCancel(SysQuartzScheduleTaskStatementOfAccountJd entity, LoginSysUser loginSysUser) {
		SysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusContext context = new SysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusContext(entity, loginSysUser);
		context.opCancel();
	}

	// 状态流转为 待确认(0, "待确认")
	public void opInitOrReset(SysQuartzScheduleTaskStatementOfAccountJd entity, LoginSysUser loginSysUser) {
		SysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusContext context = new SysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusContext(entity, loginSysUser);
		context.opInitOrReset();
	}

	// 状态流转为 已确认(1, "已确认")
	public void opConfirm(SysQuartzScheduleTaskStatementOfAccountJd entity, LoginSysUser loginSysUser) {
		SysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusContext context = new SysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusContext(entity, loginSysUser);
		context.opConfirm();
	}

	/****************************************** 状态机 外部使用 类门面   end ******************************************/
}
