package cn.superdesk.uniorder.controller._thirdpart.jd;

import cn.hutool.core.lang.Snowflake;
import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.dto.param.accounting.jd.AccountingUnifiedSupplierOrderJdImportExcelRow;
import cn.superdesk.uniorder.dubbo.consumer.accounting.AccountingUnifiedSupplierOrderJdDubboConsumerService;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description: 有个很重要的点 DemoDataListener 不能被spring管理，要每次读取excel都要new,然后里面用到spring可以构造方法传进去
 * @Author: linjinyu
 * @CreateDate: 2021/3/16 下午7:06
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021/3/16 下午7:06
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
public class ThirdPartJdReconciliationScheduleExcelRowImportDataListener extends AnalysisEventListener<AccountingUnifiedSupplierOrderJdImportExcelRow> {

	/**
	 * 每隔5条存储数据库，实际使用中可以3000条，然后清理list ，方便内存回收
	 */
	private static final int BATCH_COUNT = 200;

	private List<AccountingUnifiedSupplierOrderJdImportExcelRow> accountingUnifiedSupplierOrderJdExcelRows = new ArrayList<>();

	private String token;

	private AccountingUnifiedSupplierOrderJdDubboConsumerService accountingUnifiedSupplierOrderJdDubboConsumerService;

	private Snowflake snowflake;
	private String    batchId;

	/**
	 * 如果使用了spring,请使用这个构造方法。每次创建Listener的时候需要把spring管理的类传进来
	 *
	 * @param accountingUnifiedSupplierOrderJdDubboConsumerService
	 * @param snowflake
	 * @param token
	 */
	public ThirdPartJdReconciliationScheduleExcelRowImportDataListener(
			AccountingUnifiedSupplierOrderJdDubboConsumerService accountingUnifiedSupplierOrderJdDubboConsumerService,
			Snowflake snowflake,
			String batchId,
			String token
	) {
		this.accountingUnifiedSupplierOrderJdDubboConsumerService = accountingUnifiedSupplierOrderJdDubboConsumerService;

		this.snowflake = snowflake;
		this.batchId = batchId;

		this.token = token;
	}

	/**
	 * 这个每一条数据解析都会来调用
	 *
	 * @param excelRow one row value. Is is same as {@link AnalysisContext#readRowHolder()}
	 * @param context
	 */
	@Override
	public void invoke(AccountingUnifiedSupplierOrderJdImportExcelRow excelRow, AnalysisContext context) {

		log.info("解析到一条数据: {}", JsonUtils.toJson(excelRow));

		excelRow.setBatchId(batchId);
		excelRow.setProcessResult(1);
		excelRow.setProcessReport("");

		accountingUnifiedSupplierOrderJdExcelRows.add(excelRow);

		// 达到BATCH_COUNT了，需要去存储一次数据库，防止数据几万条数据在内存，容易OOM
		if (accountingUnifiedSupplierOrderJdExcelRows.size() >= BATCH_COUNT) {

			saveOrUpdateData();
			// 存储完成清理 list
			accountingUnifiedSupplierOrderJdExcelRows.clear();
		}
	}

	/**
	 * 所有数据解析完成了 都会来调用
	 *
	 * @param context
	 */
	@Override
	public void doAfterAllAnalysed(AnalysisContext context) {

		// 这里也要保存数据，确保最后遗留的数据也存储到数据库
		saveOrUpdateData();

		log.info("所有数据解析完成！");
	}

	/**
	 * 加上存储数据库
	 */
	private void saveOrUpdateData() {

		log.info("{} 条数据，开始存储数据库！\n{}", accountingUnifiedSupplierOrderJdExcelRows.size(), JsonUtils.toJson(accountingUnifiedSupplierOrderJdExcelRows));

		accountingUnifiedSupplierOrderJdDubboConsumerService.saveOrUpdate4ThirdPartJdReconciliationScheduleExcelRowList(accountingUnifiedSupplierOrderJdExcelRows, token);

		log.info("存储数据库成功！");
	}
}