package cn.superdesk.uniorder.controller.accounting.customer;


import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import cn.superdesk.uniorder.common.enums.common.CommonAccountingLockingEnum;
import cn.superdesk.uniorder.common.enums.common.CommonAccountingStateEnum;
import cn.superdesk.uniorder.common.enums.common.CommonOrderCompletStateEnum;
import cn.superdesk.uniorder.common.enums.credit.UnifiedOrderCreditBackStatusEnum;
import cn.superdesk.uniorder.common.enums.credit.UnifiedOrderCreditInvoiceApplyStatusEnum;
import cn.superdesk.uniorder.common.enums.credit.UnifiedOrderCreditInvoiceMakeStatusEnum;
import cn.superdesk.uniorder.common.exception.utils.ExceptionUtils;
import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.common.lamda.AnLamdaUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.rest.response.ResponseResultOuterUtils;
import cn.superdesk.uniorder.common.security.crypt.SuperdeskUniorderRsaUtils;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.accounting.score.customer.AccountingScoreCustomer;
import cn.superdesk.uniorder.dao.entity.base.UnifiedOrderGoods;
import cn.superdesk.uniorder.dto.param.accounting.score.customer.AccountingScoreCustomerCreateParam;
import cn.superdesk.uniorder.dto.param.accounting.score.customer.AccountingScoreCustomerCreateSelectedCreditParam;
import cn.superdesk.uniorder.dto.vo.accounting.score.customer.CreateAccountingPreviewReport;
import cn.superdesk.uniorder.dto.vo.score.ScoreOrderReconciliationNotYetHeadVo;
import cn.superdesk.uniorder.dto.vo.score.ScoreOrderReconciliationNotYetVo;
import cn.superdesk.uniorder.dubbo.consumer.accounting.AccountingScoreCustomerDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.accounting.AccountingUnifiedCustomerDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.base.UnifiedOrderDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.invoice.customer.InvoiceApplyDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.sys.SysConfigDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.sys.SysInvDubboConsumerService;
import cn.superdesk.uniorder.interceptor.logger.annotation.ResponseExcel;
import cn.superdesk.uniorder.common.numerical.AnCollectionUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Predicate;
import java.util.stream.Collectors;

/**
 * @Description: Java类作用描述
 * @Author: linjinyu
 * @CreateDate: 6/18/20 2:32 PM
 * @UpdateUser: linjinyu
 * @UpdateDate: 6/18/20 2:32 PM
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "[对账(结算)单]-客户结算(积分)-未对账状态", tags = {"[对账(结算)单]-客户结算(积分)-未对账状态"})
@Slf4j
@RestController
@RequestMapping("/accounting/score/customer/notyet")
public class ScoreCustomerNotYetDimCreditController extends BaseController {

	@Autowired
	private UnifiedOrderDubboConsumerService unifiedOrderDubboConsumerService;
	@Autowired
	private InvoiceApplyDubboConsumerService invoiceApplyDubboConsumerService;


	@Autowired
	private AccountingUnifiedCustomerDubboConsumerService accountingUnifiedCustomerDubboConsumerService;
	@Autowired
	private AccountingScoreCustomerDubboConsumerService   accountingScoreCustomerDubboConsumerService;

	@Autowired
	private SysConfigDubboConsumerService sysConfigDubboConsumerService;
	@Autowired
	private SysInvDubboConsumerService    sysInvDubboConsumerService;
	@Autowired
	private JavaMailSender                javaMailSender;

	@ApiOperation(value = "客户结算(积分) -> 订单流水管理")
	@RequestMapping(value = "/getOrderReconciliationAll", method = RequestMethod.POST)
	@ResponseBody
	public R<List<ScoreOrderReconciliationNotYetVo>> getOrderReconciliationAll() {

		R<List<ScoreOrderReconciliationNotYetVo>> result = new R<>();

		try {

			// 筛选条件
			Map<String, Object> params = new HashMap<>();

//			xml id getScoreOrderReconciliationNotYet

//			---- 下单时间 so.create_time
			params.put("startTime", getString("startTime"));
			params.put("endTime", getString("endTime"));

//			---- 订单编号 soc.order_sn
			params.put("orderSn", getString("orderSn"));

//			---- 企业名称 so.purchase_virt_name
			params.put("purchaseVirtId", getLong("purchaseVirtId"));
			params.put("purchaseVirtName", getString("purchaseVirtName"));

			params.put("purchaseDepartmentName", getString("purchaseDepartmentName"));

//			---- 下单人姓名 so.purchase_user_name
			params.put("purchaseUserId", getLong("purchaseUserId"));
			params.put("purchaseUserName", getString("purchaseUserName"));
			params.put("purchasePhone", getString("purchasePhone"));


//			---- 积分类型 soc.credit_name
			params.put("creditName", getString("creditName"));

//			---- 积分批次 soc.batchs
			params.put("batchs", getString("batchs"));


			// 处理到这里 已经把 table list 的数据填充了
			result = unifiedOrderDubboConsumerService.getScoreOrderReconciliationNotYet(getToken(), params, getPageNum(), getPageSize());

//			for (ScoreOrderReconciliationNotYetVo vo : result.getData()) {
//				vo.setAccountingStateCustomerName(CommonAccountingStateEnum.getDesc(vo.getAccountingStateCustomer()));
//
//				vo.setInvoiceApplyStatusName(UnifiedOrderCreditInvoiceApplyStatusEnum.getDesc(vo.getInvoiceApplyStatus()));
//				vo.setInvoiceMakeStatusName(UnifiedOrderCreditInvoiceMakeStatusEnum.getDesc(vo.getInvoiceMakeStatus()));
//				vo.setBackStatusName(UnifiedOrderCreditBackStatusEnum.getDesc(vo.getBackStatus()));
//			}


//			订单数：131 订单金额：¥ 117566.6 应收金额：¥ 99055 退货金额：¥ 18511.6
			ScoreOrderReconciliationNotYetHeadVo scoreOrderReconciliationNotYetHeadSum = unifiedOrderDubboConsumerService.getScoreOrderReconciliationNotYetHeadSum(getToken(), params);
//			unifiedOrderSupHE.setRealAmount(unifiedOrderSupHE.getOrderAmount());
//			unifiedOrderSupHE.setOrderAmount(unifiedOrderSupHE.getOrderAmount().add(unifiedOrderSupHE.getReturnAmount()));
//
			Map<String, Object> headData = new HashMap<>();
			// 订单金额：¥ 22
			headData.put("totalOrderSnNum", scoreOrderReconciliationNotYetHeadSum.getTotalOrderSnNum());
			// 订单数：1
			headData.put("totalGoodsIdNum", scoreOrderReconciliationNotYetHeadSum.getTotalGoodsIdNum());
			// 退货金额：¥ 0
			headData.put("totalBalanceAmount", scoreOrderReconciliationNotYetHeadSum.getTotalBalanceAmount());


			result.setHeadData(headData);

		} catch (Exception e) {

			result.setCode(String.valueOf(e.hashCode()));
			result.setSuccess(false);
			result.setMessage(e.getMessage());

			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}
		return result;
	}


	/***
	 * 积分结算(相当于客户结算)
	 * | ----> 积分结算 -> 未结算商品
	 *
	 * 原名字为 getnotdzpage
	 *
	 * @url http://localhost:8089/web/score/getOrderReconciliationNotYet
	 * @url http://localhost:8089/web/score/accounting/customer/getOrderReconciliationNotYet
	 *
	 * @return
	 */
	@ApiOperation(value = "客户结算(积分) -> 未对账流水")
	@RequestMapping(value = "/getOrderReconciliationNotYet", method = RequestMethod.POST)
	@ResponseBody
	public R<List<ScoreOrderReconciliationNotYetVo>> getOrderReconciliationNotYet() {

		R<List<ScoreOrderReconciliationNotYetVo>> result = new R<>();

		try {

			// 筛选条件
			Map<String, Object> params = new HashMap<>();

//			xml id getScoreOrderReconciliationNotYet

//			---- 下单时间 so.create_time
			params.put("startTime", getString("startTime"));
			params.put("endTime", getString("endTime"));

//			---- 订单编号 soc.order_sn
			params.put("orderSn", getString("orderSn"));

//			---- 企业名称 so.purchase_virt_name
			params.put("purchaseVirtId", getLong("purchaseVirtId"));
			params.put("purchaseVirtName", getString("purchaseVirtName"));

			params.put("purchaseDepartmentName", getString("purchaseDepartmentName"));

//			---- 下单人姓名 so.purchase_user_name
			params.put("purchaseUserId", getLong("purchaseUserId"));
			params.put("purchaseUserName", getString("purchaseUserName"));
			params.put("purchasePhone", getString("purchasePhone"));


//			---- 积分类型 soc.credit_name
			params.put("creditName", getString("creditName"));

//			---- 积分批次 soc.batchs
			params.put("batchs", getString("batchs"));

			//			<!-- 客户对账:1-已对账;2-未对账 只作用未结算流水 -->
			params.put("accountingStateCustomer", 2);


			// 处理到这里 已经把 table list 的数据填充了
			result = unifiedOrderDubboConsumerService.getScoreOrderReconciliationNotYet(getToken(), params, getPageNum(), getPageSize());

			for (ScoreOrderReconciliationNotYetVo vo : result.getData()) {
				vo.setAccountingStateCustomerName(CommonAccountingStateEnum.getDesc(vo.getAccountingStateCustomer()));

				vo.setInvoiceApplyStatusName(UnifiedOrderCreditInvoiceApplyStatusEnum.getDesc(vo.getInvoiceApplyStatus()));
				vo.setInvoiceMakeStatusName(UnifiedOrderCreditInvoiceMakeStatusEnum.getDesc(vo.getInvoiceMakeStatus()));
				vo.setBackStatusName(UnifiedOrderCreditBackStatusEnum.getDesc(vo.getBackStatus()));
			}


//			订单数：131 订单金额：¥ 117566.6 应收金额：¥ 99055 退货金额：¥ 18511.6
			ScoreOrderReconciliationNotYetHeadVo scoreOrderReconciliationNotYetHeadSum = unifiedOrderDubboConsumerService.getScoreOrderReconciliationNotYetHeadSum(getToken(), params);
//			unifiedOrderSupHE.setRealAmount(unifiedOrderSupHE.getOrderAmount());
//			unifiedOrderSupHE.setOrderAmount(unifiedOrderSupHE.getOrderAmount().add(unifiedOrderSupHE.getReturnAmount()));
//
			Map<String, Object> headData = new HashMap<>();
			// 订单金额：¥ 22
			headData.put("totalOrderSnNum", scoreOrderReconciliationNotYetHeadSum.getTotalOrderSnNum());
			// 订单数：1
			headData.put("totalGoodsIdNum", scoreOrderReconciliationNotYetHeadSum.getTotalGoodsIdNum());
			// 退货金额：¥ 0
			headData.put("totalBalanceAmount", scoreOrderReconciliationNotYetHeadSum.getTotalBalanceAmount());


			result.setHeadData(headData);

		} catch (Exception e) {

			result.setCode(String.valueOf(e.hashCode()));
			result.setSuccess(false);
			result.setMessage(e.getMessage());

			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}
		return result;
	}

	/**
	 * 积分结算(相当于客户结算)
	 * | ----> 积分结算 -> 未结算流水 | 此页面是以商品流水的维度去展示的 opt 搜索 下单时间 只能以订单维度 最终
	 * | ----> 积分结算 -> 订单流水管理
	 * getexcel
	 *
	 * @throws IOException
	 */
	@ResponseExcel
	@ApiOperation(value = "客户结算(积分) -> 导出报表 ")
	@RequestMapping(value = "/getOrderReconciliationNotYet2Excel", method = RequestMethod.POST)
	public R<String> exportExcelOrderReconciliationNotYet2Excel() throws IOException {


		// 筛选条件
		Map<String, Object> params = new HashMap<>();

//			xml id getScoreOrderReconciliationNotYet

//			---- 下单时间 so.create_time
		params.put("startTime", getString("startTime"));
		params.put("endTime", getString("endTime"));

//			---- 订单编号 soc.order_sn
		params.put("orderSn", getString("orderSn"));

//			---- 企业名称 so.purchase_virt_nameexportgoods
		params.put("purchaseVirtId", getLong("purchaseVirtId"));
		params.put("purchaseVirtName", getString("purchaseVirtName"));

		params.put("purchaseDepartmentName", getString("purchaseDepartmentName"));

//			---- 下单人姓名 so.purchase_user_name
		params.put("purchaseUserId", getLong("purchaseUserId"));
		params.put("purchaseUserName", getString("purchaseUserName"));
		params.put("purchasePhone", getString("purchasePhone"));


//			---- 积分类型 soc.credit_name
		params.put("creditName", getString("creditName"));

//			---- 积分批次 soc.batchs
		params.put("batchs", getString("batchs"));

		log.info("\n积分结算(相当于客户结算)\n" +
				"\n| ----> 积分结算 -> 未结算流水 | 此页面是以商品流水的维度去展示的 opt 搜索 下单时间 只能以订单维度 最终\n" +
				"\n| ----> 积分结算 -> 订单流水管理" +
				"\n{}", JsonUtils.toJson(params));


//			List<ScoreOrderReconciliationNotYetVo> result = unifiedOrderDubboConsumerService.getScoreOrderReconciliationNotYet(getToken(), params);
//
//
//			/****************************************** 待重构标志 start ******************************************/
//
//			// 20200723 附件1：对账单导出格式与字段.xls
//			String[] field = {"unifiedOrderCreditId", "outSnapshotOrderCreditId", "batchs", "orderSn", "purchaseVirtName", "purchaseUserName", "purchasePhone", "goodsName", "cateNameLv3", "price", "quantity", "totalPrice", "creditName", "balance", "balance",
//					"fiscalCode", "taxCode", "taxRate",
//					"createTime",
//					"orderStateName", "saleName", "expressCompany", "expressSn", "updateTime",
//					"accountingStateCustomerName", "invoiceApplyStatusName", "invoiceMakeStatusName", "backStatusName", "remarks"
//			};
//
//			String[] fieldZhCn = {"结算中心流水ID", "快照积分流水ID", "积分批次", "订单编号", "企业名称", "用户姓名", "手机号", "商品名称", "三级分类", "销售单价", "数量", "销售总价", "积分类型", "积分支付", "结算金额",
//					"财务编码", "税务编码", "税率%",
//					"下单时间",
//					"订单状态", "供货方", "快递公司", "快递单号", "更新时间",
//					"对账状态", "发票申请", "开票状态", "回款状态", "备注"
//			};
//
//
//			ExcelResponse.out(result, fieldZhCn, field, "未对账积分流水", "未对账积分流水", response);
//
//			/****************************************** 待重构标志   end ******************************************/

//		getScoreOrderReconciliationNotYet
		return new R<String>(unifiedOrderDubboConsumerService.exportExcelScoreOrderReconciliationNotYet(getToken(), params));


	}

	/**
	 * 积分结算(相当于客户结算)
	 * | ----> 积分结算 -> 未结算商品 | 此页面是以商品流水的维度去展示的 opt 搜索 下单时间 只能以订单维度 最终
	 * | ----> | ----> 创建结算单 预览
	 * <p>
	 * goodsdetail
	 *
	 * @return
	 */
	@ApiOperation(value = "客户结算(积分) -> 创建结算单 预览")
	@RequestMapping(value = "/createAccountingPreview", method = RequestMethod.POST)
	@ResponseBody
	public R<List<ScoreOrderReconciliationNotYetVo>> createAccountingPreview(

//			String ids
//			@ModelAttribute List<AccountingScoreCustomerCreateSelectedCreditParam> selectCredits
			String selectCredits
	) {

		log.info("createAccountingPreview params {}", selectCredits);

		R<List<ScoreOrderReconciliationNotYetVo>> response = new R<>();
		CreateAccountingPreviewReport             report   = new CreateAccountingPreviewReport();

		try {

//			String[] scoreOrderCreditIds = ids.split(",");
//			if (scoreOrderCreditIds.length == 0) {
//				result.setSuccess(false);
//				result.setMessage("请选择至少 1 条流水！");
//				return result;
//			}

			List<AccountingScoreCustomerCreateSelectedCreditParam> credits = JsonUtils.toList(selectCredits, AccountingScoreCustomerCreateSelectedCreditParam.class);

			if (AnCollectionUtils.isEmpty(credits)) {
				return ResponseResultOuterUtils.createErrorResult("500", "请选择至少 1 条流水！");
			}

			List<String> scoreOrderCreditIds = credits.stream()
					.map(s -> s.getUnifiedOrderCreditId().toString())
					.collect(Collectors.toList());

			Map<String, Object> params = new HashMap<>();
			params.put("ids", scoreOrderCreditIds.toArray());
//			params.put("accountState", Byte.valueOf("2"));

//			result = unifiedOrderDubboConsumerService.getScoreOrderReconciliationNotYet(getToken(), params, 0, 99999);
//			createAccountingCheck(result);
			List<ScoreOrderReconciliationNotYetVo> resultListCredit = unifiedOrderDubboConsumerService.getScoreOrderReconciliationNotYet(getToken(), params);

			response = createAccountingCheck(resultListCredit);

			if (!response.isSuccess()) {
				return response;
			}

			if (CollUtil.isEmpty(resultListCredit)) {
				report.setDataChangeReport(StrUtil.format("数据变动 input {} output {};", credits.size(), resultListCredit.size()));
			} else if (credits.size() != resultListCredit.size()) {
				report.setDataChangeReport(StrUtil.format("数据变动 input {} output {};", credits.size(), resultListCredit.size()));
			}

			List<String> orderSnsCompletStateEq2         = createAccountingCheckByCompletStateEq2(resultListCredit);
			List<String> orderSnsOrderLockingSupplierEq1 = createAccountingCheckByOrderLockingSupplierEq1(resultListCredit);

			report.setOrderSnsCompletStateEq2(orderSnsCompletStateEq2);
			report.setOrderSnsOrderLockingSupplierEq1(orderSnsOrderLockingSupplierEq1);

			for (ScoreOrderReconciliationNotYetVo vo : resultListCredit) {
				vo.setAccountingStateCustomerName(CommonAccountingStateEnum.getDesc(vo.getAccountingStateCustomer()));

				vo.setInvoiceApplyStatusName(UnifiedOrderCreditInvoiceApplyStatusEnum.getDesc(vo.getInvoiceApplyStatus()));
				vo.setInvoiceMakeStatusName(UnifiedOrderCreditInvoiceMakeStatusEnum.getDesc(vo.getInvoiceMakeStatus()));
				vo.setBackStatusName(UnifiedOrderCreditBackStatusEnum.getDesc(vo.getBackStatus()));
			}

			response.setData(resultListCredit);
			response.setMessage(JSONUtil.toJsonStr(report));


//			TODO 处理退货 start
//			List<UnifiedOrderDetail> returnLs = new ArrayList<>();
//			if (returnLs.size() > 0) {
//				for (UnifiedOrderDetail unifiedOrderDetail : returnLs) {
//					ls.add(unifiedOrderDetail);
//				}
//			}
//			TODO 处理退货  end


		} catch (Exception e) {

			response.setCode(String.valueOf(e.hashCode()));
			response.setSuccess(false);
			response.setMessage("系统查找失败！");

			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();

		}

		return response;
	}

	// 统一订单的 只分三种 1-已完成 ; 2-未完成 ; 3-已取消
	private List<String> createAccountingCheckByCompletStateEq2(List<ScoreOrderReconciliationNotYetVo> resultListCredit) {

		List<String> orderSns = new ArrayList<>();

		List<Predicate<ScoreOrderReconciliationNotYetVo>> predicatesScoreOrderReconciliationNotYetVoByCompletStateEq2 = new ArrayList<>();
		predicatesScoreOrderReconciliationNotYetVoByCompletStateEq2.add(r -> r.getCompletState() != null && r.getCompletState().equals(CommonOrderCompletStateEnum.未完成.getKey()));

		List<ScoreOrderReconciliationNotYetVo> resultListCreditByCompletStateEq2 = AnLamdaUtils.filters(resultListCredit, predicatesScoreOrderReconciliationNotYetVoByCompletStateEq2);

		if (CollUtil.isNotEmpty(resultListCreditByCompletStateEq2)) {
			orderSns = resultListCreditByCompletStateEq2.stream().filter(AnLamdaUtils.distinctByKey(o -> o.getOrderSn())).map(o -> o.getOrderSn()).collect(Collectors.toList());
		}

		return orderSns;
	}

	// 供应商结算-订单锁定:0-正常;1-销定;
	private List<String> createAccountingCheckByOrderLockingSupplierEq1(List<ScoreOrderReconciliationNotYetVo> resultListCredit) {

		List<String> orderSns = new ArrayList<>();

		List<Predicate<ScoreOrderReconciliationNotYetVo>> predicatesScoreOrderReconciliationNotYetVoByOrderLockingSupplierEq1 = new ArrayList<>();
		predicatesScoreOrderReconciliationNotYetVoByOrderLockingSupplierEq1.add(r -> r.getOrderLockingSupplier() != null && r.getOrderLockingSupplier().equals(CommonAccountingLockingEnum.锁定.getKey()));

		List<ScoreOrderReconciliationNotYetVo> resultListCreditByOrderLockingSupplierEq1 = AnLamdaUtils.filters(resultListCredit, predicatesScoreOrderReconciliationNotYetVoByOrderLockingSupplierEq1);

		if (CollUtil.isNotEmpty(resultListCreditByOrderLockingSupplierEq1)) {
			orderSns = resultListCreditByOrderLockingSupplierEq1.stream().filter(AnLamdaUtils.distinctByKey(o -> o.getOrderSn())).map(o -> o.getOrderSn()).collect(Collectors.toList());
		}

		return orderSns;
	}

	private R createAccountingCheck(List<ScoreOrderReconciliationNotYetVo> resultListCredit) {

		log.info("createAccountingCheck \n{}", JsonUtils.toJson(resultListCredit));

		// TODO 检查这些流水是不是同一个
		// 积分类型
		// 积分批次
		// 企业

		// 基准
		ScoreOrderReconciliationNotYetVo benchmark = null;

		StringBuffer hasRefundOrder = new StringBuffer();

		for (ScoreOrderReconciliationNotYetVo iterator : resultListCredit) {

			if ("wechat".equals(iterator.getBatchs())) {
				continue;
			}

			log.info(JsonUtils.toJson(iterator));

			// 排除 wechat ，当不是 wechat 时 设定基准
			if (null == benchmark) {
				benchmark = iterator;
			} else {

				if (!iterator.getCreditName().equals(benchmark.getCreditName())) {
					return ResponseResultOuterUtils.createErrorResult("500", "积分类型 不唯一，请重新生成！");
				}
//				if (!iterator.getBatchs().equals(benchmark.getBatchs())) {
//					return ResponseResultOuterUtils.createErrorResult("500", "积分批次 不唯一，请重新生成！");
//				}
				if (iterator.getPurchaseVirtId() == null) {
					return ResponseResultOuterUtils.createErrorResult("500", "企业信息 缺失，请重新生成！");
				}
				if (!iterator.getPurchaseVirtId().equals(benchmark.getPurchaseVirtId())) {
					return ResponseResultOuterUtils.createErrorResult("500", "企业信息 不唯一，请重新生成！");
				}
			}
		}

		return ResponseResultOuterUtils.createSuccesResult();
	}


	/**
	 * 客户结算(积分) -> 未对账流水 -> opt创建对账单
	 *
	 * @param accountingScoreCustomerCreateParam
	 * @return
	 * @RequestBody
	 * @ModelAttribute
	 * @Valid
	 */
	@ApiOperation(value = "客户结算(积分) -> 未对账流水 -> opt创建对账单")
	@RequestMapping(value = "/createAccounting", method = RequestMethod.POST)
	@ResponseBody
	public R<AccountingScoreCustomer> createAccounting(AccountingScoreCustomerCreateParam accountingScoreCustomerCreateParam) {

		R<AccountingScoreCustomer> response = new R<>();

		try {

			// 订单的 ids
//			String[] ids = accountingScoreCustomerCreateParam.getIds().split(",");
//			if (ids.length == 0) {
//				result.setSuccess(false);
//				result.setMessage("您没有选择要结算的积分流水！");
//				return result;
//			}

//			List<AccountingScoreCustomerCreateSelectedCreditParam> selectCredits = accountingScoreCustomerCreateParam.getSelectCredits();

			List<AccountingScoreCustomerCreateSelectedCreditParam> selectCredits = JsonUtils.toList(accountingScoreCustomerCreateParam.getSelectCredits(), AccountingScoreCustomerCreateSelectedCreditParam.class);

			if (AnCollectionUtils.isEmpty(selectCredits)) {
				response.setSuccess(false);
				response.setMessage("您没有选择要结算的积分流水！");
				return response;
			}

			List<String> scoreOrderCreditIds = selectCredits.stream()
					.map(s -> s.getUnifiedOrderCreditId().toString())
					.collect(Collectors.toList());

			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

			Date startDate = sdf.parse(accountingScoreCustomerCreateParam.getStartTime());
			Date endDate   = sdf.parse(accountingScoreCustomerCreateParam.getEndTime());


			// 取出选中的 积分流水
			Map<String, Object> queryMap = new HashMap<>();
			queryMap.put("ids", scoreOrderCreditIds.toArray());
			List<ScoreOrderReconciliationNotYetVo> resultListCredit = unifiedOrderDubboConsumerService.getScoreOrderReconciliationNotYet(getToken(), queryMap);

			// 检查
			response = createAccountingCheck(resultListCredit);

			if (!response.isSuccess()) {
				return response;
			}

			// 统计结算单 积分总额
			BigDecimal accountingAmount = resultListCredit.stream().map(a -> a.getBalance()).reduce(BigDecimal.ZERO, BigDecimal::add);
			accountingScoreCustomerCreateParam.setAccountingAmount(accountingAmount);

			// 创建
			AccountingScoreCustomer dzMain = accountingScoreCustomerDubboConsumerService.createScoreAccountingCustomer(getToken(), resultListCredit, accountingScoreCustomerCreateParam);

			if (dzMain.getId() > 0) {

				response.setSuccess(true);
				response.setData(dzMain);

				Map<String, Object> headMap = new HashMap<>();

				headMap.put("encrypt_id", SuperdeskUniorderRsaUtils.encode(dzMain.getId().toString()));

				response.setHeadData(headMap);

			} else {

				response.setSuccess(false);
				response.setMessage("创建 结算单 失败！");

			}

		} catch (Exception e) {

			response.setSuccess(false);
			response.setMessage("系统错误，请联系管理员！");

			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}

		return response;
	}


}
