package cn.superdesk.uniorder.controller.accounting.customer;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.extra.mail.MailUtil;
import cn.superdesk.uniorder.common.excel.ExcelResponse;
import cn.superdesk.uniorder.common.exception.utils.ExceptionUtils;
import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.config.mail.MailAccountConfigFactory;
import cn.superdesk.uniorder.controller.accounting.utils.ExcelExportAccountingOrderDetailUtils;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.accounting.score.customer.AccountingScoreCustomer;
import cn.superdesk.uniorder.dto.param.accounting.common.StatementOfAccountUpdateFieldRamarkForm;
import cn.superdesk.uniorder.dto.param.accounting.score.customer.AccountingScoreCustomerSearchParam;
import cn.superdesk.uniorder.dto.vo.accounting.score.customer.AccountingScoreCustomerDetailVo;
import cn.superdesk.uniorder.dto.vo.accounting.score.customer.AccountingScoreCustomerHeadVo;
import cn.superdesk.uniorder.dto.vo.accounting.score.customer.AccountingScoreCustomerVo;
import cn.superdesk.uniorder.dubbo.consumer.accounting.AccountingScoreCustomerDubboConsumerService;
import cn.superdesk.uniorder.interceptor.logger.annotation.ResponseExcel;
import cn.superdesk.uniorder.common.numerical.AnStringUtils;
import cn.superdesk.uniorder.common.numerical.AnBeanUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: Java类作用描述
 * @Author: linjinyu
 * @CreateDate: 2020/11/19 下午3:55
 * @UpdateUser: linjinyu
 * @UpdateDate: 2020/11/19 下午3:55
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "[对账(结算)单]-客户结算(积分)-对账单", tags = {"[对账(结算)单]-客户结算(积分)-对账单"})
@Slf4j
@RestController
@RequestMapping("/accounting/score/customer/statement")
public class ScoreCustomerStatementController extends BaseController {

	@Autowired
	private AccountingScoreCustomerDubboConsumerService accountingScoreCustomerDubboConsumerService;


//	@Autowired
//	JavaMailSender javaMailSender;
//	@Autowired
//	private SysConfigDubboConsumerService sysConfigDubboConsumerService;


	@Autowired
	private MailAccountConfigFactory mailAccountConfigFactory;


	@Value("${me.email.dzpath}")
	private String accountingCreateFileTempPath;


	/**
	 * TAPD_ID_1001175_进行中_积分流水_对账单 添加根据批次查询 batchs 已完成
	 *
	 * @param scoreAccountingCustomerSearchParam
	 * @return
	 */
	@ApiOperation(value = "客户结算(积分) -> 对账单列表")
	@RequestMapping(value = "/getAccountingList", method = RequestMethod.POST)
	@ResponseBody
	public R<List<AccountingScoreCustomerVo>> getAccountingList(AccountingScoreCustomerSearchParam scoreAccountingCustomerSearchParam) {

		R<List<AccountingScoreCustomerVo>> result = new R<List<AccountingScoreCustomerVo>>();

		try {

			// 筛选条件
			Map<String, Object> params = new HashMap<>();

			params = AnBeanUtils.beanToMap(scoreAccountingCustomerSearchParam);

			log.info(JsonUtils.toPrettyJson(scoreAccountingCustomerSearchParam));
			log.info(JsonUtils.toPrettyJson(params));


			// 处理到这里 已经把 table list 的数据填充了
			result = accountingScoreCustomerDubboConsumerService.getAccountingList(getToken(), params, getPageNum(), getPageSize());

//			结算单数：131 订单数：1 积分总数：¥ 99055
			AccountingScoreCustomerHeadVo accountingListHeadSum = accountingScoreCustomerDubboConsumerService.getAccountingListHeadSum(getToken(), params);
//
			Map<String, Object> headData = new HashMap<>();
			// 结算单数：¥ 22
			headData.put("totalAccountingOrder", accountingListHeadSum.getTotalAccountingOrder());
			// 订单数：1
			headData.put("totalQuantityOrder", accountingListHeadSum.getTotalQuantityOrder());
			// 积分总数：¥ 0
			headData.put("totalBalanceAmount", accountingListHeadSum.getTotalBalanceAmount());


			result.setHeadData(headData);

		} catch (Exception e) {

			result.setCode(String.valueOf(e.hashCode()));
			result.setSuccess(false);
			result.setMessage(e.getMessage());

			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}
		return result;

	}

	/**
	 * /accounting/score/customer/statement/getAccountingList2Excel
	 *
	 * @param scoreAccountingCustomerSearchParam
	 * @param response
	 */
	@ApiOperation(value = "客户结算(积分)-对账单-opt 导出报表(列表结果集) | opt勾选导出报表")
	@RequestMapping(value = "/getAccountingList2Excel", method = RequestMethod.GET)
	@ResponseBody
	@ResponseExcel
	public void exportExcelAccountingList(AccountingScoreCustomerSearchParam scoreAccountingCustomerSearchParam, HttpServletResponse response) {

		try {

			// 筛选条件
			Map<String, Object> params = new HashMap<>();

			if (StringUtils.isNotBlank(scoreAccountingCustomerSearchParam.getAccountingOrderIds())) {

				Object[] accountingOrderIds = AnStringUtils.stringArrayConvertToLongList(scoreAccountingCustomerSearchParam.getAccountingOrderIds(), ",").toArray();

				if (accountingOrderIds.length > 0) {
					// 对账单 Ids
					params.put("ids", accountingOrderIds);
				} else {
					return;
				}
			} else {
				scoreAccountingCustomerSearchParam.setAccountingOrderIds(null);
				params = AnBeanUtils.beanToMap(scoreAccountingCustomerSearchParam);
			}

			log.info(JsonUtils.toPrettyJson(scoreAccountingCustomerSearchParam));
			log.info(JsonUtils.toPrettyJson(params));

			// 处理到这里 已经把 table list 的数据填充了
			List<AccountingScoreCustomerVo> result = new ArrayList<>();
			result = accountingScoreCustomerDubboConsumerService.getAccountingList(getToken(), params);

			String[] field = {"batchs", "creditName",
					"accountingOrderSn", "enterpriseName", "purchaseUserName", "quantityOrder",
					"accountingAmount",
					"accountingStatusName", "sendEmailStatusName", "invoiceStatusName", "createTime"
			};

			String[] fieldZhCn = {"积分批次", "积分类型",
					"结算单编号", "企业名称", "账单联系人", "订单数量",
					"积分总数",
					"结算单状态", "邮件状态", "发票状态", "对账时间"};

			ExcelResponse.out(result, fieldZhCn, field, "对账单-概要", "客户结算_对账(结算)单", response);

		} catch (Exception e) {

			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();

		}

	}


	/**
	 * 查询详情
	 * <p>
	 * http://192.168.1.223:8089/web/dzmain/getDzMainDetail
	 */
	@ApiOperation(value = "客户结算(积分) -> 对账单 查询详情")
	@RequestMapping(value = "/getAccountingDetail", method = RequestMethod.POST)
	@ResponseBody
	public R<AccountingScoreCustomerDetailVo> getStatementOfAccountDetailDimCredit() {

		R<AccountingScoreCustomerDetailVo> result = new R<>();

		Long accountingOrderId = getLong("id");

		if (accountingOrderId == null) {
			result.setSuccess(false);
			result.setMessage("请选择查看记录。");
			return result;
		}

		AccountingScoreCustomerDetailVo accountingScoreCustomerDetailVo = accountingScoreCustomerDubboConsumerService.commonStatementOfAccountDetailDimCredit(getToken(), accountingOrderId);

		result.setSuccess(true);
		result.setData(accountingScoreCustomerDetailVo);


		return result;
	}

	// TODO  积分结算-结算单-详情 导出报表

	/**
	 * 客户结算-对账单-账单详情-电子对账单-opt 导出
	 * <p>
	 * /accounting/score/customer/statement/exportExcelDetail
	 *
	 * @throws IOException
	 */
	@ApiOperation(value = "客户结算(积分)-对账单-账单详情-opt详情 导出报表")
	@RequestMapping(value = "/exportExcelDetail", method = RequestMethod.POST)
	public R<String> exportExcelStatementOfAccountDetailDimCredit() throws IOException {


		Long accountingOrderId = getLong("accountingOrderId");

//		// TODO 添加 入参较验
//
//		AccountingScoreCustomerDetailVo accountingScoreCustomerDetailVo = accountingScoreCustomerDubboConsumerService.commonStatementOfAccountDetailDimCredit(getToken(), accountingOrderId);
//
//		// TODO 20200709 清理过
//		HSSFWorkbook workbook = ExcelExportAccountingOrderDetailUtils.score(accountingScoreCustomerDetailVo);
//
//		ExcelResponse.out(workbook, "客户结算(对账)" + "_" + "电子账单" + "_" + accountingScoreCustomerDetailVo.getAccounting().getAccountingOrderSn(), response);

		return new R<String>(accountingScoreCustomerDubboConsumerService.exportExcelStatementOfAccountDetailDimCredit(getToken(), accountingOrderId));


	}


	// TODO  积分结算-结算单-详情 导出报表-发邮件


	/**
	 * 客户结算-对账单-对账单详情 opt 发送邮件
	 * /accounting/score/customer/statement/sendEmailExcelDetail
	 * <p>
	 * 发送对账单邮件-更新二维码地址
	 * <p>
	 * TODO 正解?20200630 彩娇说目前是用这个
	 *
	 * @param id
	 * @param toEmailAddress
	 * @param WxQRCodeUrl
	 * @param htmlContent
	 * @return
	 */
	@ApiOperation(value = "客户结算(积分) -> 对账单 详情 导出报表-发邮件")
	@SuppressWarnings("restriction")
	@RequestMapping(value = "/sendEmailExcelDetail", method = RequestMethod.POST)
	@ResponseBody
	public R<AccountingScoreCustomerDetailVo> sendEmailExcelAccountingDetail(Long accountingOrderId,
	                                                                         String toEmailAddress,
	                                                                         String WxQRCodeUrl,
	                                                                         int sendWxQRCode,
	                                                                         String htmlContent
	) {


		log.info("客户结算-对账单-对账单详情 opt 发送邮件 params-id            : {}", accountingOrderId);
		log.info("客户结算-对账单-对账单详情 opt 发送邮件 params-toEmailAddress: {}", toEmailAddress);
		log.info("客户结算-对账单-对账单详情 opt 发送邮件 params-WxQRCodeUrl   : {}", WxQRCodeUrl);
		log.info("客户结算-对账单-对账单详情 opt 发送邮件 params-sendWxQRCode  : {}", sendWxQRCode);
		log.info("客户结算-对账单-对账单详情 opt 发送邮件 params-htmlContent   : {}", htmlContent);


		R<AccountingScoreCustomerDetailVo> result = new R<>();

		try {

//			if (StringUtils.isNotEmpty(WxQRCodeUrl)) {
//				AccountingScoreCustomer statementOfAccount4Update = new AccountingScoreCustomer();
//				statementOfAccount4Update.setId(accountingOrderId);
//				statementOfAccount4Update.setWxQrcodeImgurl(WxQRCodeUrl);
//				accountingScoreCustomerDubboConsumerService.updateObj(statementOfAccount4Update);
//			}

//			ViewDzMain dzMainDetail = accountingScoreCustomerDubboConsumerService.getDzMainDetail(accountingOrderId);
//
//			if (dzMainDetail == null) {
//				result.setSuccess(false);
//				result.setMessage("对账单信息为空，请先选择对账单！");
//				return result;
//			}

			// 对账单详情 复合 主体
			AccountingScoreCustomerDetailVo accountingScoreCustomerDetailVo = accountingScoreCustomerDubboConsumerService.commonStatementOfAccountDetailDimCredit(getToken(), accountingOrderId);

			// 对账单 View Object
			AccountingScoreCustomerVo accountingScoreCustomerVo = accountingScoreCustomerDetailVo.getAccounting();

			// TODO 20200709 清理过
			HSSFWorkbook workbook = ExcelExportAccountingOrderDetailUtils.score(accountingScoreCustomerDetailVo);


//			SysConfig emailConfig = sysConfigDubboConsumerService.findById("EMAIL");

			String fullpath = accountingCreateFileTempPath + accountingScoreCustomerVo.getAccountingOrderSn() + ".xls";

			FileOutputStream fos = new FileOutputStream(new File(fullpath));
			workbook.write(fos);
			workbook.close();
			fos.close();

			SimpleDateFormat sdfm = new SimpleDateFormat("yyyy年MM月");
			SimpleDateFormat sdfd = new SimpleDateFormat("yyyy年MM月dd日");

			String subject = "侨城汇-" + accountingScoreCustomerVo.getAccountingOrderSn() + "-" + (accountingScoreCustomerVo.getEnterpriseName()) + "." + sdfd.format(accountingScoreCustomerVo.getStartTime()) + "-" + sdfd.format(accountingScoreCustomerVo.getEndTime()) + "-对账单";


			MailUtil.send(
					mailAccountConfigFactory.newInstance(),
					CollUtil.newArrayList(toEmailAddress.split(",")[0]),
//					CollUtil.newArrayList("linjinyu@superdesk.cn"),
					subject,
					htmlContent, true,
					new File(fullpath)
			);


		} catch (cn.hutool.extra.mail.MailException e) {

			result.setSuccess(false);
			result.setMessage("发送邮件失败, 请检查邮箱地址: " + e.getMessage());

		} catch (Exception e) {

			result.setSuccess(false);
			result.setMessage("发送邮件失败，请联系管理员！");

			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}
		return result;
	}

	@ApiOperation(value = "设置备注-对账单-积分结算-侨城汇")
	@RequestMapping(value = "/setRemark", method = RequestMethod.POST)
	@ResponseBody
	public R setRemark(@RequestBody StatementOfAccountUpdateFieldRamarkForm form) {

		AccountingScoreCustomer statementOfAccount4Update = new AccountingScoreCustomer();
		statementOfAccount4Update.setId(form.getId());
		statementOfAccount4Update.setRemark(form.getRemark());

		accountingScoreCustomerDubboConsumerService.updateFieldRemark(getToken(), statementOfAccount4Update);

		return new R();
	}

}


