package cn.superdesk.uniorder.controller.accounting.customer;

import cn.superdesk.uniorder.common.exception.utils.ExceptionUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.accounting.score.customer.AccountingScoreCustomer;
import cn.superdesk.uniorder.dubbo.consumer.accounting.AccountingScoreCustomerDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

@Api(value = "[对账(结算)单]-客户结算(积分)-对账单-状态流转", tags = {"[对账(结算)单]-客户结算(积分)-对账单-状态流转"})
@Slf4j
@RestController
@RequestMapping("/accounting/score/customer/statement/state")
public class ScoreCustomerStatementStateFieldAccountingStatusController extends BaseController {

	@Autowired
	private AccountingScoreCustomerDubboConsumerService accountingScoreCustomerDubboConsumerService;

	/****************************************** 状态流转类 start ******************************************/

	/**
	 * 确认结算
	 * <p>
	 * http://192.168.1.223:8089/web/dzmain/confirmDz
	 */
	@ApiOperation(value = "客户结算(积分) -> 对账单 op确认")
	@RequestMapping(value = "/confirmAccounting", method = RequestMethod.POST)
	@ResponseBody
	public R<?> opConfirmAccounting(Long id) {

		R<?> result = new R<>();

		try {
			AccountingScoreCustomer main = accountingScoreCustomerDubboConsumerService.findById(id);

			if (main.getAccountingStatus() != 0) {
				result.setSuccess(false);
				result.setMessage("确认失败，该对账单已操作！");
				return result;
			}

			accountingScoreCustomerDubboConsumerService.opConfirmAccounting(getToken(), id);

			result.setSuccess(true);

		} catch (Exception e) {

			result.setSuccess(false);
			result.setMessage("系统错误，请联系管理员！");
			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}
		return result;
	}

	/**
	 * 拒绝 结算
	 */
	@ApiOperation(value = "客户结算(积分) -> 对账单 op拒绝")
	@RequestMapping(value = "/refuseAccounting", method = RequestMethod.POST)
	@ResponseBody
	public R<?> opRefuseAccounting(Long id) {

		R<?> result = new R<>();
		try {
			AccountingScoreCustomer main = accountingScoreCustomerDubboConsumerService.findById(id);

			if (main.getAccountingStatus() != 0) {
				result.setSuccess(false);
				result.setMessage("拒绝失败，该对账单已操作！");
				return result;
			}

			accountingScoreCustomerDubboConsumerService.opRefuseAccounting(getToken(), id);

			result.setSuccess(true);

		} catch (Exception e) {

			result.setSuccess(false);
			result.setMessage("系统错误，请联系管理员！");
			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}
		return result;

	}


	/**
	 * 重新结算 取消结算
	 */
	@ApiOperation(value = "客户结算(积分) -> 对账单 op重新结算|op取消结算")
	@RequestMapping(value = "/cancelAccounting", method = RequestMethod.POST)
	@ResponseBody
	public R<?> opCancelAccounting(Long id) {

		R<?> result = new R<>();
		try {

			AccountingScoreCustomer main = accountingScoreCustomerDubboConsumerService.findById(id);

			if (main.getAccountingStatus() == -1) {
				result.setSuccess(false);
				result.setMessage("该对账单已取消，不可重复操作！");
				return result;
			}

			if (main.getInvoiceStatus() != 0) {
				result.setSuccess(false);
				result.setMessage("该对账单已开票，不可重新对账！");
				return result;
			}

			if (main.getBackAccountStatus() != 0) {
				result.setSuccess(false);
				result.setMessage("该对账单已回款，不可重新对账！");
				return result;
			}

			accountingScoreCustomerDubboConsumerService.opCancelAccounting(getToken(), id);

			result.setSuccess(true);

		} catch (Exception e) {

			result.setSuccess(false);
			result.setMessage("系统错误，请联系管理员！");
			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();

		}

		return result;

	}


	/**
	 * 删除结算 只能删除已取消的
	 */
	@ApiOperation(value = "客户结算(积分) -> 对账单 op删除对账单")
	@RequestMapping(value = "/deleteAccounting", method = RequestMethod.POST)
	@ResponseBody
	public R<?> opDeleteAccounting(Long id) {

		R<?> result = new R<>();

		try {

			AccountingScoreCustomer main = accountingScoreCustomerDubboConsumerService.findById(id);

			if (main.getAccountingStatus() == -1) {

				accountingScoreCustomerDubboConsumerService.opDeleteAccounting(id);

				result.setSuccess(true);

			} else {
				result.setSuccess(false);
				result.setMessage("不能删除 未取消结算单");
				return result;
			}


		} catch (Exception e) {

			result.setSuccess(false);
			result.setMessage("系统错误，请联系管理员！");

			log.error(ExceptionUtils.getMessage(e));

			e.printStackTrace();

		}

		return result;

	}

	/****************************************** 状态流转类   end ******************************************/
}
