package cn.superdesk.uniorder.controller.accounting.supplier;

import cn.superdesk.uniorder.common.enums.common.CommonAccountingStateEnum;
import cn.superdesk.uniorder.common.exception.utils.ExceptionUtils;
import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.accounting.unified.supplier.AccountingUnifiedSupplier;
import cn.superdesk.uniorder.dto.param.accounting.unified.supplier.AccountingSupplierNotYetDimGoodsCriteria;
import cn.superdesk.uniorder.dto.param.accounting.unified.supplier.UnifiedSupplierCreateDimGoodsParam;
import cn.superdesk.uniorder.dto.vo.accounting.unified.supplier.notyet.UnifiedOrderJoinGoodsOfSupplierStatementOfAccount;
import cn.superdesk.uniorder.dto.vo.accounting.unified.supplier.notyet.UnifiedOrderSupplierHeadSumVo;
import cn.superdesk.uniorder.dubbo.consumer.accounting.AccountingUnifiedSupplierDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.base.UnifiedOrderDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.organisation.UnifiedSupplierDubboConsumerService;
import cn.superdesk.uniorder.common.numerical.AnBeanUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: Java类作用描述
 * @Author: linjinyu
 * @CreateDate: 2021/7/8 下午5:00
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021/7/8 下午5:00
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "[对账(结算)单]-供应商对账(结算)-未对账商品", tags = {"[对账(结算)单]-供应商对账(结算)-未对账商品"})
@Slf4j
@RestController
@RequestMapping(value = "/accounting/unified/supplier/dim/goods/notyet")
public class UnifiedSupplierNotYetDimGoodsController extends BaseController {


	@Autowired
	private UnifiedOrderDubboConsumerService              unifiedOrderDubboConsumerService;
	@Autowired
	private UnifiedSupplierDubboConsumerService           unifiedSupplierDubboConsumerService;
	@Autowired
	private AccountingUnifiedSupplierDubboConsumerService accountingUnifiedSupplierDubboConsumerService;

	/**
	 * 供应商结算-未对账商品
	 *
	 * @return
	 * @url : http://order.avic-s.com/#/vendorUnreconOrder?id=71
	 * <p>
	 * /accounting/unified/supplier/dim/goods/notyet/getSupplierUnAccountingDimGoodsList
	 * <p>
	 * 未对账列表分页
	 * , tags = {""}
	 */
	@ApiOperation(value = "供应商对账(结算)-未对账商品-维度-商品", notes = "前端新添加页面-测试正常")
	@RequestMapping(value = "/getSupplierUnAccountingDimGoodsList", method = RequestMethod.POST)
	@ResponseBody
	public R<List<UnifiedOrderJoinGoodsOfSupplierStatementOfAccount>> getSupplierUnAccountingDimGoodsList(@RequestBody AccountingSupplierNotYetDimGoodsCriteria criteria) {

		R<List<UnifiedOrderJoinGoodsOfSupplierStatementOfAccount>> result = new R<>();

		try {

			criteria.setAccountingStateSupplier(CommonAccountingStateEnum.未对账.getKey());

			log.info("\n供应商对账(结算)-未对账商品-维度-商品-params {}\n", JsonUtils.toJson(criteria));

			result = unifiedOrderDubboConsumerService.getSupplierUnAccountingDimGoodsListByPageQuery(getToken(), criteria);

			UnifiedOrderSupplierHeadSumVo supplierUnAccountingDimGoodsHeadSum = unifiedOrderDubboConsumerService.getSupplierUnAccountingDimGoodsHeadSum(getToken(), criteria);

			result.setHeadData(AnBeanUtils.beanToMap(supplierUnAccountingDimGoodsHeadSum));

		} catch (Exception e) {

			result.setCode(String.valueOf(e.hashCode()));
			result.setSuccess(false);
			result.setMessage(e.getMessage());

			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}
		return result;
	}


	/***
	 * 供应商结算-未对账订单 opt 导出报表 (未对账列表-导出)
	 *
	 * /accounting/unified/supplier/notyet/exportuaccountdata
	 *
	 * http://192.168.1.223:8089/web/accounting/unified/supplier/notyet/exportuaccountdata?orderIds=55032,55031&token=eyJhbGciOiJIUzI1NiJ9.eyJzYWxlaWQiOm51bGwsInZpcnRJZHMiOm51bGwsIm91dExvZ2luIjoiMCIsInVzZXJOYW1lIjoid2FuZ3N5Iiwicm9sZVR5cGUiOiIxIiwib2NpZCI6bnVsbCwiZXhwIjoxNjAxMDAwNzgyLCJ1c2VySWQiOiI2MCIsImlhdCI6MTYwMDkxNDM4Mn0.2L-ZP_c2fdgREcvSVMpLa_HWjoKCY0RLGFnx2BtJoiQ
	 * @return
	 */
	@ApiOperation(value = "供应商对账(结算)-未对账商品-opt 导出报表-维度-商品", notes = "未修改")
	@RequestMapping(value = "/exportSupplierUnAccountingDimGoodsList", method = RequestMethod.POST)
	public R<String> exportExcelAccountingSupplierNotYetDimGoods(@RequestBody AccountingSupplierNotYetDimGoodsCriteria criteria) {

		criteria.setAccountingStateSupplier(CommonAccountingStateEnum.未对账.getKey());

		log.info("供应商结算-未对账订单 opt 导出报表 (未对账列表-导出) params: \n{}", JsonUtils.toJson(criteria));

		return new R<String>(unifiedOrderDubboConsumerService.exportExcelAccountingSupplierNotYetDimGoods(getToken(), criteria));
	}


	/**
	 * 供应商对账(结算)-未对账商品-创建对账单-维度-商品
	 *
	 * @param params
	 * @return
	 */
	@ApiOperation(value = "供应商对账(结算)-未对账商品-创建对账单-维度-商品", notes = "")
	@RequestMapping(value = "/createStatementOfAccount", method = RequestMethod.POST)
	@ResponseBody
	public R<AccountingUnifiedSupplier> createStatementOfAccount(UnifiedSupplierCreateDimGoodsParam params) {

		R<AccountingUnifiedSupplier> result = new R<>();

		try {

			result = accountingUnifiedSupplierDubboConsumerService.opCreateAccountingDimGoods(getToken(), params);

		} catch (Exception e) {

			result.setCode(String.valueOf(e.hashCode()));
			result.setSuccess(false);
			result.setMessage(e.getMessage());

			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}

		return result;
	}

	/***
	 * 供应商结算-对账单-opt btn 编辑对账
	 * 供应商结算-对账单-opt btn 查看详情
	 * /getgoods
	 * @return
	 */
	@ApiOperation(value = "供应商对账(结算)-对账单-opt btn 编辑对账 | opt btn 查看详情-维度-商品", notes = "未整改")
	@RequestMapping(value = "/statementOfAccountDetail", method = RequestMethod.POST)
	@ResponseBody
	public R<List<UnifiedOrderJoinGoodsOfSupplierStatementOfAccount>> statementOfAccountDetail() {

		R<List<UnifiedOrderJoinGoodsOfSupplierStatementOfAccount>> result = new R<>();

		try {
			Map<String, Object> params = new HashMap<>();

			params.put("accountingOrderSn", getString("accountingOrderSn"));

			List<UnifiedOrderJoinGoodsOfSupplierStatementOfAccount> unifiedOrderGoodsOfStatementOfAccountList = unifiedOrderDubboConsumerService.getSupplierStatementOfAccountDetailDimGoods2Goods(params);

			result.setData(unifiedOrderGoodsOfStatementOfAccountList);

		} catch (Exception e) {

			result.setCode(String.valueOf(e.hashCode()));
			result.setSuccess(false);
			result.setMessage(e.getMessage());

			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}
		return result;
	}

	/***
	 * 供应商结算-未对账订单-对账单详情-详情导出
	 * @url /accounting/unified/supplier/dim/goods/notyet/exportStatementOfAccountDetail
	 * /exportgoods
	 * @return
	 */
	@ApiOperation(value = "供应商对账(结算)-对账单-对账单详情-详情导出-维度-商品-20210413-改版", notes = "未整改")
	@RequestMapping(value = "/exportStatementOfAccountDetail", method = RequestMethod.POST)
	public R<String> exportSupplierStatementOfAccountDetail(String accountingOrderSn) throws IOException {

		Map<String, Object> params = new HashMap<>();
		params.put("accountingOrderSn", accountingOrderSn);

		log.info("供应商结算-未对账订单-对账单详情-详情导出 params: \n{}", JsonUtils.toJson(params));

		return new R<String>(unifiedOrderDubboConsumerService.exportSupplierStatementOfAccountDetailCompatible(getToken(), params));

	}
}
