package cn.superdesk.uniorder.controller.invoice;

import cn.superdesk.uniorder.common.pagination.PageQueryParam;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.invoice.InvoiceBankInfo;
import cn.superdesk.uniorder.dubbo.consumer.invoice.InvoiceBankInfoDubboConsumerService;
import cn.superdesk.uniorder.common.exception.utils.ExceptionUtils;
import cn.superdesk.uniorder.common.numerical.AnStringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Api(value = "系统设置-收款方发票信息(银行信息)", tags = {"系统设置-收款方发票信息(银行信息)"})
@Slf4j
@RestController
@RequestMapping("/setting/unifiedBankInfo")
public class InvoiceBankInfoController extends BaseController {


	@Autowired
	private InvoiceBankInfoDubboConsumerService unifiedBankInfoDubboConsumerService;

	@ApiOperation(value = "列表分页")
	@PostMapping("/page")
	@ResponseBody
	public R<List<InvoiceBankInfo>> page() {

		R<List<InvoiceBankInfo>> result = new R<>();

		try {
			Map<String, Object> params = new HashMap<>();

			params.put("companyName", getString("companyName"));
			params.put("enabled", 1);

			result = unifiedBankInfoDubboConsumerService.getDataByPage(getToken(), params, getPageNum(), getPageSize());

		} catch (Exception e) {

			result.setCode(String.valueOf(e.hashCode()));
			result.setSuccess(false);
			result.setMessage(e.getMessage());

			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}

		return result;
	}

	@ApiOperation(value = "新增收款方")
	@PostMapping("/add")
	@ResponseBody
	public R<Object> add(InvoiceBankInfo param) {

		R<Object> result = new R<>();

		try {

			if (!StringUtils.isNotEmpty(param.getCompanyName())) {

				result.setCode(400);
				result.setSuccess(false);
				result.setMessage("收款方名称不能为空！");

				return result;
			}

			Map<String, Object> resMap = unifiedBankInfoDubboConsumerService.addObj(getToken(), param);

			if (resMap.get("code").toString().equals("1")) {

				result.setSuccess(true);
				result.setData(resMap.get("data"));

			} else {
				result.setSuccess(false);
				result.setMessage(resMap.get("msg").toString());
			}

		} catch (Exception e) {

			result.setCode(String.valueOf(e.hashCode()));
			result.setSuccess(false);
			result.setMessage(e.getMessage());

			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}
		return result;
	}

	@ApiOperation(value = "编辑收款方")
	@PostMapping("/update")
	@ResponseBody
	public R<Object> update(InvoiceBankInfo param) {

		R<Object> result = new R<>();

		try {

			if (!StringUtils.isNotEmpty(param.getCompanyName())) {
				result.setCode(400);
				result.setSuccess(false);
				result.setMessage("收款方名称不能为空！");
				return result;
			}

			Map<String, Object> resMap = unifiedBankInfoDubboConsumerService.updateObj(getToken(), param);

			if (resMap.get("code").toString().equals("1")) {

				result.setSuccess(true);
				result.setData(resMap.get("data"));
			} else {
				result.setSuccess(false);
				result.setMessage(resMap.get("msg").toString());
			}

		} catch (Exception e) {

			result.setCode(String.valueOf(e.hashCode()));
			result.setSuccess(false);
			result.setMessage(e.getMessage());

			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}
		return result;
	}

	@ApiOperation(value = "默认收款方")
	@PostMapping("/updateauto")
	@ResponseBody
	public R<Object> updateAuto(Long id) {

		R<Object> result = new R<>();

		try {

			unifiedBankInfoDubboConsumerService.updateAuto(id);
			result.setSuccess(true);

		} catch (Exception e) {

			result.setCode(String.valueOf(e.hashCode()));
			result.setSuccess(false);
			result.setMessage(e.getMessage());

			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}
		return result;
	}

	@ApiOperation(value = "删除收款方")
	@PostMapping("/del")
	@ResponseBody
	public R<Object> delete(Long id) {

		R<Object> result = new R<>();

		try {

			unifiedBankInfoDubboConsumerService.delObj(id);
			result.setSuccess(true);

		} catch (Exception e) {

			result.setCode(String.valueOf(e.hashCode()));
			result.setSuccess(false);
			result.setMessage(e.getMessage());

			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}

		return result;
	}


	/****************************************** 缓存 start ******************************************/

	/**
	 * http://192.168.6.223:8089/web/setting/unifiedBankInfo/cacheInit
	 *
	 * @return
	 */
	@ApiOperation(value = "cacheInit")
	@RequestMapping(value = "cacheInit", method = RequestMethod.POST)
	@ResponseBody
	public R<String> cacheInit() {

		unifiedBankInfoDubboConsumerService.cacheInit();
		return new R<>("200", "操作成功", true);

	}

	@ApiOperation(value = "cacheRefresh")
	@RequestMapping(value = "cacheRefresh", method = RequestMethod.POST)
	@ResponseBody
	public R<String> cacheRefresh() {

		unifiedBankInfoDubboConsumerService.cacheRefresh();
		return new R<>("200", "操作成功", true);

	}

	@ApiOperation(value = "cacheRemove")
	@RequestMapping(value = "cacheRemove", method = RequestMethod.POST)
	@ResponseBody
	public R<String> cacheRemove() {

		unifiedBankInfoDubboConsumerService.cacheRemove();
		return new R<>("200", "操作成功", true);

	}


	/**
	 * hashtable relation:sys_enterprise_ext_invoice:company_name field 中航物业管理有限公司济南分公司
	 * <p>
	 * {
	 *     "conditions": {
	 *         "hashTable": "relation:sys_enterprise_ext_invoice:company_name",
	 *         "keyField": "中航物业管理有限公司长沙分公司"
	 *     }
	 * }
	 *
	 * @return
	 */
	@ApiOperation(value = "cacheGet")
	@RequestMapping(value = "cacheGet", method = RequestMethod.POST)
	@ResponseBody
	public R<String> cacheGet(@RequestBody PageQueryParam param) {

//		String hashTable = "relation:sys_enterprise_ext_invoice:company_name";
//		String keyField = "中航物业管理有限公司";

		String hashTable = AnStringUtils.getString(param.getConditions().get("hashTable"));
		String keyField  = AnStringUtils.getString(param.getConditions().get("keyField"));

		log.info("{}{}", hashTable, keyField);

		String responseData = unifiedBankInfoDubboConsumerService.cacheGet(hashTable, keyField);
		return new R<>("200", "操作成功", responseData);

	}
}
