package cn.superdesk.uniorder.controller.invoice;

import cn.superdesk.libs.common.json.JsonUtils;
import cn.superdesk.uniorder.common.pagination.PageQueryParam;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.invoice.InvoiceType;
import cn.superdesk.uniorder.dto.param.invoice.InvoiceTypeCacheCriteria;
import cn.superdesk.uniorder.dto.param.invoice.InvoiceTypeParam;
import cn.superdesk.uniorder.dubbo.consumer.invoice.InvoiceTypeDubboConsumerService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;


/**
 * @Description:
 * @Author: linjinyu
 * @Email linjinyu@avic-s.com
 * @CreateDate: 2021-07-23 15:30:45
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021-07-23 15:30:45
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@RestController
@RequestMapping("/invoiceType")
public class InvoiceTypeController extends BaseController {

	@Autowired
	private InvoiceTypeDubboConsumerService invoiceTypeDubboConsumerService;

	@ApiOperation(value = "分页查询")
	@RequestMapping(value = "list", method = RequestMethod.POST)
	@ResponseBody
	public R<List<InvoiceType>> pageQueryInvoiceTypes(@RequestBody PageQueryParam param) {

		log.info("pageQueryInvoiceType:\n{}", JsonUtils.toJson(param));

		R<List<InvoiceType>> result = new R<>();

		// TODO 参数合法性判定

		result = invoiceTypeDubboConsumerService.pageQuery(param.getConditions(), param.getPageNo(), param.getPageSize());

		return result;
	}

	@ApiOperation(value = "按id查询")
	@RequestMapping(value = "{id}", method = RequestMethod.GET)
	@ResponseBody
	public R<List<InvoiceType>> getById(@PathVariable("id") long id) {

		R<List<InvoiceType>> result = new R<>();

		// TODO 参数合法性判定

		InvoiceType entity = invoiceTypeDubboConsumerService.findInvoiceTypeById(id);

		List<InvoiceType> data = new ArrayList<>();
		data.add(entity);

		result.setData(data);

		return result;
	}

	@ApiOperation(value = "新增")
	@RequestMapping(value = "add", method = RequestMethod.POST)
	@ResponseBody
	public R<String> addInvoiceType(@RequestBody InvoiceTypeParam param) {

		log.info("addInvoiceType:\n{}", JsonUtils.toJson(param));


		// TODO 参数合法性判定

		invoiceTypeDubboConsumerService.addInvoiceType(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "更新")
	@RequestMapping(value = "update", method = RequestMethod.POST)
	@ResponseBody
	public R<String> updateInvoiceType(@RequestBody InvoiceTypeParam param) {

		log.info("updateInvoiceType:\n{}", JsonUtils.toJson(param));

		invoiceTypeDubboConsumerService.updateInvoiceType(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "删除")
	@RequestMapping(value = "delete/{id}", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteInvoiceType(@PathVariable("id") long id) {

		// TODO 参数合法性判定

		invoiceTypeDubboConsumerService.deleteInvoiceType(id);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "批量删除")
	@RequestMapping(value = "delete_ids", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteIdsInvoiceType(@RequestParam(value = "ids") long[] ids) {

		log.info(JsonUtils.toJson(ids));

		for (long id : ids) {
			invoiceTypeDubboConsumerService.deleteInvoiceType(id);
		}

		return new R<>("200", "操作成功", true);

	}

	/****************************************** 缓存 start ******************************************/

	/**
	 * http://192.168.6.223:8089/web/invoiceType/cacheInit
	 *
	 * @return
	 */
	@ApiOperation(value = "cacheInit")
	@RequestMapping(value = "cacheInit", method = RequestMethod.POST)
	@ResponseBody
	public R<String> cacheInit(@RequestBody InvoiceTypeCacheCriteria criteria) {

		invoiceTypeDubboConsumerService.cacheInit(criteria);
		return new R<>("200", "cacheInit 操作成功", true);

	}

	@ApiOperation(value = "cacheRefresh")
	@RequestMapping(value = "cacheRefresh", method = RequestMethod.POST)
	@ResponseBody
	public R<String> cacheRefresh(@RequestBody InvoiceTypeCacheCriteria criteria) {

		invoiceTypeDubboConsumerService.cacheRefresh(criteria);
		return new R<>("200", "cacheRefresh 操作成功", true);

	}

	@ApiOperation(value = "cacheRemove")
	@RequestMapping(value = "cacheRemove", method = RequestMethod.POST)
	@ResponseBody
	public R<String> cacheRemove(@RequestBody InvoiceTypeCacheCriteria criteria) {

		invoiceTypeDubboConsumerService.cacheRemove(criteria);
		return new R<>("200", "cacheRemove 操作成功", true);

	}


	/**
	 * @param criteria
	 * @return
	 */
	@ApiOperation(value = "cacheGet")
	@RequestMapping(value = "cacheGet", method = RequestMethod.POST)
	@ResponseBody
	public R<String> cacheGet(@RequestBody InvoiceTypeCacheCriteria criteria) {

//		{
//			"hashTable": "relation:invoice_type",
//				"hashTableKeyField": "type_name",
//				"keyField": "增值税专票"
//		}

		String hashTable = criteria.getHashTable();
		String keyField  = criteria.getKeyField();

		log.info("{}{}", hashTable, keyField);

		String responseData = invoiceTypeDubboConsumerService.cacheGet(criteria);
		return new R<>("200", "操作成功", responseData);

	}


}
