package cn.superdesk.uniorder.controller.invoice.customer;

import cn.superdesk.uniorder.common.enums.statement.AccountingOrderDimCreateByEnum;
import cn.superdesk.uniorder.common.enums.statement.AccountingOrderTypeDimEnum;
import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dto.param.invoice.ExportExcelInvoiceApplyQueryCriteria;
import cn.superdesk.uniorder.dto.vo.invoice.customer.InvoiceApplyPage;
import cn.superdesk.uniorder.dubbo.consumer.invoice.customer.InvoiceApplyDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.organisation.SysEnterpriseDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.organisation.SysEnterpriseExtInvoiceDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 积分订单 产生的 结算单 开票
 * @Author: linjinyu
 * @CreateDate: 7/1/20 11:37 AM
 * @UpdateUser: linjinyu
 * @UpdateDate: 7/1/20 11:37 AM
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "[开票发票]-积分结算-开票申请(发票申请管理)", tags = {"[开票发票]-积分结算-开票申请(发票申请管理)"})
@Slf4j
@RestController
@RequestMapping("/invoice/apply/customer/score")
public class InvoiceApplyCustomerScoreController extends BaseController {


	@Autowired
	private InvoiceApplyDubboConsumerService            invoiceApplyDubboConsumerService;
	@Autowired
	private SysEnterpriseDubboConsumerService           sysEnterpriseDubboConsumerService;
	@Autowired
	private SysEnterpriseExtInvoiceDubboConsumerService sysEnterpriseExtInvoiceDubboConsumerService;


	@ApiOperation(value = "积分结算-发票申请管理-List")
	@RequestMapping(value = "/findViewByPage", method = RequestMethod.POST)
	public R<List<InvoiceApplyPage>> findViewByPage(@RequestBody ExportExcelInvoiceApplyQueryCriteria criteria) {

		R<List<InvoiceApplyPage>> result = new R<>();

		log.info("\n\n\nfindViewByPage\nparams \n{}\ntoken  \n{}\n\n\n\n", JsonUtils.toJson(criteria), getToken());

		result = invoiceApplyDubboConsumerService.findViewByPageFocusScore(getToken(), criteria, criteria.getPageNumber(), criteria.getPageSize());

		return result;
	}

	@ApiOperation(value = "积分结算-发票申请管理-Head")
	@RequestMapping(value = "/findViewHeadData", method = RequestMethod.POST)
	public R findViewHeadData(@RequestBody ExportExcelInvoiceApplyQueryCriteria criteria) {

		return new R(invoiceApplyDubboConsumerService.findViewHeadDataFocusScore(getToken(), criteria));
	}


	@ApiOperation(value = "积分结算-发票申请管理|发票管理-opt 查看详情")
	@RequestMapping(value = "/getDetail", method = RequestMethod.POST)
	public R opDetail(@RequestBody ExportExcelInvoiceApplyQueryCriteria criteria) {

		log.info("积分结算-发票申请管理-opt 查看详情 params : \n{}", JsonUtils.toJson(criteria));

		return new R(invoiceApplyDubboConsumerService.getUnifiedCustomerInvoice2CreditDetail(getToken(), criteria));
	}

	/**
	 * /invoice/apply/customer/score/deleteInvoiceApply
	 *
	 * @return
	 */
	@ApiOperation(value = "积分结算-发票申请管理|发票管理-opt 删除")
	@RequestMapping(value = "/deleteInvoiceApply", method = RequestMethod.POST)
	public R opDelete() {

		Map<String, Object> params = new HashMap<>();

		// 发票申请Id
		params.put("id", getLong("id"));

		log.info("积分结算-发票申请管理-opt 删除 params : \n{}", JsonUtils.toJson(params));

		invoiceApplyDubboConsumerService.opDelete(getToken(), getLong("id"));

		return new R("200", "删除成功", true);
	}

	/**
	 * /invoice/apply/customer/score/exportInvoiceApplyExcel
	 * 侨城汇-发票申请管理 -
	 *
	 * 	// TODO 有逻辑 侨城汇
	 * 	//	|---- 1. 搜索框 条件 结算集 op 导出报表
	 * 	//	|---- 2. 搜索框 条件 结算集 后 界面上 勾选 要以 开票申请编号 分 sheet 分合并前 合并后 op 勾选导出报表
	 * 	//	|---- 3. 搜索框 条件 结算集 后 界面上 勾选 要以 开票申请编号 分 sheet 财务专用 op 导出开票数据
	 * 	private Integer mode;
	 *
	 * @param criteria
	 * @throws IOException
	 */
	@ApiOperation(value = "积分结算-发票申请管理-opt 导出报表")
	@RequestMapping(value = "/exportInvoiceApplyExcel", method = RequestMethod.POST)
	public R<String> exportExcelInvoiceApply(@RequestBody ExportExcelInvoiceApplyQueryCriteria criteria) throws IOException {

		log.info("{}", JsonUtils.toJson(criteria));

		return new R<>(invoiceApplyDubboConsumerService.exportExcelInvoiceApplyFocusScoreDimCredit(getToken(), criteria));
	}


	/**
	 * 对映一般订单 开票申请 搜索
	 *
	 * @return
	 */
	@ApiOperation(value = "积分结算-开票申请页面-opt 搜索")
	@PostMapping("/invoiceApplySelect")
	public R opApplySelect(@RequestBody ExportExcelInvoiceApplyQueryCriteria criteria) {

		if (null == criteria.getAccountingScoreCustomerId()) {
			return new R("500", "参数错误,对账单Id号不能为空", false);
		}
		if (null != criteria.getAccountingOrderSn()) {
			return new R("500", "参数错误,不支持对账单Sn号", false);
		}

		return new R(invoiceApplyDubboConsumerService.invoiceApplySelectFocusScoreDimCredit(criteria));
	}


	@ApiOperation(value = "积分结算-开票申请页面-op 申请开票-编辑页面 opt 提交")
	@PostMapping("/invoiceApply")
	public R opApply() {

		Map<String, Object> params = new HashMap<>();

//		orderGoodsObj
//		{
//			'unifiedOrderCreditId' : "对映 接口 invoiceApplySelect 返回数据 .unifiedOrderCreditId",
//			'unifiedOrderGoodsId' : "对映 接口 invoiceApplySelect 返回数据 .unifiedOrderGoodsId",
//			'unifiedOrderId' : "对映 接口 invoiceApplySelect 返回数据 .unifiedOrderId",
//			'accountingOrderId' : "对映 接口 invoiceApplySelect 返回数据 .accountingOrderId",
//		}


		/**
		 * 商品Id列表，','隔开 弃用 20200716 by linjinyu
		 */
//		params.put("orderGoodIds", getString("orderGoodIds"));


		params.put("accountingOrderTypeDim", AccountingOrderTypeDimEnum.积分订单.getKey());
		params.put("dimCreateBy", AccountingOrderDimCreateByEnum.流水.getKey());

		/**
		 * InvoiceApplyOrderGoodsParams
		 */
		params.put("orderGoodsObj", getString("orderGoodsObj"));
		/**
		 * 添加一个 企业ID 20200716 by linjinyu 如果这个开票不是以企业维度开， 这个想法就是错的了
		 */
		params.put("purchaseVirtId", getLong("purchaseVirtId"));
		/**
		 * 发票抬头
		 */
		params.put("invoiceCompanyName", getString("invoiceCompanyName"));
		/**
		 * 发票类型
		 */
		params.put("invoiceType", getString("invoiceType"));
		/**
		 * 备注
		 */
		params.put("remark", getString("remark"));
		/**
		 * 税率Id
		 */
		params.put("taxRate", getString("taxRate"));

		if (StringUtils.isEmpty(getString("freeAmount"))
				|| StringUtils.isEmpty(getString("realAmount"))) {
			return new R("400", "金额错误！", false);
		}

		params.put("freeAmount", getString("freeAmount"));
		params.put("realAmount", getString("realAmount"));

		log.info("客户结算 > 开票申请管理 > opt 申请开票 -> 编辑页面 opt 提交 params: \n{}", JsonUtils.toJson(params));

		return invoiceApplyDubboConsumerService.invoiceApplyFocusScore(getToken(), params);
	}


	/**
	 * 客户结算-开票申请管理-opt row 审核 (申请开票) 按扭 同意 拒绝
	 * /invoice/apply/customer/score/audit
	 * <p>
	 * 弃用
	 * 【开发中 202107151422 积分结算2.0优化需求（保留现有的银行流水和收款核销模块）】 https://www.tapd.cn/33003868/prong/stories/view/1133003868001002470
	 * 需求7：发票申请管理优化
	 * ② 隐藏发票申请管理列表的字段：审核状态、审核人、审核时间
	 * ③ 隐藏查询栏：审核状态下拉查询
	 * ④ 隐藏发票申请管理列表操作按钮：同意、拒绝
	 *
	 * @return
	 */
	@Deprecated
	@ApiOperation(value = "积分结算-开票申请管理-opt row 审核向按扭 同意 拒绝")
	@PostMapping("/audit")
	public R opAudit() {

		Map<String, Object> params = new HashMap<>();

		params.put("id", getString("id"));
		params.put("auditStatus", getInteger("auditStatus"));

		log.info("客户结算-开票申请管理-opt row 审核 (申请开票) 按扭 同意 拒绝 params : \n{}", JsonUtils.toJson(params));

		invoiceApplyDubboConsumerService.opAudit(getToken(), params);

		return new R();
	}


	/**
	 * 客户结算-开票申请管理-opt 开票登记
	 * TODO 标记 可以共用
	 * /invoice/apply/customer/score/invoiceRegister
	 *
	 * @return
	 */
	@ApiOperation(value = "积分结算-开票申请管理-opt 开票登记")
	@PostMapping("/invoiceRegister")
	public R opRegister() {

		Map<String, Object> params = new HashMap<>();

		/**
		 * 发票申请Id
		 */
		params.put("id", getString("id"));
		/**
		 * 开票日期 手动填写
		 */
		params.put("datestr", getString("datestr"));
		/**
		 * 发票号
		 */
		params.put("invoiceNumber", getString("invoiceNumber"));

		invoiceApplyDubboConsumerService.opRegister(getToken(), params);

		return new R();
	}
}