package cn.superdesk.uniorder.controller.invoice.customer;

import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dto.param.invoice.ExportExcelInvoiceApplyQueryCriteria;
import cn.superdesk.uniorder.dubbo.consumer.invoice.customer.InvoiceApplyDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;


/**
 * @Description: Java类作用描述
 * @Author: linjinyu
 * @CreateDate: 2021/2/23 下午4:20
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021/2/23 下午4:20
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "[开票发票]-客户结算-发票管理-Excel导出)", tags = {"[开票发票]-客户结算-发票管理-Excel导出"})
@Slf4j
@RestController
@RequestMapping("/invoice/apply/customer/export")
public class InvoiceApplyExportExcelController extends BaseController {

	@Autowired
	private InvoiceApplyDubboConsumerService invoiceApplyDubboConsumerService;

	/**
	 * 前端
	 * <p>
	 * 客户结算-申请开票-查看详情-opt 导出详情
	 * 客户结算-发票管理-查看详情-opt 导出明细
	 *
	 * @return
	 * @url /invoice/apply/customer/export/unified/UnifiedCustomerInvoice2GoodsDetail/{invoiceApplyId}
	 * 6379
	 */
	@ApiOperation(value = "客户结算-申请开票|发票管理-查看详情-opt 导出详情")
	@RequestMapping(value = "/unified/UnifiedCustomerInvoice2GoodsDetail/{invoiceApplyId}", method = RequestMethod.POST)
	public R<String> exportUnifiedCustomerInvoice2GoodsDetail(@PathVariable Long invoiceApplyId) {

		Map<String, Object> params = new HashMap<>();

		// 发票申请Id invoiceApplyId
		params.put("id", invoiceApplyId);

		log.info("客户结算-发票申请-opt 查看详情 params : \n{}", JsonUtils.toJson(params));

		return new R<String>(invoiceApplyDubboConsumerService.exportUnifiedCustomerInvoice2GoodsDetail(params, getToken()));
	}

	/**
	 * 前端
	 * <p>
	 * 积分结算-申请开票-opt 查看详情
	 * 积分结算-发票管理-opt 查看详情
	 * mark by linjinyu at 20211112 已完成
	 *
	 * @url /invoice/apply/customer/export/score/UnifiedCustomerInvoice2CreditDetail
	 * @return
	 */
	@ApiOperation(value = "积分结算-申请开票|发票管理-查看详情-opt 导出详情 (未完成 TODO)")
	@RequestMapping(value = "/score/UnifiedCustomerInvoice2CreditDetail", method = RequestMethod.POST)
	public R<String> exportUnifiedCustomerInvoice2CreditDetail(@RequestBody ExportExcelInvoiceApplyQueryCriteria criteria) {

//		Map<String, Object> params = new HashMap<>();
//
//		// 发票申请Id
//		params.put("id", getLong("id"));

		log.info("积分结算-发票申请管理-opt 查看详情 params : \n{}", JsonUtils.toJson(criteria));

		return new R<String>(invoiceApplyDubboConsumerService.exportUnifiedCustomerInvoice2CreditDetail(criteria, getToken()));
	}


}
