package cn.superdesk.uniorder.controller.invoice.supplier;

import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.common.numerical.AnBeanUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.enums.invoice.InvoiceApplyBackAccountStatusEnum;
import cn.superdesk.uniorder.common.enums.invoice.InvoiceApplyMakeInvoiceStatusEnum;
import cn.superdesk.uniorder.common.enums.pay.UnifiedOrderPayTypeEnum;
import cn.superdesk.uniorder.common.enums.statement.AccountingOrderAccountingStatusEnum;
import cn.superdesk.uniorder.common.enums.statement.AccountingOrderInvoiceStatusEnum;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.accounting.unified.supplier.AccountingUnifiedSupplier;
import cn.superdesk.uniorder.dao.entity.invoice.supplier.UnifiedSupplierInv;
import cn.superdesk.uniorder.dto.vo.invoice.supplier.UnifiedSupplierInvDetail;
import cn.superdesk.uniorder.dto.vo.invoice.supplier.UnifiedSupplierInvGoodsDetail;
import cn.superdesk.uniorder.dto.vo.invoice.supplier.UnifiedSupplierInvoiceAddParams;
import cn.superdesk.uniorder.dto.vo.unified.InvoiceApplySupplierSelectGoodsVo;
import cn.superdesk.uniorder.dto.vo.unified.UnifiedInvHE;
import cn.superdesk.uniorder.dubbo.consumer.accounting.AccountingUnifiedSupplierDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.invoice.supplier.UnifiedSupplierInvDubboConsumerService;
import cn.superdesk.uniorder.common.exception.utils.ExceptionUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping("/invoice/apply/supplier")
public class InvoiceApplySupplierController extends BaseController {

    @Autowired
    private UnifiedSupplierInvDubboConsumerService unifiedSupplierInvDubboConsumerService;

    @Autowired
    private AccountingUnifiedSupplierDubboConsumerService accountingUnifiedSupplierDubboConsumerService;

    /***
     * 供应商结算-发票登记管理
     * 供应商结算-货款管理
     *
     * 共用接口
     * @return
     */
    @PostMapping("/getdatapage")
    @ResponseBody
    public R<List<UnifiedSupplierInv>> getInvoiceApplyList() {

        R<List<UnifiedSupplierInv>> result = new R<>();

        try {

            Map<String, Object> params = new HashMap<>();

            params.put("invoiceSn", getString("invoiceSn"));

            // 开票状态 0未开票 1 已开票 2重新开票 前端写死了?
            params.put("invoiceStatus", getString("invoiceStatus"));

            // 回款状态
            params.put("backAccountStatus", getString("backAccountStatus"));

            params.put("supplierId", getLong("supplierId"));
            params.put("supplierName", getString("supplierName"));

            //申请人 姓名
            params.put("applyUserName", getString("applyUserName"));

            params.put("startTime", getString("startTime"));
            params.put("endTime", getString("endTime"));

            log.info("供应商结算-发票登记管理 params : \n{}", JsonUtils.toJson(params));

            result = unifiedSupplierInvDubboConsumerService.getInvoiceApplyList(getToken(), params, getPageNum(), getPageSize());

            UnifiedInvHE head = unifiedSupplierInvDubboConsumerService.getInvoiceApplyListHeadSum(getToken(), params);

            result.setHeadData(AnBeanUtils.beanToMap(head));

        } catch (Exception e) {

            result.setCode(String.valueOf(e.hashCode()));
            result.setSuccess(false);
            result.setMessage(e.getMessage());

            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }

        return result;
    }


    /***
     * 供应商结算-发票登记管理-详情
     * 供应商结算-货款管理-详情
     *
     * 共用接口
     * @return
     */
    @PostMapping("/getdetail")
    @ResponseBody
    public R<UnifiedSupplierInvDetail> getInvoiceDetail(Long id) {

        R<UnifiedSupplierInvDetail> result = new R<>();

        try {

            UnifiedSupplierInvDetail detail = unifiedSupplierInvDubboConsumerService.getInvoiceDetail(id);

            for (UnifiedSupplierInvGoodsDetail vo : detail.getGoods()) {
                vo.setPayTypeName(UnifiedOrderPayTypeEnum.getDesc(vo.getPayType()));
            }

            result.setData(detail);
            result.setSuccess(true);

        } catch (Exception e) {

            result.setCode(String.valueOf(e.hashCode()));
            result.setSuccess(false);
            result.setMessage(e.getMessage());

            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }

        return result;
    }

    /***
     * 供应商结算-发票申请管理-opt 申请开票(搜索)
     * @return
     */
    @PostMapping("/getdzgoods")
    @ResponseBody
    public R<List<InvoiceApplySupplierSelectGoodsVo>> getInvoiceApplySelect() {

        R<List<InvoiceApplySupplierSelectGoodsVo>> result = new R<>();

        try {

            Boolean checkParams = false;

            if (null != getString("accountingOrderSn") || null != getString("orderSn") || null != getString("supplierName")) {
                checkParams = true;
            }

            if (checkParams.equals(false)) {
                result.setCode(400);
                result.setSuccess(false);
                result.setMessage("选择择一种条件进行搜索");
                return result;
            }

            Map<String, Object> params = new HashMap<>();

            // 对账单编号
            params.put("accountingOrderSn", getString("accountingOrderSn"));
            // 订单编号
            params.put("orderSn", getString("orderSn"));
            // 供应商名称
            params.put("supplierName", getString("supplierName"));

            log.info("供应商结算-发票申请管理-opt 申请开票 params: \n{}", JsonUtils.toJson(params));

            String accountingOrderSn = getString("accountingOrderSn");

            // TODO 在此加入判定
            AccountingUnifiedSupplier statementOfAccount = accountingUnifiedSupplierDubboConsumerService.findByAccountingOrderSn(accountingOrderSn);

//<!--			已取消(-1, "已取消"),-->
//<!--			未确认(0, "未确认"),-->
//<!--			已确认(1, "已确认"),-->
//<!--			已拒绝(2, "已拒绝"),-->
//<!--			待提交(3, "待提交"),-->
            if (!statementOfAccount.getAccountingStatus().equals(AccountingOrderAccountingStatusEnum.已确认.getKey())) {
                result.setCode(500);
                result.setSuccess(false);
//                result.setMessage("对账单 对账状态 非[已确认] 不能申请开票");
                result.setMessage("只有 [已确认] 的账单可进行发票申请管理，请选进行商户确认");
                return result;
            }

//<!--			已开票(1, "已开票"),-->
//<!--			未开票(2, "未开票"),-->
//<!--			部分开票(3, "部分开票");-->
            if (statementOfAccount.getInvoiceStatus().equals(AccountingOrderInvoiceStatusEnum.全部申请.getKey())) {
                result.setCode(500);
                result.setSuccess(false);
                result.setMessage("对账单 开票状态 为[已开票] 不能重复申请开票");
                return result;
            }

            // 对账单 部分 申请开票问题未解决
            List<InvoiceApplySupplierSelectGoodsVo> goods = new ArrayList<>();
            // 没有过滤 已经 开票申请的商品
//            goods = unifiedSupplierInvDubboConsumerService.getStatementOfAccountGoodsListByAccountingOrderSn(accountingOrderSn);
            // 过滤 已经 开票申请的商品
            goods = unifiedSupplierInvDubboConsumerService.invoiceApplySelectFocusUnified(accountingOrderSn);

            result.setData(goods);
            result.setSuccess(true);

        } catch (Exception e) {

            result.setCode(String.valueOf(e.hashCode()));
            result.setSuccess(false);
            result.setMessage(e.getMessage());

            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }
        return result;
    }


    /**
     * 供应商结算-发票申请管理-申请开票-opt btn 提交
     *
     * @return
     */
    @PostMapping("/apply")
    @ResponseBody
    public R<UnifiedSupplierInv> opInvoiceApply(UnifiedSupplierInvoiceAddParams unifiedSupplierInvoiceAddParams) {

        R<UnifiedSupplierInv> result = new R<>();

        try {

            if (!StringUtils.isNotEmpty(unifiedSupplierInvoiceAddParams.getJsonStr())) {
                result.setCode(400);
                result.setSuccess(false);
                result.setMessage("请选择对账单！");
            }

            UnifiedSupplierInv inv = unifiedSupplierInvDubboConsumerService.opInvoiceApply(getToken(), unifiedSupplierInvoiceAddParams);

            result.setData(inv);
            result.setSuccess(true);

        } catch (Exception e) {

            result.setCode(String.valueOf(e.hashCode()));
            result.setSuccess(false);
            result.setMessage(e.getMessage());

            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }

        return result;
    }

    /***
     * 供应商结算-发票申请管理-opt row 开票登记
     * @return
     */
    @PostMapping("/register")
    @ResponseBody
    public R<UnifiedSupplierInv> opRegister() {

        R<UnifiedSupplierInv> result = new R<>();

        try {
            Long   id              = getLong("id");
            String invoiceNumber   = getString("invoiceNumber");
            String makeInvoiceTime = getString("makeInvoiceTime");

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

            UnifiedSupplierInv unifiedSupplierInv = new UnifiedSupplierInv();

            unifiedSupplierInv.setId(id);
            unifiedSupplierInv.setMakeInvoiceTime(sdf.parse(makeInvoiceTime));
            unifiedSupplierInv.setInvoiceNumber(invoiceNumber);
            unifiedSupplierInv.setMakeInvoiceStatus(InvoiceApplyMakeInvoiceStatusEnum.已开票.getKey());

            UnifiedSupplierInv inv = unifiedSupplierInvDubboConsumerService.updateObj(getToken(), unifiedSupplierInv);

            result.setData(inv);
            result.setSuccess(true);

        } catch (Exception e) {

            result.setCode(String.valueOf(e.hashCode()));
            result.setSuccess(false);
            result.setMessage(e.getMessage());

            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }

        return result;
    }


    /***
     * 供应商结算-货款管理-opt row 重新开票
     * @return
     */
    @PostMapping("/openagain")
    @ResponseBody
    public R<UnifiedSupplierInv> opAgain() {

        R<UnifiedSupplierInv> result = new R<>();

        try {

            Long id = getLong("id");

            unifiedSupplierInvDubboConsumerService.opAgain(id);

            result.setSuccess(true);

        } catch (Exception e) {

            result.setCode(String.valueOf(e.hashCode()));
            result.setSuccess(false);
            result.setMessage(e.getMessage());

            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }

        return result;
    }

    /***
     * 供应商结算-货款管理-opt row 确定付款
     * @return
     */
    @PostMapping("/payment")
    @ResponseBody
    public R<UnifiedSupplierInv> opConfirmPay() {

        R<UnifiedSupplierInv> result = new R<>();

        try {
            Long id = getLong("id");

            UnifiedSupplierInv unifiedSupplierInv4Update = new UnifiedSupplierInv();
            unifiedSupplierInv4Update.setId(id);
            unifiedSupplierInv4Update.setBackAccountStatus(InvoiceApplyBackAccountStatusEnum.已回款.getKey());

            UnifiedSupplierInv inv = unifiedSupplierInvDubboConsumerService.opConfirmPay(getToken(), unifiedSupplierInv4Update);

            result.setData(inv);
            result.setSuccess(true);

        } catch (Exception e) {

            result.setCode(String.valueOf(e.hashCode()));
            result.setSuccess(false);
            result.setMessage(e.getMessage());

            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }

        return result;
    }

    /**
     * 供应商结算-货款管理-opt row 删除
     * /invoice/apply/supplier/opDelete/{invoiceApplyId}
     *
     * @return
     */
    @RequestMapping(value = "/opDelete/{invoiceApplyId}", method = RequestMethod.GET)
    @ResponseBody
    public R<?> opDelete(@PathVariable("invoiceApplyId") Long invoiceApplyId) {

        R<?> result = new R<>();

        try {
            unifiedSupplierInvDubboConsumerService.opDelete(invoiceApplyId);

            result.setSuccess(true);
            result.setMessage("删除成功");

        } catch (Exception e) {

            result.setSuccess(false);
            result.setMessage("系统错误，请联系管理员！");

            log.error(ExceptionUtils.getMessage(e));

            e.printStackTrace();

        }

        return result;
    }
}