package cn.superdesk.uniorder.controller.organisation;

import cn.superdesk.libs.common.json.JsonUtils;
import cn.superdesk.uniorder.common.exception.utils.ExceptionUtils;
import cn.superdesk.uniorder.common.pagination.PageQueryGenericParam;
import cn.superdesk.uniorder.common.pagination.PageQueryParam;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.organisation.UnifiedSupplier;
import cn.superdesk.uniorder.dto.param.organisation.UnifiedSupplierCriteria;
import cn.superdesk.uniorder.dto.param.organisation.UnifiedSupplierParam;
import cn.superdesk.uniorder.dto.vo.organisation.UnifiedSupplierVo;
import cn.superdesk.uniorder.dubbo.consumer.organisation.UnifiedSupplierDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;


/**
 * @Description: 供应商-聚合数据-非统一订单产生
 * @Author: linjinyu
 * @Email linjinyu@avic-s.com
 * @CreateDate: 2021-05-14 12:51:18
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021-05-14 12:51:18
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "[组织架构]-供应商-聚合数据-非统一订单产生", tags = {"[组织架构]-供应商-聚合数据-非统一订单产生"})
@Slf4j
@RestController
@RequestMapping("/unifiedSupplier")
public class UnifiedSupplierController extends BaseController {

	@Autowired
	private UnifiedSupplierDubboConsumerService unifiedSupplierDubboConsumerService;

	/**
	 * http://192.168.6.223:8089/web/accounting/unified/supplier/notyet/page
	 * http://192.168.6.223:8089/web/unifiedSupplier/getSupplierList
	 * <p>
	 * 供应商结算-成本核算-商户列表(下拉)
	 *
	 * @return
	 */
	@ApiOperation(value = "getSupplierList")
	@RequestMapping(value = "/getSupplierList", method = RequestMethod.POST)
	@ResponseBody
	public R<List<UnifiedSupplierVo>> pageQueryUnifiedSuppliers(@RequestBody PageQueryGenericParam<UnifiedSupplierCriteria> criteria) {

		log.info("getSupplierList \n{}", JsonUtils.toJson(criteria));

		R<List<UnifiedSupplierVo>> result = new R<>();

		try {

			result = unifiedSupplierDubboConsumerService.getDataByPage(getToken(), criteria.getConditions(), criteria.getPageNo(), criteria.getPageSize());

		} catch (Exception e) {

			result.setCode(String.valueOf(e.hashCode()));
			result.setSuccess(false);
			result.setMessage(e.getMessage());

			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}

		return result;
	}

	@ApiOperation(value = "分页查询")
	@RequestMapping(value = "list", method = RequestMethod.POST)
	@ResponseBody
	public R<List<UnifiedSupplier>> pageQueryUnifiedSuppliers(@RequestBody PageQueryParam param) {

		log.info("pageQueryUnifiedSupplier:\n{}", JsonUtils.toJson(param));

		R<List<UnifiedSupplier>> result = new R<>();

		// TODO 参数合法性判定

		result = unifiedSupplierDubboConsumerService.pageQuery(param.getConditions(), param.getPageNo(), param.getPageSize());

		return result;
	}

	@ApiOperation(value = "按id查询")
	@RequestMapping(value = "{id}", method = RequestMethod.GET)
	@ResponseBody
	public R<List<UnifiedSupplier>> getById(@PathVariable("id") long id) {

		R<List<UnifiedSupplier>> result = new R<>();

		// TODO 参数合法性判定

		UnifiedSupplier entity = unifiedSupplierDubboConsumerService.findUnifiedSupplierById(id);

		List<UnifiedSupplier> data = new ArrayList<>();
		data.add(entity);

		result.setData(data);

		return result;
	}

	@ApiOperation(value = "新增")
	@RequestMapping(value = "add", method = RequestMethod.POST)
	@ResponseBody
	public R<String> addUnifiedSupplier(@RequestBody UnifiedSupplierParam param) {

		log.info("addUnifiedSupplier:\n{}", JsonUtils.toJson(param));


		// TODO 参数合法性判定

		unifiedSupplierDubboConsumerService.addUnifiedSupplier(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "更新")
	@RequestMapping(value = "update", method = RequestMethod.POST)
	@ResponseBody
	public R<String> updateUnifiedSupplier(@RequestBody UnifiedSupplierParam param) {

		log.info("updateUnifiedSupplier:\n{}", JsonUtils.toJson(param));

		unifiedSupplierDubboConsumerService.updateUnifiedSupplier(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "删除")
	@RequestMapping(value = "delete/{id}", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteUnifiedSupplier(@PathVariable("id") long id) {

		// TODO 参数合法性判定

		unifiedSupplierDubboConsumerService.deleteUnifiedSupplier(id);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "批量删除")
	@RequestMapping(value = "delete_ids", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteIdsUnifiedSupplier(@RequestParam(value = "ids") long[] ids) {

		log.info(JsonUtils.toJson(ids));

		for (long id : ids) {
			unifiedSupplierDubboConsumerService.deleteUnifiedSupplier(id);
		}

		return new R<>("200", "操作成功", true);

	}


}
