package cn.superdesk.uniorder.controller.scheduler;

import cn.hutool.core.util.ObjectUtil;
import cn.superdesk.uniorder.common.enums.common.CommonAuditStatusEnum;
import cn.superdesk.uniorder.common.exception.utils.ExceptionUtils;
import cn.superdesk.uniorder.common.json.JsonUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.rest.response.ResponseResultOuterUtils;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.scheduler.SysQuartzScheduleTaskStatementOfAccountJd;
import cn.superdesk.uniorder.dubbo.consumer.scheduler.SysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.scheduler.SysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @Description: 简易状态机 星型结构
 * @Author: linjinyu
 * @CreateDate: 10/12/20 4:28 PM
 * @UpdateUser: linjinyu
 * @UpdateDate: 10/12/20 4:28 PM
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "定时任务-时刻任务-对账单(京东对账单Excel文件上传) SysQuartzScheduleTaskStatementOfAccountJd - AccountingStatus", tags = {"定时任务-时刻任务-对账单(京东对账单Excel文件上传) SysQuartzScheduleTaskStatementOfAccountJd - AccountingStatus"})
@Slf4j
@RestController
@RequestMapping("/sysQuartzScheduleTaskStatementOfAccountJd/state/accountingStatus")
public class SysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusController extends BaseController {

	@Autowired
	private SysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService                      sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService;
	@Autowired
	private SysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusDubboConsumerService sysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusDubboConsumerService;

	@ApiOperation(value = "SysQuartzScheduleTaskStatementOfAccountJd - AccountingStatus - 操作 取消对账单任务 状态改变为 已取消_-1")
	@RequestMapping(value = "/opCancel/{id}", method = RequestMethod.POST)
	@ResponseBody
	public R<?> opCancel(@PathVariable("id") Long id) {

		R<?> result = new R<>();

		try {
			SysQuartzScheduleTaskStatementOfAccountJd entity = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.findById(id);

			if (ObjectUtil.isNull(entity)) {
				return ResponseResultOuterUtils.createErrorResult("500", "异常:未找到相对映的京东对账单任务");
			} else if (entity.getAccountingStatus().equals(CommonAuditStatusEnum.待确认.getKey())) {
				return ResponseResultOuterUtils.createErrorResult("500", "异常:此京东对账单任务状态非 待确认'，不能取消");
			} else {
				sysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusDubboConsumerService.opCancel(getToken(), entity);
				result.setSuccess(true);
			}


		} catch (Exception e) {

			result.setSuccess(false);
			result.setMessage("系统错误，请联系管理员！");
			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}
		return result;
	}

	@ApiOperation(value = "SysQuartzScheduleTaskStatementOfAccountJd - AccountingStatus - 操作 创建或重置 状态改变为 待确认_0")
	@RequestMapping(value = "/opInitOrReset/{id}", method = RequestMethod.POST)
	@ResponseBody
	public R<?> opInitOrReset(@PathVariable("id") Long id) {

		R<?> result = new R<>();

		try {
//			SysQuartzScheduleTaskStatementOfAccountJd entity = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.findById(id);
//			sysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusDubboConsumerService.opInitOrReset(getToken(), entity);
//			result.setSuccess(true);

			return ResponseResultOuterUtils.createErrorResult("500", "异常:不能在此初始化状态或重置状态");

		} catch (Exception e) {

			result.setSuccess(false);
			result.setMessage("系统错误，请联系管理员！");
			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}
		return result;
	}

	/**
	 * http://192.168.6.223:8089/web/sysQuartzScheduleTaskStatementOfAccountJd/state/accountingStatus/opConfirm/
	 * <p>
	 * 测试
	 * batchId 1450042895204749312
	 * id 778
	 *
	 * @param id
	 * @return
	 */
	@ApiOperation(value = "SysQuartzScheduleTaskStatementOfAccountJd - AccountingStatus - 操作 确认对账单任务 状态改变为 已确认_1")
	@RequestMapping(value = "/opConfirm/{id}", method = RequestMethod.POST)
	@ResponseBody
	public R<?> opConfirm(@PathVariable("id") Long id) {

		R<?> result = new R<>();

		try {
			SysQuartzScheduleTaskStatementOfAccountJd entity = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.findById(id);

			log.info("{}", JsonUtils.toPrettyJson(entity));

			if (ObjectUtil.isNull(entity)) {
				return ResponseResultOuterUtils.createErrorResult("500", "异常:未找到相对映的京东对账单任务");
			} else if (!entity.getAccountingStatus().equals(CommonAuditStatusEnum.待确认.getKey())) {
				return ResponseResultOuterUtils.createErrorResult("500", "异常:此京东对账单任务状态非'待确认'，不能确认");
			} else {


				// TODO 执行前要先看一下这个有没有异常的 如果有就不能运行 直接返回
				sysQuartzScheduleTaskStatementOfAccountJdFieldAccountingStatusDubboConsumerService.opConfirm(getToken(), entity);
				result.setSuccess(true);
			}

		} catch (Exception e) {

			result.setSuccess(false);
			result.setMessage("系统错误，请联系管理员！");
			log.error(ExceptionUtils.getMessage(e));
			e.printStackTrace();
		}
		return result;
	}


}
