package cn.superdesk.uniorder.dubbo.consumer.accounting;

import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.security.JwtUtil;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dao.entity.accounting.score.customer.AccountingScoreCustomer;
import cn.superdesk.uniorder.dto.param.accounting.score.customer.AccountingScoreCustomerCreateParam;
import cn.superdesk.uniorder.dto.vo.accounting.score.customer.AccountingScoreCustomerDetailVo;
import cn.superdesk.uniorder.dto.vo.accounting.score.customer.AccountingScoreCustomerHeadVo;
import cn.superdesk.uniorder.dto.vo.accounting.score.customer.AccountingScoreCustomerVo;
import cn.superdesk.uniorder.dto.vo.score.ScoreOrderReconciliationNotYetVo;
import cn.superdesk.uniorder.dubbo.consumer._permission.PermissionDubboCustomerService;
import cn.superdesk.uniorder.dubbo.facade.accounting.DubboAccountingScoreCustomerService;
import com.alibaba.dubbo.config.annotation.Reference;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @Description: 积分对账(结算)
 * @Author: linjinyu
 * @CreateDate: 6/28/20 1:06 PM
 * @UpdateUser: linjinyu
 * @UpdateDate: 6/28/20 1:06 PM
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Component
public class AccountingScoreCustomerDubboConsumerService {


	@Reference(version = "1.0.0", timeout = 30000, retries = -1)
	private DubboAccountingScoreCustomerService dubboAccountingScoreCustomerService;

	@Resource
	private PermissionDubboCustomerService permissionDubboCustomerService;

	/****************************************** 积分对账(结算) 对账单 start ******************************************/


	public List<Long> getAccountingXCreditIds(Long id) {
		return dubboAccountingScoreCustomerService.getAccountingXCreditIds(id);
	}

	//
//	public ScoreAccountingCustomerHeadVo getAccountingListHeadSum(Map<String, Object> params) {
//
//		return dubboAccountingScoreCustomerService.getAccountingListHeadSum(params);
//
//	}
	public R<List<AccountingScoreCustomerVo>> getAccountingList(String token, Map<String, Object> params, int pageNum, int pageSize) {

		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);

		// 打开 20210721 mark by linjinyu
		Boolean isPass = permissionDubboCustomerService.addOnPerm(token, params);
		if (isPass.equals(false)) {
			return null;
		}

		return dubboAccountingScoreCustomerService.getAccountingList(params, pageNum, pageSize);
	}

	public List<AccountingScoreCustomerVo> getAccountingList(String token, Map<String, Object> params) {

		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);

		return dubboAccountingScoreCustomerService.getAccountingList(params, loginSysUser);
	}

	public AccountingScoreCustomerHeadVo getAccountingListHeadSum(String token, Map<String, Object> params) {

		// 打开 20210721 mark by linjinyu
		Boolean isPass = permissionDubboCustomerService.addOnPerm(token, params);
		if (isPass.equals(false)) {
			return null;
		}

		return dubboAccountingScoreCustomerService.getAccountingListHeadSum(params);
	}

	/****************************************** 积分对账(结算) 对账单   end ******************************************/

	/****************************************** 积分对账(结算) 对账单 详情 start ******************************************/

	public AccountingScoreCustomerDetailVo commonStatementOfAccountDetailDimCredit(String token, Long accountingOrderId) {

		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);

//		Boolean isPass = permissionDubboCustomerService.addOnPerm(token, params);
//		if (isPass.equals(false)) {
//			return null;
//		}

		return dubboAccountingScoreCustomerService.commonStatementOfAccountDetailDimCredit(accountingOrderId, loginSysUser);
	}

	public String exportExcelStatementOfAccountDetailDimCredit(String token, Long accountingOrderId) {

		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);

//		Boolean isPass = permissionDubboCustomerService.addOnPerm(token, params);
//		if (isPass.equals(false)) {
//			return null;
//		}

		return dubboAccountingScoreCustomerService.exportExcelStatementOfAccountDetailDimCredit(accountingOrderId, loginSysUser);
	}

	/****************************************** 积分对账(结算) 对账单 详情   end ******************************************/


	/**
	 * 积分订单 的 客户结算单
	 * 与 createProduceNew 对映
	 *
	 * @param token
	 * @param scoreOrderReconciliationNotYetVoList
	 * @param scoreAccountingCustomerCreateParam
	 * @return
	 * @throws NumberFormatException
	 * @throws Exception
	 */
	public AccountingScoreCustomer createScoreAccountingCustomer(String token, List<ScoreOrderReconciliationNotYetVo> scoreOrderReconciliationNotYetVoList, AccountingScoreCustomerCreateParam scoreAccountingCustomerCreateParam) throws NumberFormatException, Exception {

		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);

		return dubboAccountingScoreCustomerService.createScoreAccountingCustomer(
				loginSysUser.getSysUserId(),
				scoreOrderReconciliationNotYetVoList,
				scoreAccountingCustomerCreateParam);
	}

	/**
	 * *********************************************** 积分结算单 end  ******************************************************
	 **/


	/**
	 * *********************************************** 对账 操作 start ***********************************************
	 * **/
	/**
	 * 重新对账
	 *
	 * @param id
	 */
	public void opCancelAccounting(String token, Long id) {
		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);
		dubboAccountingScoreCustomerService.opCancelAccounting(id, loginSysUser);
	}

	/**
	 * 拒绝对账
	 *
	 * @param token
	 * @param id
	 */
	public void opRefuseAccounting(String token, Long id) {

		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);

		dubboAccountingScoreCustomerService.opRefuseAccounting(id, loginSysUser);

	}

	/**
	 * 确认对账
	 *
	 * @param token
	 * @param id
	 */
	public void opConfirmAccounting(String token, Long id) {

		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);

		dubboAccountingScoreCustomerService.opConfirmAccounting(id, loginSysUser);
	}

	/**
	 * 删除 对账单 及 对账单X积分流水 上位去处理 是否能删除吧 这里不管 物理删除
	 *
	 * @param id
	 */
	public void opDeleteAccounting(Long id) {
		dubboAccountingScoreCustomerService.opDeleteAccounting(id);
	}

	/**
	 * *********************************************** 对账 操作   end ***********************************************
	 **/


	/**
	 * 删除 对账单 物理删除 不删除 对账单X积分流水
	 *
	 * @param id
	 */
	public void deleteObj(Long id) {

		dubboAccountingScoreCustomerService.deleteObj(id);
	}

	public void updateObj(String token, AccountingScoreCustomer statementOfAccount4Update) {

		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);
		dubboAccountingScoreCustomerService.updateObj(statementOfAccount4Update, loginSysUser);
	}

	public void updateFieldRemark(String token, AccountingScoreCustomer statementOfAccount4Update) {

		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);
		dubboAccountingScoreCustomerService.updateFieldRemark(statementOfAccount4Update, loginSysUser);
	}

	public AccountingScoreCustomer findById(Long id) {
		return dubboAccountingScoreCustomerService.findById(id);
	}

	public void addObj(AccountingScoreCustomer scoreAccountingCustomer) {

	}

	public List<AccountingScoreCustomer> findAll(Map<String, Object> queryMap) {

		return null;
	}

}
