package cn.superdesk.uniorder.dubbo.consumer.invoice.supplier;

import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.security.JwtUtil;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dao.entity.invoice.supplier.UnifiedSupplierInv;
import cn.superdesk.uniorder.dto.vo.invoice.supplier.UnifiedSupplierInvDetail;
import cn.superdesk.uniorder.dto.vo.invoice.supplier.UnifiedSupplierInvoiceAddParams;
import cn.superdesk.uniorder.dto.vo.unified.InvoiceApplySupplierSelectGoodsVo;
import cn.superdesk.uniorder.dto.vo.unified.UnifiedInvHE;
import cn.superdesk.uniorder.dubbo.consumer._permission.PermissionDubboCustomerService;
import cn.superdesk.uniorder.dubbo.consumer.common.RedisDubboConsumerService;
import cn.superdesk.uniorder.dubbo.facade.invoice.supplier.DubboUnifiedSupplierInvService;
import com.alibaba.dubbo.config.annotation.Reference;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;


@Slf4j
@Component
public class UnifiedSupplierInvDubboConsumerService {

	@Reference(version = "1.0.0", timeout = 30000, retries = -1)
	private DubboUnifiedSupplierInvService dubboUnifiedSupplierInvService;

	@Resource
	private PermissionDubboCustomerService permissionDubboCustomerService;
	@Resource
	private RedisDubboConsumerService      redisDubboConsumerService;

	public R<List<UnifiedSupplierInv>> getInvoiceApplyList(String token, Map<String, Object> params,
	                                                       int pageNum,
	                                                       int pageSize
	) {

		// 20191205 add 数据权限
		Boolean isPass = permissionDubboCustomerService.addOnPerm(token, params);
		if (isPass.equals(false)) {
			R<List<UnifiedSupplierInv>> responseResult = new R<>();
			responseResult.setSuccess(false);
			responseResult.setMessage("您没有企业数据权限！");
			return responseResult;
		}

		return dubboUnifiedSupplierInvService.getInvoiceApplyList(params, pageNum, pageSize);
	}

	public UnifiedInvHE getInvoiceApplyListHeadSum(String token, Map<String, Object> params) {

		// 20191205 add 数据权限
		Boolean isPass = permissionDubboCustomerService.addOnPerm(token, params);
		if (isPass.equals(false)) {
			return null;
		}

		return dubboUnifiedSupplierInvService.getInvoiceApplyListHeadSum(params);
	}

	public UnifiedSupplierInvDetail getInvoiceDetail(Long id) {

		return dubboUnifiedSupplierInvService.getInvoiceDetail(id);
	}

	public List<InvoiceApplySupplierSelectGoodsVo> getStatementOfAccountGoodsListByAccountingOrderSn(String accountingOrderSn) {

		return dubboUnifiedSupplierInvService.getStatementOfAccountGoodsListByAccountingOrderSn(accountingOrderSn);
	}

	public List<InvoiceApplySupplierSelectGoodsVo> getStatementOfAccountGoodsListBySysUnifiedOrderGoodsIds(String accountingOrderSn, List<Long> processSysUnifiedOrderGoodsIds) {
		return dubboUnifiedSupplierInvService.getStatementOfAccountGoodsListBySysUnifiedOrderGoodsIds(accountingOrderSn, processSysUnifiedOrderGoodsIds);
	}

	public List<InvoiceApplySupplierSelectGoodsVo> invoiceApplySelectFocusUnified(String accountingOrderSn) {

		return dubboUnifiedSupplierInvService.invoiceApplySelectFocusUnified(accountingOrderSn);
	}

	public UnifiedSupplierInv opInvoiceApply(String token, UnifiedSupplierInvoiceAddParams invoiceApplyParams) {

//		Map<String, Object> tokenVo = JwtUtil.getTokenVo(token);
//
//		return dubboUnifiedSupplierInvService.opInvoiceApply(invoiceApplyParams,
//				tokenVo.get("userId").toString(),
//				tokenVo.get("userName").toString(),
//				tokenVo.get("roleType").toString()
//		);

		// mark by linjinyu at 20210312
		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);
		return dubboUnifiedSupplierInvService.opInvoiceApply(invoiceApplyParams, loginSysUser);
	}

	public UnifiedSupplierInv updateObj(String token, UnifiedSupplierInv update) {

		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);

		update.setMakeInvoiceUserId(loginSysUser.getSysUserId());
		update.setMakeInvoiceUserName(loginSysUser.getSysUserName());

		return dubboUnifiedSupplierInvService.updateObj(update);
	}

	public UnifiedSupplierInv opConfirmPay(String token, UnifiedSupplierInv update) {

		return dubboUnifiedSupplierInvService.updateObj(update);
	}

	public void opAgain(Long id) {

		dubboUnifiedSupplierInvService.opAgain(id);
	}

	public void opDelete(Long id) {

		dubboUnifiedSupplierInvService.opDelete(id);
	}


	public void patchInvoiceApplyFieldStatistics() {
		dubboUnifiedSupplierInvService.patchInvoiceApplyFieldStatistics();
	}
}
