package cn.superdesk.uniorder.mq.consumer.superdeskmall;

import cn.superdesk.uniorder.common.exception.utils.ExceptionUtils;
import cn.superdesk.uniorder.dubbo.consumer.base.UnifiedOrderDubboConsumerService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.stereotype.Component;

import java.util.Optional;


/**
 * @Description: 商城订单数据
 * @Author: linjinyu
 * @CreateDate: 2021/5/7 上午10:17
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021/5/7 上午10:17
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Component
public class ReceiveSuperdeskMallConsumerHandler {

	@Autowired
	private UnifiedOrderDubboConsumerService unifiedOrderDubboConsumerService;
//	@Autowired
//	private OrderDubboConsumerService            orderDubboConsumerService;
//	@Autowired
//	private GoodsDubboConsumerService            goodsDubboConsumerService;
//	@Autowired
//	private SysOderUserDubboConsumerService      sysOderUserDubboConsumerService;
//	@Autowired
//	private UnifiedEncourageDubboConsumerService unifiedEncourageDubboConsumerService;
//	@Autowired
//	private UnifiedSupplierDubboConsumerService  unifiedSupplierDubboConsumerService;


	/**
	 * 老的订单数据
	 * <p>
	 * 这个大概就是
	 * <p>
	 * 燕璐
	 * <p>
	 * 代客下单系统：https://ui-a.superdesk.cn/login?redirect=%2Fdashboard
	 * <p>
	 * 账号：yanlu@avic-s.com  密码：123456
	 * <p>
	 * SUBMIT20200206184939749204
	 * <p>
	 * http://order.avic-s.com/#/homePage
	 * yanl   密码：123456
	 * <p>
	 * PHP
	 * $topic = 'orderpro';            // 配置在env中
	 * $url   = '39.108.55.24:9092';   // 配置在env中 统一订单中心
	 * <p>
	 * topic orderpro
	 *
	 * @param record
	 */
	@KafkaListener(topics = "${superdesk.mall.kafka.topic.order.main}")
	public void listenUnifiedOrder(ConsumerRecord<String, String> record) {

		Optional<?> kafkaMessage = Optional.ofNullable(record.value());

		if (kafkaMessage.isPresent()) {

			Object message = kafkaMessage.get();

//			log.info("########## #msg #" + "\n" + record);
//			log.info("########## #key #" + "\n" + record.key());
//			log.info("########## #body#" + "\n" + message);

			if (
//					!StringUtils.isEmpty(String.valueOf(record.key())) &&
					!StringUtils.isEmpty(String.valueOf(message))) {

				try {
					unifiedOrderDubboConsumerService.kafkaUnifiedOrderReceive(String.valueOf(message));

				} catch (Exception e) {
					log.error("kafka 侨城汇 分润 error\n{}" + ExceptionUtils.getMessage(e));
					e.printStackTrace();
				}
			}

		}
	}

	/**
	 * 积分订单
	 *
	 * @param record
	 */
	@KafkaListener(topics = "${superdesk.mall.kafka.topic.order.score}")
	public void listenScoreOrder(ConsumerRecord<String, String> record) {

		Optional<?> kafkaMessage = Optional.ofNullable(record.value());

		if (kafkaMessage.isPresent()) {

			Object message = kafkaMessage.get();

//			log.info("########## #msg #" + "\n" + record);
//			log.info("########## #key #" + "\n" + record.key());
//			log.info("########## #body#" + "\n" + message);

			if (!StringUtils.isEmpty(String.valueOf(record.key()))
					&& !StringUtils.isEmpty(String.valueOf(message))
			) {

				try {
					unifiedOrderDubboConsumerService.kafkaScoreOrderReceive(String.valueOf(message));

				} catch (Exception e) {
					log.error("kafka 侨城汇 分润 error\n{}" + ExceptionUtils.getMessage(e));
					e.printStackTrace();
				}
			}
		}
	}


	/**
	 * 一般订单
	 * 退货数据 add by qzy
	 *
	 * @param record
	 */
	@KafkaListener(topics = "${superdesk.mall.kafka.topic.order.refund}")
	public void listenUnifiedOrderRefund(ConsumerRecord<String, String> record) {

		Optional<?> kafkaMessage = Optional.ofNullable(record.value());

		if (kafkaMessage.isPresent()) {

			Object message = kafkaMessage.get();

//			log.info("########## #msg #" + "\n" + record);
//			log.info("########## #key #" + "\n" + record.key());
//			log.info("########## #body#" + "\n" + message);

			if (!StringUtils.isEmpty(String.valueOf(record.key()))
					&& !StringUtils.isEmpty(String.valueOf(message))
			) {
				try {
					unifiedOrderDubboConsumerService.kafkaUnifiedOrderRefundReceive(String.valueOf(message));
				} catch (Exception e) {
					log.error("kafka 超级前台 退单 error\n{}" + ExceptionUtils.getMessage(e));
					e.printStackTrace();
				}
			}
		}
	}


	/**
	 * 积分订单
	 * 退货数据
	 *
	 * @param record
	 */
	@KafkaListener(topics = "${superdesk.mall.kafka.topic.order.score.refund}")
	public void listenScoreOrderRefund(ConsumerRecord<String, String> record) {

		Optional<?> kafkaMessage = Optional.ofNullable(record.value());

		if (kafkaMessage.isPresent()) {

			Object message = kafkaMessage.get();

//			log.info("########## #msg #" + "\n" + record);
//			log.info("########## #key #" + "\n" + record.key());
//			log.info("########## #body#" + "\n" + message);

			if (!StringUtils.isEmpty(String.valueOf(record.key()))
					&& !StringUtils.isEmpty(String.valueOf(message))
			) {

				try {
					unifiedOrderDubboConsumerService.kafkaScoreOrderRefundReceive(String.valueOf(message));

				} catch (Exception e) {
					log.error("kafka 侨城汇 退单 error\n{}" + ExceptionUtils.getMessage(e));
					e.printStackTrace();
				}
			}
		}
	}
}


